/*!  \file kfbxcharacterpose.h
 */

#ifndef _KFbxCharacterPose_h
#define _KFbxCharacterPose_h

/**************************************************************************************

 Copyright  2002 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxplugins/kfbxobject.h>

#include <klib/kstring.h>

#include <kfbxmath/kfbxxmatrix.h>

#include <kfbxplugins/kfbxnode.h>

#include <kfbxcharacter/kfbxcharacter.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxScene;
class KFbxSdkManager;


/**	\class KFbxCharacterPose
  * \brief A character pose is a character and an associated hierarchy of nodes.
  * 
  * Only the default position of the nodes is considered, the animation data is ignored.
  * \nosubgrouping
  *	To access the content of a character pose, functions KFbxCharacterPose::GetOffset(),
  * KFbxCharacterPose::GetLocalPosition() and KFbxCharacterPose::GetGlobalPosition() 
  * are sufficient. Their source code is provided inline as examples on how to access 
  * the character pose data. 
  *
  * \par 
  * To create a character pose, one must create a hierarchy of nodes under the root 
  * node provided by function KFbxCharacterPose::GetRootNode(). Then, this hierarchy
  * of nodes must be fed into the character returned by function KFbxCharacterPose::GetCharacter().
  * Offsets are set in the character links. Local positions are set using functions
  * KFbxNode::SetDefaultT(), KFbxNode::SetDefaultR() and KFbxNode::SetDefaultS(). 
  * Here are steps on how to set local positions from global positions:
  *
  * 1) Let lCharacterPose be a valid pointer to a KFbxCharacterPose;
  *
  * 2) Call lCharacterPose->GetRootNode()->SetLocalStateId(0, true);
  *
  * 3) Call lCharacterPose->GetRootNode()->SetGlobalStateId(1, true);
  *
  * 4) Call function KFbxNode::SetGlobalState() for all nodes found in the hierarchy under lCharacterPose->GetRootNode();
  *
  * 5) Call lCharacterPose->GetRootNode()->ComputeLocalState(1, true);
  *
  * 6) Call lCharacterPose->GetRootNode()->SetCurrentTakeFromLocalState(KTIME_ZERO, true).
  *
  */
class KFBX_DLL KFbxCharacterPose : public KFbxObject
{

public:

	//! Reset to an empty character pose.
	void Reset();

	/** Get the root node.
	  *	\return Pointer to the root node.
	  */
    KFbxNode* GetRootNode();

	/** Get the character.
	  * \return Pointer to the character.
	  */
	KFbxCharacter* GetCharacter();

	//! Get offset matrix for a given character node.
	bool GetOffset(ECharacterNodeId pCharacterNodeId, KFbxXMatrix& pOffset)
	{
		KFbxCharacterLink lCharacterLink;

		if (!GetCharacter()) return false;

		if (!GetCharacter()->GetCharacterLink(pCharacterNodeId, &lCharacterLink)) return false;

		pOffset.SetTRS(lCharacterLink.mOffsetT, lCharacterLink.mOffsetR, lCharacterLink.mOffsetS);

		return true;
	}

	//! Get local position for a given character node.
	bool GetLocalPosition(ECharacterNodeId pCharacterNodeId, KFbxXMatrix& pLocalPosition)
	{
		KFbxCharacterLink lCharacterLink;

		if (!GetCharacter()) return false;

		if (!GetCharacter()->GetCharacterLink(pCharacterNodeId, &lCharacterLink)) return false;

		pLocalPosition = lCharacterLink.mNode->GetLocalFromDefaultTake();

		return true;
	}

	//! Get global position for a given character node.
	bool GetGlobalPosition(ECharacterNodeId pCharacterNodeId, KFbxXMatrix& pGlobalPosition)
	{
		KFbxCharacterLink lCharacterLink;

		if (!GetCharacter()) return false;

		if (!GetCharacter()->GetCharacterLink(pCharacterNodeId, &lCharacterLink)) return false;

		pGlobalPosition = lCharacterLink.mNode->GetGlobalFromDefaultTake();

		return true;
	}

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS	

public:

	virtual KFbxObject::ENameSpace GetNameSpace() const; 
	KFbxCharacterPose& operator=(KFbxCharacterPose& pCharacterPose); 

private:

	KFbxCharacterPose(KFbxSdkManager& pManager, char* pName);
	~KFbxCharacterPose();

	void CopyNode(KFbxNode* pSrcNode, KFbxNode* pDstNode);
	void CopyCharacter(KFbxCharacter* pSrcCharacter, KFbxCharacter* pDstCharacter, KFbxNode* pDstRootNode);

	KFbxScene* mScene;

	friend class KFbxScene;
	friend class KFbxReaderFbx;
	friend class KFbxWriterFbx;
	friend class KFbxWriterFbx6;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

} // namespace
#endif // #ifndef _KFbxCharacterPose_h


