/*!  \file kfbxfilterset.h
 */


#ifndef _KFbxFilterSet_h
#define _KFbxFilterSet_h

/**************************************************************************************

 Copyright  2002 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

class KFilterSetBase;

#include <kfbxplugins/kfbxnamespacedef.h>

namespace FBXSDK_NAMESPACE {

/** \class KFbxFilterSet
  *
  * This class contains all the parameters of one character.
  *
  * Use this class to import or export all parameters of one character.
  * You can get access to the character's filter set with the KFbxCharacter
  * member function KFbxFilterSet& KFbxCharacter::GetFilterSet().
  *
  */
class KFBX_DLL KFbxFilterSet
{

public:

	/** \enum EParameterModeType    Parameter mode types.
      * - \e eNoMode                Parameter has no mode and is always active.
      * - \e eOffOn                 Parameter can be on or off.
      * - \e eOffAutoUser           Parameter can be off, auto or user.
      * - \e eAutoUser              Parameter can be auto or user.
      */
	typedef enum 
	{
		eNoMode,		
		eOffOn,			
		eOffAutoUser,	
		eAutoUser		
	} EParameterModeType;

	/** \enum EParameterValueType   Parameter value types.
	  * - \e eScalar                Value contains a single element.
	  * - \e eVector                Value contains a vector of 3 elements.
	  */
	typedef enum 
	{ 
		eScalar,
		eVector
	} EParameterValueType;

	/** \enum EParameterUnit        Parameter units.
      * - \e eNoUnit                Parameter has no unit.
	  * - \e ePercent               
      * - \e eSecond                
	  * - \e eCentimeter    
      * - \e eDegree
      */
	typedef enum 
	{ 
		eNoUnit,
		ePercent,
		eSecond,
		eCentimeter,
		eDegree
	} EParameterUnit;

    /** Reset all parameter to default values.
      */
	void    Reset();

	/** Set parameter mode.
      * \param pParameterIndex      Parameter index.
      * \param pParameterMode       Parameter mode.
	  *                                 - If parameter mode type is eOffOn: 0 is off, 1 is on.
	  *                                 - If parameter mode type is eOffAutoUser: 0 is off, 1 is auto, 2 is user.
	  *                                 - If parameter mode type is eAutoUser: 0 is auto, 1 is user.
	  * \remarks This function has no effect if the given mode is not allowed.
	  */
	void    SetParameterMode(int pParameterIndex, int pParameterMode);
	
	/** Get parameter mode.
      * \param pParameterIndex      Parameter index.
      * \return                     Parameter mode.
	  *                                 - If parameter mode type is eOffOn: 0 is off, 1 is on.
	  *                                 - If parameter mode type is eOffAutoUser: 0 is off, 1 is auto, 2 is user.
	  *                                 - If parameter mode type is eAutoUser: 0 is auto, 1 is user.
	  */
	int    GetParameterMode(int pParameterIndex);
	
	/** Set parameter value.
	  * \param pParameterIndex      Parameter index.
	  * \param pValue               Parameter value.
	  * \param pIndex               Value index. 
      *                                 - If value type is eScalar, index can only equal 0.
      *                                 - If value type is eVector, index can equal 0, 1 or 2.
	  */ 
	void    SetParameterValue(int pParameterIndex, double pValue, int pIndex = 0);	

	/** Get parameter value.
	  * \param pParameterIndex      Parameter index.
	  * \param pIndex               Value index.
      *                                 - If value type is eScalar, index can only equal 0.
      *                                 - If value type is eVector, index can equal 0, 1 or 2.
	  * \return                     Parameter value.
	  */ 
	double GetParameterValue(int pParameterIndex, int pIndex = 0);	

	
	/** Get number of parameters.
      * \return                     Total parameter count.
      */
	static int                 GetParameterCount();

	/** Get parameter name.
      * \param  pParameterIndex     Parameter index.
      * \return                     Parameter name.
      */
	static char*             GetParameterName(int pParameterIndex);
	
	/** Get parameter index by name.
      * \param pParameterName       Parameter name.
      * \return                     Parameter index.
	  * \remarks If no parameter with given name exists, -1 is returned.
	  */
	static int                 GetParameterIndex(char* pParameterName);

	/** Get parameter mode type.
      * \param pParameterIndex      Parameter index.
      * \return                     Possible parameter mode type, see enum EParameterModeType for possible value.
      */
	static EParameterModeType   GetParameterModeType(int pParameterIndex);

	/** Get parameter default mode.
      * \param pParameterIndex      Parameter index.
      * \return                     Default parameter mode.
	  *                                 - If parameter mode type is eOffOn: 0 is off, 1 is on.
	  *                                 - If parameter mode type is eOffAutoUser: 0 is off, 1 is auto, 2 is user.
	  *                                 - If parameter mode type is eAutoUser: 0 is auto, 1 is user.
	  */
	static int                 GetParameterDefaultMode(int pParameterIndex);

	/** Get parameter value type.
      * \param pParameterIndex      Parameter index.
      * \return                     Possible parameter value type, see enum EParameterValueType for possible value.
      */
	static EParameterValueType  GetParameterValueType(int pParameterIndex);

	/** Get parameter unit.
      * \param pParameterIndex      Parameter index.
      * \return                     Possible parameter unit type, see enum EParameterUnit for possible value.
      */
	static EParameterUnit       GetParameterUnit(int pParameterIndex);

	/** Get parameter default value.
      * \param pParameterIndex      Parameter index.
      * \return                     Default parameter value.
      */
	static double              GetParameterDefaultValue(int pParameterIndex);

    /** Get parameter minimum value.
      * \param pParameterIndex      Parameter index.
	  * \return                     Parameter minimum value.
      */
	static double              GetParameterMinValue(int pParameterIndex);

	/** Get parameter maximum value.
      * \param pParameterIndex      Parameter index.
	  * \return                     Parameter maximum value.
      */
	static double              GetParameterMaxValue(int pParameterIndex);    

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS	

private:

	KFbxFilterSet();
	~KFbxFilterSet();

	KFilterSetBase* mFilterSetBase;

	friend class KFbxCharacter;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

} // namespace
#endif // #ifndef _KFbxFilterSet_h


