#ifndef _KFbxCamera_h
#define _KFbxCamera_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxplugins/kfbxnodeattribute.h>
#include <kfbxplugins/kfbxcolor.h>

#include <kfbxmath/kfbxvector4.h>

#include <klib/kstring.h>

#ifdef KARCH_DEV_MACOSX_CFM
	#include <CFURL.h>
	#include <Files.h>
#endif

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxTexture;
class KFbxSdkManager;
class KFbxCameraProperties;

/**	This node attribute contains the properties of a camera.
  * \nosubgrouping
  * A camera can be set to automatically point at and follow 
  * another node in the hierarchy. To do so, the focus source
  * must be set to ECameraFocusSource::eCAMERA_INTEREST and the 
  * followed node associated with function KFbxNode::SetTarget().
  */
class KFBX_DLL KFbxCamera : public KFbxNodeAttribute
{
public:

	//! Return the type of node attribute which is EAttributeType::eCAMERA.
	virtual EAttributeType GetAttributeType();

	//! Reset the camera to default values.
	void Reset();

	/**
	  * \name Camera Position and Orientation
	  */
	//@{

    /** Set the default XYZ camera position.
	  * \param pPosition X, Y and Z values of the camera position.
	  * \remark If this attribute is not attached to a node yet, this
	  * method does nothing.
	  */
    void SetPosition(const KFbxVector4& pPosition);
	
    /** Get the default position of the camera.
	  * \return The X, Y and Z values of the default position of the camera.
	  * \remark If this attribute is not attached to a node yet, this 
	  * method will return 0,0,0.
	  */
    KFbxVector4 GetPosition();

    /** Set the camera's Up vector.
	  * \param pVector The X, Y and Z values for the Up vector.
	  */
    void SetUpVector(const KFbxVector4& pVector);

    /** Get the current Up vector.
	  * \return The X, Y and Z values of the currently set Up vector.
	  */
    KFbxVector4 GetUpVector();

    /** Set the default point the camera is looking at.
	  * \param pPosition X, Y and Z values of the camera interest point.
	  * \remarks During the computations of the camera position and orientation,
	  * the default camera interest position is overridden by the position of a 
	  * valid target in the parent node.
	  */
    void SetDefaultCameraInterestPosition(const KFbxVector4& pPosition);

    /** Return the camera interest to default coordinates.
	  * \return The X,Y and Z values of the default interest point.
	  * \remarks During the computations of the camera position and orientation,
	  * the default camera interest position is overridden by the position of a 
	  * valid target in the parent node.
	  */
    KFbxVector4 GetDefaultCameraInterestPosition();

    /** Set the camera roll angle.
	  * \param pRoll The roll angle in degrees.
	  */
    void SetRoll(double pRoll);

    /** Get the camera roll angle.
	  * \return The roll angle in degrees.
	  */
    double GetRoll();

    /** Set the camera turntable angle.
	  * \param pTurnTable The turntable angle in degrees.
	  */
    void SetTurnTable(double pTurnTable);

    /** Get the camera turntable angle.
	  * \return The turntable angle in degrees.
	  */
    double GetTurnTable();

    /**	Camera projection types.
	  * \remarks By default, the camera projection type is set to ePERSPECTIVE.
	  * If the camera projection type is set to eORTHOGONAL, the following options 
	  * are not relevant:
	  *		- aperture format
	  *		- aperture mode
	  *     - aperture width and height
	  *		- angle of view/focal length
	  *		- squeeze ratio
	  */

	/** \enum ECameraProjectionType Camera projection types.
	  * - \e ePERSPECTIVE
	  * - \e eORTHOGONAL
	  */
    typedef enum 
    { 
	    ePERSPECTIVE, 
	    eORTHOGONAL 
    } ECameraProjectionType;

	/** Set the camera projection type.
	  * \param pProjectionType The camera projection identifier.
	  */
    void SetProjectionType(ECameraProjectionType pProjectionType);

	/** Get the camera projection type.
	  * \return The camera's current projection identifier.
	  */
    ECameraProjectionType GetProjectionType();

	//@}

	/**
	  * \name Viewing Area Controls
	  */
	//@{

	/** \enum ECameraFormat Camera formats.
	  * - \e eCUSTOM_FORMAT
	  * - \e eD1_NTSC
	  * - \e eNTSC
	  * - \e ePAL
	  * - \e eD1_PAL
	  * - \e eHD
	  * - \e e640x480
	  * - \e e320x200
	  * - \e e320x240
	  * - \e e128x128
	  * - \e eFULL_SCREEN
	  */
	typedef enum 
	{
		eCUSTOM_FORMAT,
		eD1_NTSC,
		eNTSC,
		ePAL,
		eD1_PAL,
		eHD,
		e640x480,
		e320x200,
		e320x240,
		e128x128,
		eFULL_SCREEN,
	} ECameraFormat;

	/** Set the camera format.
	  * \param pFormat The camera format identifier.
	  * \remarks Changing the camera format sets the camera aspect 
	  * ratio mode to eFIXED_RESOLUTION and modifies the aspect width 
	  * size, height size and pixel ratio accordingly.
	  */
    void SetFormat(ECameraFormat pFormat);

	/** Get the camera format.
	  * \return The current camera format identifier.
	  */
    ECameraFormat GetFormat();

	/** \enum ECameraAspectRatioMode Camera aspect ratio modes.
	  * - \e eWINDOW_SIZE
	  * - \e eFIXED_RATIO
	  * - \e eFIXED_RESOLUTION
	  * - \e eFIXED_WIDTH
	  * - \e eFIXED_HEIGHT
	  */
    typedef enum 
    {
	    eWINDOW_SIZE,
	    eFIXED_RATIO,
	    eFIXED_RESOLUTION,
	    eFIXED_WIDTH,
	    eFIXED_HEIGHT
    } ECameraAspectRatioMode;

	/** Set the camera aspect.
	  * \param pRatioMode Camera aspect ratio mode.
	  * \param pWidth Camera aspect width, must be a positive value.
	  * \param pHeight Camera aspect height, must be a positive value.
	  * \remarks Changing the camera aspect sets the camera format to eCustom.
	  *		- If the ratio mode is eWINDOW_SIZE, both width and height values aren't relevant.
	  *		- If the ratio mode is eFIXED_RATIO, the height value is set to 1.0 and the width value is relative to the height value.
	  *		- If the ratio mode is eFIXED_RESOLUTION, both width and height values are in pixels.
	  *		- If the ratio mode is eFIXED_WIDTH, the width value is in pixels and the height value is relative to the width value. 
	  *		- If the ratio mode is eFIXED_HEIGHT, the height value is in pixels and the width value is relative to the height value. 
	  */
    void SetAspect(ECameraAspectRatioMode pRatioMode, double pWidth, double pHeight);

	/** Get the camera aspect ratio mode.
	  * \return The current aspect ratio identifier.
	  */
    ECameraAspectRatioMode GetAspectRatioMode();

	/** Get the aspect width.
	  * \return The aspect width value or an undefined value if aspect ratio mode is set to eWINDOW_SIZE.
	  */
	double GetAspectWidth();

	/** Get the aspect height.
	  * \return The aspect height value or an undefined value if aspect ratio mode is set to eWINDOW_SIZE.
	  */
	double GetAspectHeight();

	/** Set the pixel ratio.
	  * \param pRatio The pixel ratio value.
	  * \remarks The value must be a positive number. Comprised between 0.05 and 20.0. Values
	  * outside these limits will be clamped. Changing the pixel ratio sets the camera format to eCUSTOM_FORMAT.
	  */
	void SetPixelRatio(double pRatio);

	/** Get the pixel ratio.
	  * \return The current camera's pixel ratio value.
	  */
	double GetPixelRatio();

	/** Set the near plane distance from camera.
	  * The near plane is the minimum distance to render a scene on the camera display.
	  * \param pDistance The near plane distance value.
	  *	\remarks The near plane value is limited to the range [0.001, 600000.0] and
	  * must be inferior to the far plane value. 
	  */
	void SetNearPlane(double pDistance);

	/** Get the near plane distance from camera.
	  * The near plane is the minimum distance to render a scene on the camera display.
	  * \return The near plane value.
	  */
	double GetNearPlane();

	/** Set the far plane distance from camera.
	  * The far plane is the maximum distance to render a scene on the camera display.
	  * \param pDistance The far plane distance value.
	  *	\remarks The far plane value is limited to the range [0.001, 600000.0] and
	  * must be superior to the near plane value. 
	  */
	void SetFarPlane(double pDistance);
	
	/** Get the far plane distance from camera.
	  * The far plane is the maximum distance to render a scene on the camera display.
	  * \return The far plane value.
	  */
	double GetFarPlane();

	/** Set interactive camera lock flag.
	  * \param pMouseLock If \c true, disable modifications of the view area controls using mouse and keyboard commands.
	  * \remark It is the responsability of the client application to perform the required tasks according to the state
	  * of this flag.
	  */
	void SetMouseLock(bool pMouseLock);

	/** Get the camera lock flag.
	  * \return \c true If modifications of the view area controls using mouse and keyboard commands are disabled.
	  * \remark It is the responsability of the client application to perform the required tasks according to the state
	  * of this flag.
	  */
	bool GetMouseLock();

	//@}

	/**
	  * \name Aperture and Film Controls
	  */
	//@{

    /** \enum ECameraApertureFormat Camera aperture formats.
	  *	- \e eCUSTOM_APERTURE_FORMAT
	  *	- \e e16MM_THEATRICAL
	  * - \e eSUPER_16MM
	  *	- \e e35MM_ACADEMY
	  *	- \e e35MM_TV_PROJECTION
      *	- \e e35MM_FULL_APERTURE
	  *	- \e e35MM_185_PROJECTION
	  *	- \e e35MM_ANAMORPHIC
	  *	- \e e70MM_PROJECTION
	  *	- \e eVISTAVISION
	  *	- \e eDYNAVISION
	  *	- \e eIMAX
	  */
    typedef enum 
    {
		eCUSTOM_APERTURE_FORMAT = 0,
		e16MM_THEATRICAL,
		eSUPER_16MM,
		e35MM_ACADEMY,
		e35MM_TV_PROJECTION,
		e35MM_FULL_APERTURE,
		e35MM_185_PROJECTION,
		e35MM_ANAMORPHIC,
		e70MM_PROJECTION,
		eVISTAVISION,
		eDYNAVISION,
		eIMAX		
	} ECameraApertureFormat;

	/** Set the camera aperture format.
	  * \param pFormat The camera aperture format identifier.
	  * \remarks Changing the aperture format modifies the
	  * aperture width, height and squeeze ratio accordingly.
	  */
    void SetApertureFormat(ECameraApertureFormat pFormat);

	/** Get the camera aperture format.
      * \return The camera's current aperture format identifier.
	  */
    ECameraApertureFormat GetApertureFormat();

    /** \enum ECameraApertureMode Camera aperture modes.
	  * - \e eHORIZONTAL_AND_VERTICAL
	  *	- \e eHORIZONTAL
	  *	- \e eVERTICAL
	  */
    typedef enum 
    { 
	    eHORIZONTAL_AND_VERTICAL,
	    eHORIZONTAL,
	    eVERTICAL
    } ECameraApertureMode;

	/** Set the camera aperture mode.
	  * \param pMode The camera aperture mode identifier.
	  */
    void SetApertureMode(ECameraApertureMode pMode);

	/** Get the camera aperture mode.
	  * \return The camera's current aperture mode identifier.
	  */
    ECameraApertureMode GetApertureMode();

	/** Set the camera aperture width in inches.
	  * \param pWidth The aperture width value.
	  * \remarks Must be a positive value. The minimum accepted value is 0.0001. 
	  * Changing the aperture width sets the camera aperture format to eCUSTOM_FORMAT.
	  */
	void SetApertureWidth(double pWidth);
	
	/** Get the camera aperture width in inches.
	  * \return The camera's current aperture width value in inches.
	  */
	double GetApertureWidth();

	/** Set the camera aperture height in inches.
	  * \param pHeight The aperture height value.
	  * \remarks Must be a positive value. The minimum accepted value is 0.0001. 
	  * Changing the aperture height sets the camera aperture format to eCUSTOM_FORMAT.
	  */
	void SetApertureHeight(double pHeight);

	/** Get the camera aperture height in inches.
	  * \return The camera's current aperture height value in inches.
	  */
	double GetApertureHeight();

	/** Set the squeeze ratio.
	  * \param pRation The sqeeze ratio value.
	  * \remarks Must be a positive value. The minimum accepted value is 0.0001. 
	  * Changing the squeeze ratio sets the camera aperture format to eCUSTOM_FORMAT.
	  */
	void SetSqueezeRatio(double pRatio);

	/** Get the camera squeeze ratio.
	  * \return The camera's current squeeze ratio value.
	  */
	double GetSqueezeRatio();

	/** Set the focal length of the camera in millimeters. 
	  * \param pFocalLength The focal length in mm. 
	  * \remarks Be aware that modifying the focal length will automatically change the angle of view.
      */
    void SetFocalLength(double pFocalLength);

    /** Get the camera focal length in millimeters.
	  * \return The camera's current focal length value.
	  */
    double GetFocalLength();

    /** Set the camera angle of view in degrees. 
	  * \param pAngleOfView The camera angle of view value in degrees. 
	  * \remarks Be aware that modifying the angle of view will automatically change the focal length.
	  * \remarks This value is limited to the range [1.0, 179.0].
      */
    void SetAngleOfView(double pAngleOfView);

    /** Get the camera angle of view in degrees.
	  * \return The camera's current angle of view value in degrees.
	  */
    double GetAngleOfView();

	/** Set the orthographic cameras' zoom. 
	  * \param pOrthoZoom This parameter's default value is 1.0.
	  *	\remarks This parameter is not used if the camera is not orthographic.
      */
    void SetOrthoZoom(double pOrthoZoom);

    /** Get the orthographic camera's zoom.
	  * \return The camera's current zoom.
	  */
    double GetOrthoZoom();

	//@}

	/**
	  * \name Background Properties
	  */
	//@{

    /** Set the associated background image file. 
      * \param pFileName The path of the background image file.   
	  *	\remarks The background image file name must be valid.
      */
	void SetBackgroundFileName(char* pFileName);

#ifdef KARCH_DEV_MACOSX_CFM
    bool SetBackgroundFile(const FSSpec &pMacFileSpec);
    bool SetBackgroundFile(const FSRef &pMacFileRef);
    bool SetBackgroundFile(const CFURLRef &pMacURL);
#endif

	/** Get the background image file name.
	  * \return Pointer to the background filename string or \c NULL if not set.
	  */
	char* GetBackgroundFileName();

#ifdef KARCH_DEV_MACOSX_CFM
	bool GetBackgroundFile(FSSpec &pMacFileSpec) const;
    bool GetBackgroundFile(FSRef &pMacFileRef) const;
    bool GetBackgroundFile(CFURLRef &pMacURL) const;
#endif

    /** Set the media name associated to the background image file. 
      * \param pFileName The media name of the background image file.   
	  *	\remarks The media name is a unique name used to identify the background
	  * image file.
      */
	void SetBackgroundMediaName(char* pFileName);

	/** Get the media name associated to the background image file.
	  * \return Pointer to the media name string or \c NULL if not set.
	  */
	char* GetBackgroundMediaName();

	/** \enum ECameraBackgroundDisplayMode Background display modes.
	  * - \e eDISABLED
	  * - \e eALWAYS
	  * - \e eWHEN_MEDIA
	  */
	typedef enum 
	{
		eDISABLED,
		eALWAYS,  
		eWHEN_MEDIA
	} ECameraBackgroundDisplayMode;

	/** Set the background display mode.
	  * \param pMode The background display mode identifier.
	  */
	void SetBackgroundDisplayMode(ECameraBackgroundDisplayMode pMode);

	/** Get the background display mode.
	  * \return The currently set background display mode identifier.
	  */
	ECameraBackgroundDisplayMode GetBackgroundDisplayMode();

	/** \enum ECameraBackgroundDrawingMode Background drawing modes.
	  * - \e eBACKGROUND                 Image is drawn behind models.
	  * - \e eFOREGROUND                 Image is drawn in front of models based on alpha channel.
	  * - \e eBACKGROUND_AND_FOREGROUND  Image is drawn behind and in front of models depending on alpha channel.
	  */
	typedef enum 
	{
		eBACKGROUND,
		eFOREGROUND,
		eBACKGROUND_AND_FOREGROUND
	} ECameraBackgroundDrawingMode;

	/** Set the background drawing mode.
	  * \param pMode The background drawing mode identifier.
	  */
	void SetBackgroundDrawingMode(ECameraBackgroundDrawingMode pMode);

	/** Get the background drawing mode.
	  * \return The currently set background drawing mode identifier.
	  */
	ECameraBackgroundDrawingMode GetBackgroundDrawingMode();

	/** Set the foreground matte threshold flag.
	  * \param pEnable If \c true enable foreground matte threshold. 
	  * \remark It is the responsability of the client application to perform the required tasks according to the state
	  * of this flag.
	  */
	void SetForegroundMatteThresholdEnable(bool pEnable);
    
	/** Get the foreground matte threshold flag.
	  * \return \c true if foreground matte threshold is enabled.
	  * \remark It is the responsability of the client application to perform the required tasks according to the state
	  * of this flag.
	  */
	bool GetForegroundMatteThresholdEnable();

	/** Set foreground matte threshold. 
	  * \param pThreshold Threshold value on a range from 0.0 to 1.0.
	  * \remarks This option is only relevant if the background drawing mode
	  * is set to eFOREGROUND or eBACKGROUND_AND_FOREGROUND.
	  */
	void SetForegroundMatteThreshold(double pThreshold);
    
	/** Get foreground matte threshold. 
	  * \return Threshold value on a range from 0.0 to 1.0.
	  * \remarks This option is only relevant if the background drawing mode
	  * is set to eFOREGROUND or eBACKGROUND_AND_FOREGROUND.
	  */
	double GetForegroundMatteThreshold();

	/** \enum ECameraBackgroundPlacementOptions Background placement options.
	  * - \e eFIT
	  * - \e eCENTER
	  * - \e eKEEP_RATIO
	  * - \e eCROP
	  */
	typedef enum 
	{
		eFIT = 1<<0,
		eCENTER = 1<<1,
		eKEEP_RATIO = 1<<2,
		eCROP = 1<<3
	} ECameraBackgroundPlacementOptions;
	
	/** Set background placement options.
	  * \param pOptions Bitwise concatenation of one or more background placement options flags.
	  */
	void SetBackgroundPlacementOptions(kUInt pOptions);

	/** Get background placement options.
	  * \return The bitwise concatenation of the currently set background placement options flags.
	  */
	kUInt GetBackgroundPlacementOptions();

	/** ECamerabackgroundDistanceMode Background distance modes.
	  * - \e eRELATIVE_TO_INTEREST
	  * - \e eABSOLUTE_FROM_CAMERA
	  */
	typedef enum 
    {
	    eRELATIVE_TO_INTEREST,
	    eABSOLUTE_FROM_CAMERA
    } ECameraBackgroundDistanceMode;

	/** Set the background distance mode.
      * \param pMode The background distance mode identifier.
	  */
    void SetBackgroundDistanceMode(ECameraBackgroundDistanceMode pMode);

	/** Get the background distance mode.
	  * \return Return the background distance mode identifier.
	  */
    ECameraBackgroundDistanceMode GetBackgroundDistanceMode();

	/** Set the background distance.
	  * \param pDistance Distance of the background plane. This value can be either
	  * relative to the camera interest point or absolute from the camera position.
	  */
	void SetBackgroundDistance(double pDistance);

	/** Get the background distance.
      * \return The distance of the background plane.
	  */
	double GetBackgroundDistance();

	//@}

	/**
	  * \name Camera View Options
	  * \remark It is the responsability of the client application to perform the required tasks according to the state
	  * of these flags.
	  */
	//@{

	/** Change the camera interest visibility flag.
	  * \param pEnable Set to \c true if camera interest must be shown.
	  */
	void SetViewCameraInterest(bool pEnable);

	/** Get current visibility state of the camera interest.
	  * \return \c true if camera interest is shown.
	  */
	bool GetViewCameraInterest();

	/** Change the camera near and far planes visibility flag.
	  * \param pEnable Set to \c true if near and far planes must be shown.
	  */
	void SetViewNearFarPlanes(bool pEnable);

	/** Get current visibility state of the camera near and far planes.
	  * \return \c true if near and far planes are shown.
	  */
	bool GetViewNearFarPlanes();
	
	/** Change the draw floor grid flag.
	  * \param pEnable Set to \c true if grid must be shown.
	  */
	void SetShowGrid(bool pEnable);
	
	/** Get current floor grid draw state.
	  * \return \c true if grid is shown.
	  */
	bool GetShowGrid();

	/** Change the draw system axis flag.
	  * \param pEnable Set to \c true if axis must be shown.
	  */
	void SetShowAxis(bool pEnable);

	/** Get current system axis draw state.
	  * \return \c true if axis is shown.
	  */
	bool GetShowAxis();

	/** Change the show camera name flag.
	  * \param pEnable Set to \c true if camera name must be shown.
	  */
	void SetShowName(bool pEnable);

	/** Get current camera name show state.
	  * \return \c true if camera name is shown.
	  */
	bool GetShowName();

	/** Change the show info on moving flag.
	  * \param pEnable Set to \c true if info on moving must be shown.
	  */
	void SetShowInfoOnMoving(bool pEnable);

	/** Get current info on moving show state.
	  * \return \c true if info on moving is shown.
	  */
	bool GetShowInfoOnMoving();

	/** Change the time code show flag.
	  * \param pEnable Set to \c true if time code must be shown.
	  */
	void SetShowTimeCode(bool pEnable);
	
	/** Get current time code show state.
	  * \return \c true if time code is shown.
	  */
	bool GetShowTimeCode();

	/** Change the display camera safe area flag.
	  * \param pEnable Set to \c true if safe area must be displayed.
	  */
	void SetDisplaySafeArea(bool pEnable);

	/** Get current safe area display state.
	  * \return \c true if safe area is displayed.
	  */
	bool GetDisplaySafeArea();

	/** Change the display of the camera's safe area on render flag.
	  * \param pEnable Set to \c true if safe area must be displayed on render.
	  */
	void SetDisplaySafeAreaOnRender(bool pEnable);

	/** Get current safe area on render display state.
	  * \return  \c true if safe area on render is displayed.
	  */
	bool GetDisplaySafeAreaOnRender();

    /** \enum ECameraSafeAreaStyle Camera safe area display styles.
	  * - \e eROUND
	  * - \e eSQUARE
	  */
    typedef enum 
    { 
	    eROUND = 0, 
	    eSQUARE = 1 
    } ECameraSafeAreaStyle;

	/** Set the safe area style.
	  * \param pStyle Safe area style identifier.
	  */
    void SetSafeAreaStyle(ECameraSafeAreaStyle pStyle);

	/** Get the currently set safe area style.
	  * \return Safe area style identifier.
	  */
    ECameraSafeAreaStyle GetSafeAreaStyle();

	/** Change the show audio flag.
	  * \param pEnable Set to \c true if audio must be shown.
	  */
	void SetShowAudio(bool pEnable);

	/** Get current audio show state.
	  * \return \c true if audio is shown.
	  */
	bool GetShowAudio();

	/**	Set audio color.
	  * \param pColor RGB values for the audio color.
	  */
	void SetAudioColor(const KFbxColor& pColor);

	/**	Get audio color.
	  * \return Currently set audio color.
	  */
	KFbxColor GetAudioColor();

	/** Change the use frame color flag.
	  * \returns \c true if frame color is used.
	  */
	void SetUseFrameColor(bool pEnable);

	/** Get the use frame color state.
	  * \return \c true if frame color is used.
	  */
	bool GetUseFrameColor();

	/**	Set frame color.
	  * \param pColor RGB values for the frame color.
	  */
	void SetFrameColor(const KFbxColor& pColor);

	/** Get frame color.
	  * \return Currently set frame color.
	  */
	KFbxColor GetFrameColor();

	//@}

	/**
	  * \name Render Options
	  * \remark It is the responsability of the client application to perform the required tasks according to the state
	  * of these flags.
	  */
	//@{

    /** \enum ECameraRenderOptionsUsageTime Render options usage time.
	  * - \e eINTERACTIVE
	  * - \e eAT_RENDER
	  */
    typedef enum 
    {
	    eINTERACTIVE,
	    eAT_RENDER
    } ECameraRenderOptionsUsageTime;

	/** Set the render options usage time.
	  * \param pUsageTime The render options usage time identifier.
	  */
    void SetRenderOptionsUsageTime(ECameraRenderOptionsUsageTime pUsageTime);

	/** Get the render options usage time.
	  * \return Rrender options usage time identifier.
	  */
    ECameraRenderOptionsUsageTime GetRenderOptionsUsageTime();

	/** Change the use antialiasing flag.
	  * \param Set to \c true if antialiasing is enabled.
	  */
	void SetUseAntialiasing(bool pEnable);

	/** Get the use antialiasing state.
	  * \return \c true if antialiasing is enabled.
	  */
	bool GetUseAntialiasing();

	/** Set antialiasing intensity.
	  * \param pIntensity Antialiasing intensity value.
	  */
	void SetAntialiasingIntensity(double pIntensity);

	/** Get the antialiasing intensity.
	  * \return Return the currently set antialiasing intensity value. 
	  */
	double GetAntialiasingIntensity();

    /** \enum ECameraAntialiasingMethod Antialiasing methods.
	  * - \e eOVERSAMPLING_ANTIALIASING
	  * - \e eHARDWARE_ANTIALIASING
	  */
    typedef enum 
    {
	    eOVERSAMPLING_ANTIALIASING, 
	    eHARDWARE_ANTIALIASING
    } ECameraAntialiasingMethod;

	/** Set antialiasing method.
	  * \param pMethod The antialiasing method identifier.
	  */
    void SetAntialiasingMethod(ECameraAntialiasingMethod pMethod);

	/** Get antialiasing method.
	  * \return The currently set antialiasing method identifier.
	  */
    ECameraAntialiasingMethod GetAntialiasingMethod();

	/** Set the number of samples used to process oversampling.
	  * \param pNumberOfSamples Number of samples used to process oversampling.
	  * \remarks This option is only relevant if antialiasing 
	  * method is set to eOVERSAMPLING_ANTIALIASING.
	  */
	void SetNumberOfSamples(int pNumberOfSamples);

	/** Get the number of samples used to process oversampling.
	  * \return The currently set number of samples used to process oversampling.
	  * \remarks This option is only relevant if antialiasing 
	  * method is set to eOVERSAMPLING_ANTIALIASING.
	  */
	int GetNumberOfSamples();

    /** \enum ECameraSamplingType Oversampling types.
	  * - \e eUNIFORM
	  * - \e eSTOCHASTIC
	  */
    typedef enum 
    {
	    eUNIFORM,
	    eSTOCHASTIC
    } ECameraSamplingType;

	/** Set sampling type.
	  * \param pType Sampling type identifier.
	  * \remarks This option is only relevant if antialiasing type 
	  * is set to eOVERSAMPLING_ANTIALIASING.
	  */
    void SetSamplingType(ECameraSamplingType pType);

	/** Get sampling type.	
	  * \return The currently set sampling type identifier.
	  * \remarks This option is only relevant if antialiasing type 
	  * is set to eOVERSAMPLING_ANTIALIASING.
	  */
	ECameraSamplingType GetSamplingType();

	/** Change the use accumulation buffer flag.
      * \param pUseAccumulationBuffer Set to \c true to enable use of accumulation buffer.
	  */
	void SetUseAccumulationBuffer(bool pUseAccumulationBuffer);

	/** Get use of accumulation buffer state.
	  * \return \c true if use of accumulation buffer is enabled.
	  */
	bool GetUseAccumulationBuffer();

	/** Change use depth of field flag.
	  * \param pUseDepthOfField Set to \c true if depth of field is used.
	  */
	void SetUseDepthOfField(bool pUseDepthOfField);

	/** Get use depth of field state.
	  * \return \c true if depth of field is used.
	  */
	bool GetUseDepthOfField();

    /** \enum ECameraFocusDistanceSource Camera focus sources.
	  * - \e eCAMERA_INTEREST
	  * - \e eSPECIFIC_DISTANCE
	  */
    typedef enum 
    {
	    eCAMERA_INTEREST, 
	    eSPECIFIC_DISTANCE
    } ECameraFocusDistanceSource;

	/** Set source of camera focus distance.
	  * \param pSource Focus distance source identifier.
	  */
    void SetFocusDistanceSource(ECameraFocusDistanceSource pSource);

	/** Get source of camera focus distance.
	  * \return Focus distance source identifier.
	  */
	ECameraFocusDistanceSource GetFocusDistanceSource();

	/** Set the focus distance of the lens in millimiters.
	  * \param pDistance Focus distance value.
	  * \remarks This parameter is only relevant if focus
	  * distance source is set to eSPECIFIC_DISTANCE.
	  */
	void SetSpecificDistance(double pDistance);

	/** Get the focus distance of the lens in millimiters.
	  * \return Focus distance value.
	  * \remarks This parameter is only relevant if focus
	  * distance source is set to eSPECIFIC_DISTANCE.
	  */
	double GetSpecificDistance();

	/** Set the focus angle in degrees.
	  * \param pAngle Focus angle value.
	  */
	void SetFocusAngle(double pAngle);
	
	/** Get the focus angle in degrees.
	  * \return Focus angle value.
	  */
	double GetFocusAngle();

	//@}

	/**
	  * \name Default Animation Values
	  * \remarks This set of functions provide direct access to default
	  * animation values specific to a camera. The default animation 
	  * values are found in the default take node of the associated node.
	  * Hence, these functions only work if the camera has been associated
	  * with a node.
	  */
	//@{

	/** Set default field of view in degrees.
	  * When the camera aperture mode is set to \e eHORIZONTAL, this parameter 
	  * sets the horizontal field of view in degrees and the vertical field of
	  * view is adjusted accordingly.
	  * When the camera aperture mode is set to \e eVERTICAL, this parameter 
	  * sets the vertical field of view in degrees and the horizontal field of
	  * view is adjusted accordingly.
	  * This parameter has no effect if the camera aperture mode is set to
	  * \e eHORIZONTAL_AND_VERTICAL. Its default value is 25.115.
	  * \param pFieldOfView Field of view value.
	  */
	void SetDefaultFieldOfView(double pFieldOfView);

	/** Get default field of view.
	  * When the camera aperture mode is set to \e eHORIZONTAL, this parameter 
	  * sets the horizontal field of view in degrees and the vertical field of
	  * view is adjusted accordingly.
	  * When the camera aperture mode is set to \e eVERTICAL, this parameter 
	  * sets the vertical field of view in degrees and the horizontal field of
	  * view is adjusted accordingly.
	  * This parameter has no effect if the camera aperture mode is set to
	  * \e eHORIZONTAL_AND_VERTICAL. Its default value is 25.115.
	  * \return Currently set field of view value.
	  */
	double GetDefaultFieldOfView();

	/** Set default field of view X.
	  * When the camera aperture mode is set to \e eHORIZONTAL_AND_VERTICAL, 
	  * this parameter sets the horizontal field of view in degrees.
	  * This parameter has no effect otherwise. Its default value is 40.
	  * \param pFieldOfViewX Field of view value.
	  */
	void SetDefaultFieldOfViewX(double pFieldOfViewX);

	/** Get default field of view X.
	  * When the camera aperture mode is set to \e eHORIZONTAL_AND_VERTICAL, 
	  * this parameter sets the horizontal field of view in degrees.
	  * This parameter has no effect otherwise. Its default value is 40.
	  * \return Currently set field of view X value.
	  */
	double GetDefaultFieldOfViewX();

	/** Set default field of view Y.
	  * When the camera aperture mode is set to \e eHORIZONTAL_AND_VERTICAL, 
	  * this parameter sets the vertical field of view in degrees.
	  * This parameter has no effect otherwise. Its default value is 40.
	  * \param pFieldOfViewY Field of view value.
	  */
	void SetDefaultFieldOfViewY(double pFieldOfViewY);

	/** Get default field of view Y.
	  * When the camera aperture mode is set to \e eHORIZONTAL_AND_VERTICAL, 
	  * this parameter sets the vertical field of view in degrees.
	  * This parameter has no effect otherwise. Its default value is 40.
	  * \return Currently set field of view Y value.
	  */
	double GetDefaultFieldOfViewY();

	/** Set default optical center X.
	  * This parameter sets the optical center horizontal offset when the 
	  * camera aperture mode is set to \e eHORIZONTAL_AND_VERTICAL. This 
	  * parameter has no effect otherwise. Its default value is 0.
	  * \param pOpticalCenterX Optical center value.
	  */
	void SetDefaultOpticalCenterX(double pOpticalCenterX);

	/** Get default optical center X.
	  * This parameter sets the optical center horizontal offset when the 
	  * camera aperture mode is set to \e eHORIZONTAL_AND_VERTICAL. This 
	  * parameter has no effect otherwise. Its default value is 0.
	  * \return Currently set optical center value.
	  */
	double GetDefaultOpticalCenterX();

	/** Set default optical center Y.
	  * This parameter sets the optical center vertical offset when the 
	  * camera aperture mode is set to \e eHORIZONTAL_AND_VERTICAL. This 
	  * parameter has no effect otherwise. Its default value is 0.
	  * \param pOpticalCenterX Optical center value.
	  */
	void SetDefaultOpticalCenterY(double pOpticalCenterY);

	/** Get default optical center Y.
	  * This parameter sets the optical center vertical offset when the 
	  * camera aperture mode is set to \e eHORIZONTAL_AND_VERTICAL. This 
	  * parameter has no effect otherwise. Its default value is 0.
	  * \return Currently set optical center value.
	  */
	double GetDefaultOpticalCenterY();

	/** Set default roll in degrees.
	  * \param pRoll Roll value. This parameter's default value is 0 degrees.
	  */
	void SetDefaultRoll(double pRoll);

	/** Get default roll in degrees.
	  * \return Currently set roll value. This parameter's default value is 0 degrees.
	  */
	double GetDefaultRoll();

	/** Set default turn table in degrees.
	  * \param pTurnTable TurnTable value. This parameter's default value is 0 degrees.
	  */
	void SetDefaultTurnTable(double pTurnTable);

	/** Get default turn table in degrees.
	  * \return Currently set turntable value. This parameter's default value is 0 degrees.
	  */
	double GetDefaultTurnTable();

	/** Set default background color.
	  * \param pColor RGB values of the background color. This parameter's default value is black.
	  */
	void SetDefaultBackgroundColor(const KFbxColor& pColor);

	/** Get default background color.
	  * \return Currently set background color. This parameter's default value is black.
	  */
	KFbxColor GetDefaultBackgroundColor();

	//@}

	/**
	  * \name Obsolete Functions
	  * \remarks This set of functions is obsolete since animated background
	  * color is now supported.
	  */
	//@{

	/**	Set background color. This method is replaced by the SetDefaultBackgroundColor when
      * setting a non animated value. For animated values, the client application need to 
	  * access the BackgroundColor fcurves in the take.
	  */
	void SetBackgroundColor(const KFbxColor& pColor);

	/**	Get background color. This method is replaced by the SetDefaultBackgroundColor when
      * setting a non animated value. For animated values, the client application need to 
	  * access the BackgroundColor fcurves in the take.
	  */
	KFbxColor GetBackgroundColor();

	//@}

	//! Assignment operator.
    KFbxCamera& operator=(KFbxCamera& pCamera);


	void SetBackgroundTexture(KFbxTexture* pTexture);

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

protected:

	KFbxCamera(KFbxSdkManager& pManager);
	virtual ~KFbxCamera ();

	/** Add channels specific to a camera attribute.
      * \param pTakeNode Take node to add specialized channels to.
	  */
	virtual void AddChannels(KFbxTakeNode *pTakeNode);

	/** Remove channels specific to a camera attribute.
      * \param pTakeNode Take node to remove specialized channels from.
	  */
	virtual void RemoveChannels(KFbxTakeNode *pTakeNode);

	/**
	  *	Used to retrieve the KProperty list from an attribute
	  */
	virtual void FillPropertyList(void* pPropertyList);
	KFbxCameraProperties* mPH;

	KString		GetTypeName();
	KStringList	GetTypeFlags();

private:

	double ComputePixelRatio(kUInt pWidth, kUInt pHeight, double pScreenRatio = 1.3333333333);
    double ComputeAngleOfView(double pApertureWidth, double pApertureHeight, double pFocalLength);
    double ComputeFocalLength(double pApertureWidth, double pApertureHeight, double pAngleOfView);

	// Camera Position and Orientation
    KFbxVector4 mUpVector;
	KFbxVector4 mDefaultCameraInterestPosition;

	// Camera Ortho Zoom
	double mOrthoZoom;

	// Background Properties
	KString mBackgroundMediaName;
	KString mBackgroundFileName;

	// Camera View Options
	bool mShowInfoOnMoving;
	bool mDisplaySafeAreaOnRender;
	bool mShowAudio;
	KFbxColor mAudioColor;

	// Render Options
	ECameraAntialiasingMethod mAntialiasingMethod;
	ECameraSamplingType mSamplingType;

	friend class KFbxSdkManager;
	friend class KFbxNode;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

typedef KFbxCamera* HKFbxCamera;

} // namespace
#endif // #ifndef _KFbxCamera_h


