#ifndef _KFbxGeometry_h 
#define _KFbxGeometry_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxplugins/kfbxgeometrybase.h>

#include <klib/karrayul.h>
#include <klib/kerror.h>

class KFCurve;
class KFCurveNode;

#include <kfbxmath/kfbxxmatrix.h>
#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxLink;
class KFbxMaterial;
class KFbxNode;
class KFbxShape;
class KFbxTexture;
class KFbxSdkManager;
class KFbxVector4;
class KFbxTakeNode;

/** Contains common properties to mesh, nurb and patch node attributes.
  * \nosubgrouping
  * A geometry node attribute has arrays of links, shapes, materials and 
  * textures. It also has arrays for control points, normals, material indices
  * texture indices and texture UV coordinates. Some of these are only used
  * in mesh node attributes.
  */
class KFBX_DLL KFbxGeometry : public KFbxGeometryBase
{

public:

	/** Return the type of node attribute.
	  * This class is pure virtual.
	  */
	virtual EAttributeType GetAttributeType() = 0;

	/**
	  * \name Link Management
	  */
	//@{

	/** Add a link.
	  * \param pLink Pointer to the link object to add.
	  * \return Index of added link.
	  */
	int AddLink(KFbxLink* pLink);

	/** Remove all links without destroying them.
      * If links aren't explicitly destroyed before calling this function, they will be
	  * destroyed along with the SDK manager.
	  */
    void ClearLink();

	/** Get the number of links.
	  * \return Number of links that have been added to this object.
	  */
	int GetLinkCount();

	/** Get link at given index.
      * \param pIndex Index of link.
	  * \return Pointer to link or \c NULL if index is out of range in wich case
	  *	KFbxGeometry::GetLastErrorID() returns eINDEX_OUT_OF_RANGE.
	  */
	KFbxLink* GetLink(int pIndex);

	//@}

	/**
	  * \name Shape Management 
	  */
	//@{

	/** Add a shape and its associated name.
	  * \param pShape Pointer to the shape object.
	  * \param pShapeName Name given to the shape.
	  * \return Index of added shape, -1 if operation failed.
	  *	In the last case, KFbxGeometry::GetLastErrorID() can return one of the following:
	  *		- eNULL_PARAMETER: Pointer to shape is \c NULL.
	  *     - eSHAPE_ALREADY_ADDED: Shape has already been added.
	  *		- eSHAPE_INVALID_NAME: The provided name is empty.
	  *		- eSHAPE_NAME_CLASH: The provided name is already used by another shape.
	  * \remarks The provided name is stripped from surrounding whitespaces before being 
	  * compared with other shape names. It is a good practice not to prefix the shape
	  * name with its enclosing node name because Motionbuilder is known to strip this prefix 
	  * and not save it back.
	  */
	int AddShape(KFbxShape* pShape, char* pShapeName);

	/** Remove all shapes without destroying them.
      * If shapes aren't explicitly destroyed before calling this function, they will be
	  * destroyed along with the SDK manager.
	  */
    void ClearShape();

	/** Get the number of shapes.
	  * \return Number of shapes that have been added to this object.
	  */
	int GetShapeCount();

	/** Get shape at given index.
      * \param pIndex Index of shape.
	  * \return Pointer to shape or \c NULL if index is out of range in wich case
	  *	KFbxGeometry::GetLastErrorID() returns eINDEX_OUT_OF_RANGE.
	  */
	KFbxShape* GetShape(int pIndex);

	/** Get shape name at given index.
      * \param pIndex Index of shape.
	  * \return Shape name or \c NULL if index is out of range in wich case
	  *	KFbxGeometry::GetLastErrorID() returns eINDEX_OUT_OF_RANGE.
	  */
	char* GetShapeName(int pIndex);

	/** Get a shape channel.
	  * This parameter has a scale from 0 to 100, 100 meaning full shape deformation.
	  * This parameter's default value is 0.
	  * \param pTakeNode Take node.
	  * \param pShapeIndex Shape index.
	  *	\return Animation curve or NULL if an error occured in wich case
	  *	KFbxGeometry::GetLastErrorID() can return one of the following:
	  *		- eINDEX_OUT_OF_RANGE: Shape index is out of range.
	  *		- eNULL_PARAMETER: Pointer to take node is \c NULL.
	  *		- eSHAPE_NO_CURVE_FOUND: Shape curve could not be found.
	  */
	KFCurve* GetShapeChannel(KFbxTakeNode* pTakeNode, int pShapeIndex);

	//@}

    /** Surface modes
	  * This information is only used in nurbs and patches.
	  */

	/** \enum ESurfaceMode Types of surfaces.
	  * - \e eRAW
	  * - \e eLOW_NO_NORMALS
	  * - \e eLOW
	  * - \e eHIGH_NO_NORMALS
	  * - \e eHIGH
	  */
    typedef enum 
    {
        eRAW,
	    eLOW_NO_NORMALS,
	    eLOW,
	    eHIGH_NO_NORMALS,
	    eHIGH
    } ESurfaceMode;

	/**
	  * \name Pivot Management
	  * The geometry pivot is used to specify additional translation, rotation 
	  * and scaling applied to all the control points when the model is 
	  * exported.
	  */
	//@{

	/** Get pivot matrix.	
	  * \param pXMatrix Placeholder for the returned matrix.
	  * \return Reference to the passed argument.
	  */
	KFbxXMatrix& GetPivot(KFbxXMatrix& pXMatrix);
	
	/** Set pivot matrix.	
	  * \param pXMatrix The Transformation matrix.
	  */
	void SetPivot(KFbxXMatrix& pXMatrix);

	/** Apply the pivot matrix to all vertices/normals of the geometry.
	  */
	void ApplyPivot();

	//@}

	/**
	  * \name Default Animation Values
	  * \remarks This set of functions provides direct access to default
	  * animation values specific to a geometry. The default animation 
	  * values are found in the default take node of the associated node.
	  * Hence, these functions only work if the geometry has been associated
	  * with a node.
	  */
	//@{

	/** Set default deformation for a given shape.
	  * This parameter has a scale from 0 to 100, 100 meaning full shape deformation.
	  * This parameter's default value is 0.
	  * \param pIndex Shape index.
	  * \param pPercent Deformation percentage on a scale ranging from 0 to 100.
	  * \remarks This function has no effect if the given index is out of range.
	  */
	void SetDefaultShape(int pIndex, double pPercent); 

	/** Get default deformation for a given shape.
	  * This parameter has a scale from 0 to 100, 100 meaning full shape deformation.
	  * This parameter's default value is 0.
	  * \param pShapeName Shape name.
	  * \return The deformation value for the given shape.
	  * \remarks If the index is out of range, return 0.
	  */
	double GetDefaultShape(int pIndex); 

	//@}

	//! Assignment operator.
    KFbxGeometry& operator=(KFbxGeometry& pGeometry);

	/**
	  * \name Error Management
	  */
	//@{

	/** Retrieve error object.
	 *	\return Reference to error object.
	 */
	KError& GetError ();

    /** \enum EError Error identifiers.
	  * - \e eINDEX_OUT_OF_RANGE
	  * - \e eNULL_PARAMETER
	  * - \e eMATERIAL_NOT_FOUND
	  * - \e eMATERIAL_ALREADY_ADDED
	  * - \e eTEXTURE_NOT_FOUND
	  * - \e eTEXTURE_ALREADY_ADDED
	  * - \e eSHAPE_ALREADY_ADDED
	  * - \e eSHAPE_INVALID_NAME
	  * - \e eSHAPE_NAME_CLASH
	  * - \e eSHAPE_NO_CURVE_FOUND
	  */
	typedef enum 
	{
		eINDEX_OUT_OF_RANGE,
		eNULL_PARAMETER,
		eMATERIAL_NOT_FOUND,
		eMATERIAL_ALREADY_ADDED,
		eTEXTURE_NOT_FOUND,
		eTEXTURE_ALREADY_ADDED,
		eSHAPE_ALREADY_ADDED,
		eSHAPE_INVALID_NAME,
		eSHAPE_NAME_CLASH,
		eSHAPE_NO_CURVE_FOUND,
		eERROR_COUNT
	} EError;	

	/** Get last error code.
	 *	\return Last error code.
	 */
	EError GetLastErrorID();

	/** Get last error string.
	 *	\return Textual description of the last error.
	 */
	char* GetLastErrorString();

	//@}

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS	

public:

	KFCurveNode* GetShapeCurveNode(KFbxTakeNode* pTakeNode, int pShapeIndex);

protected:

	KFbxGeometry(KFbxSdkManager& pManager);
	virtual ~KFbxGeometry();

	/** Overloaded function called in KFbxNode::SetNodeAttribute().
	*   \param pNode Associated node.
	*   \remarks Do not call this function anywhere else.
    */
	virtual void SetNode(KFbxNode& pNode);

	/** Overloaded function called in KFbxNode::SetNodeAttribute().
	*   \remarks Do not call this function anywhere else.
    */
	virtual void UnsetNode();

	/** Add channels specific to a geometry attribute.
    *   \param pTakeNode Take node to add specialized channels to.
	*/
	virtual void AddChannels(KFbxTakeNode *pTakeNode);
	bool AddShapeChannel(KFbxTakeNode *pTakeNode, int pShapeIndex);

	/** Remove channels specific to a geometry attribute.
    *   \param pTakeNode Take node to remove specialized channels from.
	*/
	virtual void RemoveChannels(KFbxTakeNode *pTakeNode);
	bool RemoveShapeChannel(KFbxTakeNode *pTakeNode, int pShapeIndex);

	// MotionBuilder 4.01 and earlier versions saved nurb and patch shape channel names 
	// following the template "Shape 0x (Shape)" where x is the index of the shape starting 
	// at 1. Since then, Jori modified shape channels to turn them into animated properties. 
	// As a result, nurb and patch shape channel names are now saved following the template 
	// "<shape name> (Shape)". The FBX SDK keeps the old shape channel naming scheme but has 
	// been modifed to handle the new one and convert shape channel names to the old shape 
	// channel naming scheme.
	void CleanShapeChannels(KFbxTakeNode *pTakeNode);
	void CleanShapeChannel(KFbxTakeNode *pTakeNode, int pShapeIndex);

	// Shape channel name creation for nurb and patch.
	KString CreateShapeChannelName(int pShapeIndex);

	// Shape channel name creation for mesh.
	KString CreateShapeChannelName(KString pShapeName);

	void CreateShapeChannelProperties(KString& pShapeName);
	
	void ConvertShapeNamesToV5Format(KFbxTakeNode *pTakeNode);
	void ConvertShapeNamesToV5Format(KFbxTakeNode *pTakeNode, int pShapeIndex);
	void RevertShapeNamesToV6Format(KFbxTakeNode *pTakeNode);
	void RevertShapeNamesToV6Format(KFbxTakeNode *pTakeNode, int pShapeIndex);
	void ClearTemporaryShapeNames();

	/** Remove a link.
	  * \param pIndex Index of link to remove.
	  * \return Pointer to removed link if success, \c NULL otherwise.
	  *	In the last case, KFbxGeometry::GetLastErrorID() returns eINDEX_OUT_OF_RANGE.
      */
	KFbxLink* RemoveLink(int pIndex);

	/** Remove a shape.
	  * \param pIndex Index of shape to remove.
	  * \return Pointer to removed shape if success, \c NULL otherwise.
	  *	In the last case, KFbxGeometry::GetLastErrorID() returns eINDEX_OUT_OF_RANGE.
      */
	// This function implies renaming shape channels. The decision has been made to avoid changing 
	// KFCurveNode interface to allow renaming.
	// KFbxShape* RemoveShape(int pIndex);

	void CopyLinks(KFbxGeometry* pGeometry);
	void CopyShapes(KFbxGeometry* pGeometry);

	void PivotInit();
	void CopyPivot(KFbxGeometry* pSource);
	

    KArrayTemplate <KFbxLink*> mLinkArray;
    KArrayTemplate <KFbxShape*> mShapeArray;

	KArrayTemplate <KString*> mShapeNameArray;

	// Used during FBX v5 file store
	KArrayTemplate<KString*> mShapeNameArrayV6;
	KArrayTemplate<KString*> mShapeNameArrayV5;
	KArrayTemplate<KString*> mShapeChannelNameArrayV5;

	KFbxXMatrix mPivot;
	
		
	KError mError;

	friend class KFbxSdkManager;
	friend class KFbxWriterFbx;
	friend class KFbxWriterFbx6;
	friend class KFbxReaderFbx;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

} // namespace
#endif // #ifndef _KFbxGeometry_h


