#ifndef _KFbxLink_h
#define _KFbxLink_h

/**************************************************************************************

 Copyright  1999 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxmath/kfbxvector4.h>
#include <klib/kstring.h>
#include <klib/karrayul.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxNode;
class KFbxXMatrix;
class KFbxDeformer;

/** Geometrical link between nodes.
  * \nosubgrouping
  * A link is set to an instance of class KFbxGeometry by calling 
  * KFbxGeometry::AddLink(). The link node associated by calling 
  * KFbxLink::SetLink() is also referenced as a cluster or a bone.
  * \par
  * A typical example of a link is the relationship between a skin and a bone. 
  * In this case:
  *		- the skin is the instance of class KFbxGeometry owning the link
  *		- the bone is the node associated by calling KFbxLink::SetLink()
  *		- if the link mode is set to eADDITIVE, the parent of the bone is the node associated by calling KFbxLink::SetAssociateModel()
  */
class KFBX_DLL KFbxLink  
{

public:

	/** Restore the link to its initial state.
	  * Calling this function will clear the following:
	  *		- pointer to linked node
	  *		- pointer to associate model
	  *		- control point indices and weights
	  *		- transformation matrices
	  */
	void Reset();

    /** \enum ELinkMode Link modes.
	  * - \e eNORMALIZE
	  * - \e eADDITIVE
	  * - \e eTOTAL1
	  *
	  * The link mode sets how the link influences the position of a control 
	  * point and the relationship between the weights assigned to a control 
	  * point. The weights assigned to a control point are distributed among 
	  * the set of links associated with an instance of class KFbxGeometry. 
	  * Here is a short description of the available link modes:
	  * \par
	  *	In mode eNORMALIZE, the sum of the weights assigned to a control point 
	  * is normalized to 1.0. Setting the associate model in this mode is not 
	  * relevant. The influence of the link is a function of the displacement of the 
	  * link node relative to the node containing the control points.
	  * \par
	  *	In mode eADDITIVE, the sum of the weights assigned to a control point 
	  * is kept as is. It is the only mode where setting the associate model is
	  * relevant. The influence of the link is a function of the displacement of 
	  * the link node relative to the node containing the control points or, 
	  * if set, the associate model. The weight gives the proportional displacement 
	  * of a control point. For example, if the weight of a link over a control 
	  * point is set to 2.0, a displacement of the link node of 1 unit in the X 
	  * direction relative to the node containing the control points or, if set, 
	  * the associate model, triggers a displacement of the control point of 2 
	  * units in the same direction.
	  * \par
	  * Mode eTOTAL1 is identical to mode eNORMALIZE except that the sum of the 
	  * weights assigned to a control point is not normalized and must equal 1.0.
	  */
    typedef enum   
    { 
	    eNORMALIZE, 
        eADDITIVE, 
	    eTOTAL1
    } ELinkMode;

	/** Set the link mode. 
	  * \param pMode The link mode.
	  * \remarks All the links associated to an instance of class KFbxGeometry 
	  * must have the same link mode.
	  */
	void SetLinkMode(ELinkMode pMode);
	
	/** Get the link mode. 
	  * \return The link mode.
	  */
	ELinkMode GetLinkMode();
		
	/** Set the link node.
	  * \param pNode The link node.
	  * \remarks The link node is the node which influences the displacement 
	  * of the control points. Typically, the link node is the bone a skin is 
	  * attached to.
	  */
	void SetLink(KFbxNode *pNode);
	
	/** Get the link node.
	  * \return The link node or \c NULL if KFbxLink::SetLink() has not been called before.
	  * \remarks The link node is the node which influences the displacement 
	  * of the control points. Typically, the link node is the bone a skin is 
	  * attached to.
	  */
	KFbxNode* GetLink();
	
	/** Set the associate model.
	  * The associate model is optional. It is only relevant if the link mode 
	  * is of type eADDITIVE.
	  * \param pNode The associate model node. 
	  * \remarks If set, the associate model is the node used as a reference to 
	  * measure the relative displacement of the link node. Otherwise, the 
	  * displacement of the link node is measured relative the the node 
	  * containing the control points. Typically, the associate model node is 
	  * the parent of the bone a skin is attached to. 
	  */
	void SetAssociateModel(KFbxNode *pNode);
	
	/** Get the associate model.
	  * The associate model is optional. It is only relevant if the link mode is of type
	  * eADDITIVE.
	  * \return The associate model node or \c NULL if KFbxLink::SetAssociateModel() has not been called before.
	  * \remarks If set, the associate model is the node used as a reference to 
	  * measure the relative displacement of the link node. Otherwise, the 
	  * displacement of the link node is measured relative the the node 
	  * containing the control points. Typically, the associate model node is 
	  * the parent of the bone a skin is attached to. 
	  */
	KFbxNode* GetAssociateModel();
	
	/**
	  * \name Control Points
	  * \remarks A link has an array of indices to control points and associated weights.
	  * The indices refer to the control points in the instance of class KFbxGeometry 
	  * owning the link. The weights are the influence of the link node over the 
	  * displacement of the indexed control points.
	  */
	//@{

	/** Add an element in both arrays of control point indices and weights.
	  * \param pIndex The index of the control point.
	  * \param pWeight The link weight.
	  */
	void AddControlPointIndex(int pIndex, double pWeight);

	/** Get the length of the arrays of control point indices and weights.
	  * \return Length of the arrays of control point indices and weights.
	  * 0 if no control point index has been added yet or the arrays have been reset.
	  */
	int GetControlPointIndicesCount();

	/** Get the array of control point indices.
	  * \return Pointer to the array of control point indices.
	  * NULL if no control point index has been added yet or the array has been reset.
	  */
	int* GetControlPointIndices();
	
	/** Get the array of control point weights.
	  * \return Pointer to the array of control point weights.
	  * NULL if no control point index has been added yet or the array has been reset.
	  */
	double* GetControlPointWeights();
	
	//@}


	/**
	  * \name Transformation Matrices
	  * A link has three transformation matrices:
	  *		- Transform refers to the global initial position of the node containing the link 
	  *		- TransformLink refers to global initial position of the link node 
	  *		- TransformAssociateModel refers to the global initial position of the associate model 
	  * These matrices are used to set the positions where the 
	  * influences of the link node and associate model over the 
	  * control points are null.
	  */
	//@{

	/** Set translation vector in matrix associated with the node containing the link.
	  * \param pVector Translation vector.
	  */
	void SetTransformT(const KFbxVector4& pVector);

	/** Get translation vector in matrix associated with the node containing the link.
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxVector4& GetTransformT(KFbxVector4& pVector);
	
	/** Set rotation vector in matrix associated with the node containing the link.
	  * \param pVector Rotation vector.
	  */
	void SetTransformR(const KFbxVector4& pVector);

	/** Get rotation vector in matrix associated with the node containing the link.
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxVector4& GetTransformR(KFbxVector4& pVector);
	
	/** Set scaling vector in matrix associated with the node containing the link.
	  * \param pVector Scaling vector.
	  */
	void SetTransformS(const KFbxVector4& pVector);

	/** Get scaling vector in matrix associated with the node containing the link.
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxVector4& GetTransformS(KFbxVector4& pVector);
	
	/** Set translation vector in matrix associated with the link node.
	  * \param pVector Translation vector.
	  */
	void SetTransformLinkT(const KFbxVector4& pVector);

	/** Get translation vector in matrix associated with the link node. 
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxVector4& GetTransformLinkT(KFbxVector4& pVector);
	
	/** Set rotation vector in matrix associated with the link node. 
	  * \param pVector Rotation vector.
	  */
	void SetTransformLinkR(const KFbxVector4& pVector);

	/** Get rotation vector in matrix associated with the link node. 
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxVector4& GetTransformLinkR(KFbxVector4& pVector);
	
	/** Set scaling vector in matrix associated with the link node.
	  * \param pVector Scaling vector.
      */
	void SetTransformLinkS(const KFbxVector4& pVector);

	/** Get scaling vector in matrix associated with the link node.
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxVector4& GetTransformLinkS(KFbxVector4& pVector);
	
	/** Set translation vector in matrix associated with the associate model. 
	  * \param pVector Translation vector.
	  */
	void SetTransformAssociateModelT(const KFbxVector4& pVector);
	
	/** Get translation vector in matrix associated with the associate model.  
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxVector4& GetTransformAssociateModelT(KFbxVector4& pVector);

	/** Set rotation vector in matrix associated with the associate model.  
	  * \param pVector Rotation vector.
      */
	void SetTransformAssociateModelR(const KFbxVector4& pVector);
	
	/** Get rotation vector in matrix associated with the associate model.  
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxVector4& GetTransformAssociateModelR(KFbxVector4& pVector);

	/** Set scaling vector in matrix associated with the associate model. 
	  * \param pVector Scaling vector.
	  */
	void SetTransformAssociateModelS(const KFbxVector4& pVector);
		
	/** Get scaling vector in matrix associated with the associate model. 
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxVector4& GetTransformAssociateModelS(KFbxVector4& pVector);

	/** Set translation vector in matrix associated with the parent node.
	  * \param pVector translation vector.
	  */
	void SetTransformParentT(const KFbxVector4& pVector);

	/** Get translation vector in matrix associated with the parent node.
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxVector4& GetTransformParentT(KFbxVector4& pVector);
	
	/** Set rotation vector in matrix associated with the parent node.
	  * \param pVector Rotation vector.
	  */
	void SetTransformParentR(const KFbxVector4& pVector);

	/** Get rotation vector in matrix associated with the parent node.
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxVector4& GetTransformParentR(KFbxVector4& pVector);
	
	/** Set scaling vector in matrix associated with the parent node.
	  * \param pVector Scaling vector.
	  */
	void SetTransformParentS(const KFbxVector4& pVector);

	/** Get scaling vector in matrix associated with the parent node.
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxVector4& GetTransformParentS(KFbxVector4& pVector);
	
	/** Set matrix associated with the node containing the link.
	  * \param pMatrix Transformation matrix.
	  */
	void SetTransformMatrix(KFbxXMatrix& pMatrix);

	/** Get matrix associated with the node containing the link.
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxXMatrix& GetTransformMatrix(KFbxXMatrix& pMatrix);
	
	//! Set matrix associated with the link node.
	void SetTransformLinkMatrix(KFbxXMatrix& pMatrix);

	/** Set matrix associated with the link node.
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxXMatrix& GetTransformLinkMatrix(KFbxXMatrix& pMatrix);
	
	/** Set matrix associated with the associate model.
	  * \param pMatrix Transformation matrix.
	  */
	void SetTransformAssociateModelMatrix(KFbxXMatrix& pMatrix);

	/** Get matrix associated with the associate model.
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxXMatrix& GetTransformAssociateModelMatrix(KFbxXMatrix& pMatrix);

	/** Set matrix associated with the parent node.
	  * \param pMatrix Transformation matrix.
	  */
	void SetTransformParentMatrix(KFbxXMatrix& pMatrix);

	/** Get matrix associated with the parent node.
	  * \return Input parameter filled with appropriate data.
	  */
	KFbxXMatrix& GetTransformParentMatrix(KFbxXMatrix& pMatrix);

	/** Get the Transform Parent set flag value.
	  * \return \c true if transform matrix associated with parent node is set.
	  */
	bool IsTransformParentSet() { return mIsTransformParentSet; }

	//@}

	//! Assignment operator.
    KFbxLink& operator=(KFbxLink& pLink);
	KFbxLink& operator=(KFbxDeformer& pDeformer);

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

protected:

	KFbxLink ();
	virtual ~KFbxLink ();
	KString GetLinkName ();
	KString GetAssociateModelName ();

	/**
	  * \name User Data
	  * Service functions to store and retrieve extra data about the link.
	  * Only used by the Maya plugin, should move somewhere out of here...
	  */
	//@{

	/** Set user data.
	  * \param pUserDataID Identifier of user data.
	  * \param pUserData User data.
	  */
	void SetUserData(KString pUserDataID, KString pUserData);
	
	//! Get the user data identifier.
	KString GetUserDataID ();
	
	//! Get the user data.
	KString GetUserData ();
	
	//! Get the user data by identifier. 
	KString GetUserData (KString pUserDataID);

	//@}

	KFbxNode*   mLinkNode;
    KString     mLinkNodeName;

	ELinkMode   mLinkMode;

	KFbxNode*   mAssociateModel;
    KString     mAssociateModelName;

	KString     mUserDataID;
	KString     mUserData;

	KFbxVector4 mT, mR, mS;
	KFbxVector4 mLT, mLR, mLS;
	KFbxVector4 mAT, mAR, mAS;

	bool mIsTransformParentSet;
	KFbxVector4 mPT, mPR, mPS;

	KArrayTemplate<int> mControlPointIndices;
	KArrayTemplate<double> mControlPointWeights;

	friend class KFbxSdkManager;
	friend class KFbxScene;
	friend class KFbxWriterFbx;
	friend class KFbxWriterFbx6;
	friend class KFbxReaderFbx;
	friend class KFbxGeometryConverter;
	friend class KFbxDeformer;
	friend class FbxMayaExport;
	friend class FbxMayaImport;
	

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS 

};

typedef KFbxLink* HKFbxLink;

} // namespace
#endif // #ifndef _KFbxLink_h


