#ifndef _KFbxNodeLimits_h
#define _KFbxNodeLimits_h

/**************************************************************************************

 Copyright  2002 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxmath/kfbxvector4.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxNode;

/** The KFbxLimits defines a 3 component min, max limit. It uses KFbxVector4
  * objects to store the values, but although the members are identified as
  * X, Y and Z (the W component is ignored) at this level, they are unitless
  * values and they will only get a meaning in the context they are queried.
  * \nosubgrouping
  */
class KFBX_DLL KFbxLimits
{
	public:
		//! Constructor.
		KFbxLimits()
		{
			mMinActive[0]    = mMinActive[1]    = mMinActive[2]    = false;
			mMaxActive[0]    = mMaxActive[1]    = mMaxActive[2]    = false;
		}

		/** Set the active state of min limit.
		  * \param pXActive If \c true, the X component of the min limit is active.
		  * \param pYActive If \c true, the Y component of the min limit is active.
		  * \param pZActive If \c true, the Z component of the min limit is active.
		  */
		void SetLimitMinActive(const bool pXActive, const bool pYActive, const bool pZActive);

		/** Get the active states of the three components of the min limit.
		  * \param pXActive If \c true, the X component of the min limit is active.
		  * \param pYActive If \c true, the Y component of the min limit is active.
		  * \param pZActive If \c true, the Z component of the min limit is active.
		  */
		void GetLimitMinActive(bool& pXActive, bool& pYActive, bool& pZActive) const;

		/** Set the active state of max limit.
		  * \param pXActive If \c true, the X component of the max limit is active.
		  * \param pYActive If \c true, the Y component of the max limit is active.
		  * \param pZActive If \c true, the Z component of the max limit is active.
		  */
		void SetLimitMaxActive(const bool pXActive, const bool pYActive, const bool pZActive);

		/** Get the active states of the three components of the max limit.
		  * \param pXActive If \c true, the X component of the max limit is active.
		  * \param pYActive If \c true, the Y component of the max limit is active.
		  * \param pZActive If \c true, the Z component of the max limit is active.
		  */
		void GetLimitMaxActive(bool& pXActive, bool& pYActive, bool& pZActive) const;

		/** Check if at least one of the active flags is set.
		  * \return \c true if one of the six active flags is set.
		  */
		bool GetLimitSomethingActive() const;
		
		/** Set the min limit.
		  * \param pMin The X, Y and Z values for the min limit.
		  */
		void        SetLimitMin(const KFbxVector4& pMin);

		/** Get the min limit.
		  * \return Min limit triplet.
		  */
		KFbxVector4 GetLimitMin() const;

		/** Set the max limit.
		  * \param pMax The X, Y and Z values for the max limit.
		  */
		void        SetLimitMax(const KFbxVector4& pMax);

		/** Get the max limit.
		  * \return Max limit triplet.
		  */
		KFbxVector4 GetLimitMax() const;

	private:
		bool        mMinActive[3], mMaxActive[3];
		KFbxVector4 mMin, mMax;
};

/** The KFbxNodeLimits defines limits for transforms.
  * \nosubgrouping
  */
class KFBX_DLL KFbxNodeLimits
{
	
	public:
		/** Constructor.
		  * pLimitedNode Pointer to the node these limits apply to.
		  */
		KFbxNodeLimits(KFbxNode* pLimitedNode);

		/** Get the limited node.
		  * \return Pointer to the node these limits apply to. This node is the same pointer as 
		  * the one passed to the constructor.
		  */
		KFbxNode* GetLimitedNode();

		/**
		  * \name Node Translation Limits
		  */
		//@{

		/** Change the translation limit active flag. If this flag is set to \c false,
		  * the values in the mTranslationLimits have to be ignored.
		  * \param pActive State of the translation limits active flag.
		  */
		void SetTranslationLimitActive(bool pActive);

		/** Get the translation limit active flag.
		  * \return Translation limit active flag state. If this flag is set to \c false,
		  * the values in the mTranslationLimits have to be ignored.
		  */
		bool GetTranslationLimitActive() const;

		//! The translation limits.
		KFbxLimits mTranslationLimits;

		//@}

		/**
		  * \name Node Rotation Limits
		  */
		//@{

		/** Change the rotation limit active flag. If this flag is set to \c false,
		  * the values in the mRotationLimits have to be ignored.
		  * \param pActive State of the rotation limits active flag.
		  */
		void SetRotationLimitActive(bool pActive);

		/** Get the rotation limit active flag.
		  * \return Rotation limit active flag state. If this flag is set to \c false,
		  * the values in the mRotationLimits have to be ignored.
		  */
		bool GetRotationLimitActive() const;

		//! The rotation limits.
		KFbxLimits mRotationLimits;

		//@}

		/**
		  * \name Node Scale Limits
		  */
		//@{

		/** Change the scaling limit active flag. If this flag is set to \c false,
		  * the values in the mScalinLimits have to be ignored.
		  * \param pActive State of the scaling limits active flag.
		  */
		void SetScalingLimitActive(bool pActive);

		/** Get the scaling limit active flag.
		  * \return Scaling limit active flag state. If this flag is set to \c false,
		  * the values in the mScalingLimits have to be ignored.
		  */
		bool GetScalingLimitActive() const;

		// The scaling limits.
		KFbxLimits mScalingLimits;

		//@}

	private:
		// Node on which the limit are applied
		KFbxNode*			  mLimitedNode;
		// Translation 
		bool                  mTActive;
		// Rotation
		bool                  mRActive;
		// Scaling		
		bool                  mSActive;
};

typedef KFbxLimits* HKFbxLimits;

} // namespace
#endif // #ifndef _KFbxNodeLimits_h