#ifndef _KFbxNurb_h
#define _KFbxNurb_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxplugins/kfbxgeometry.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxPatch;
class KFbxSdkManager;


/** A nurb is a type of parametric geometry.
  * \nosubgrouping
  */
class KFBX_DLL KFbxNurb : public KFbxGeometry 
{

public:

	//! Return the type of node attribute which is EAttributeType::eNURB.
	virtual EAttributeType GetAttributeType();

	//! Reset the nurb to default values.
	void Reset();

	/**
	  * \name Nurb Properties
	  */
	//@{

    /** Set surface mode.
	  * \param pMode Surface mode identifier (see class KfbxGeometry)
	  */
    void SetSurfaceMode(KFbxGeometry::ESurfaceMode pMode);

    /** Get surface mode.
	  * \return Currently set surface mode identifier.
	  */
    inline ESurfaceMode GetSurfaceMode() {return mSurfaceMode;}

	/** \enum ENurbType Nurb types.
	  * - \e ePERIODIC
	  * - \e eCLOSED
	  * - \e eOPEN
	  */
    typedef enum 
    {
	    ePERIODIC, 
	    eCLOSED, 
	    eOPEN 
    } ENurbType;

	/** Allocate memory space for the array of control points as well as the knot 
	  * and multiplicity vectors.
      * \param pUCount Number of control points in U direction.
      * \param pUType Nurb type in U direction.
      * \param pVCount Number of control points in V direction.
      * \param pVType Nurb type in V direction.
	  * \remarks This function should always be called after KFbxNurb::SetOrder(). 
      */
	void InitControlPoints(int pUCount, ENurbType pUType, int pVCount, ENurbType pVType);

	/** Get number of control points in U direction.
	  * \return Number of control points in U.
	  */
	inline int GetUCount() {return mUCount;}
	
	/** Get number of control points in V direction.
	  * \return Number of control points in V.
	  */
	inline int GetVCount() {return mVCount;}
	
	/** Get nurb type in U direction.
	  * \return Nurb type identifier.
	  */
	inline ENurbType GetNurbUType() {return mUType;}
	
	/** Get nurb type in V direction.
	  * \return Nurb type identifier.
	  */
	inline ENurbType GetNurbVType() {return mVType;}

	/** Get the number of elements in the knot vector in U direction.
	  * \return The number of control points in U direction. Nurb order in U 
	  * direction minus 1 is added if nurb type in U direction is set to
	  * ePERIODIC or eCLOSED.
	  */
	int GetUKnotCount();

	/** Get knot vector in U direction.
	  * \return Pointer to the array of control points.
	  */
	double* GetUKnotVector();
	
	/** Get the number of elements in the knot vector in V direction.
	  * \returns The number of control points in V direction. Nurb order in V 
	  * direction minus 1 is added if nurb type in V direction is set to
	  * ePERIODIC or eCLOSED.
	  */
	int GetVKnotCount();

	/** Get knot vector in V direction.
	  * \return Pointer to the array of control points.
	  */
	double* GetVKnotVector();
	
	/** Get multiplicity control points in U direction.
	  * \return Pointer to the array of multiplicity values.
	  * \remarks The length of this vector is equal to U count and
	  * its elements are set to 1 by default.
	  */
	int* GetUMultiplicityVector();
	
	/** Get multiplicity control points in V.
	  * \return Pointer to the array of multiplicity values.
	  * \remarks The length of this vector is equal to V count and
	  * its elements are set to 1 by default.
	  */
	int* GetVMultiplicityVector();

	/** Set order.
      * \param pUOrder Nurb order in U direction.
      * \param pVOrder Nurb order in V direction.
      */
	void SetOrder(kUInt pUOrder, kUInt pVOrder);

	/** Get nurb order in U direction.
	  * \return U order value.
	  */
	inline int GetUOrder() {return mUOrder;}
	
	/** Get nurb order in V direction.
	  * \return V order value.
	  */
	inline int GetVOrder() {return mVOrder;}

	/** Set step.
	  * The step is the number of divisions between adjacent control points.
      * \param pUStep Steps in U direction.
      * \param pVStep Steps in V direction.
      */
	void SetStep(int pUStep, int pVStep);

	/** Get the number of divisions between adjacent control points in U direction.
	  * \return Step value in U direction.
	  */
	inline int GetUStep() {return mUStep;}
	
	/** Get the number of divisions between adjacent control points in V direction.
	  * \return Step value in V direction.
	  */
	inline int GetVStep() {return mVStep;}

	//@}

	/**
	  * \name Nurb Export Flags
	  */
	//@{

	/** Set the flag inducing UV flipping at export time.
	  * \param pFlag If \c true UV flipping will occur.
	  */
	void SetApplyFlipUV(bool pFlag);

	/** Get the flag inducing UV flipping at export time.
	  * \return Current state of the UV flip flag.
	  */
	bool GetApplyFlipUV();

	/** Set the flag inducing link flipping at export time.
	  * \param pFlag If \c true the links control points indices will be flipped.
	  */
	void SetApplyFlipLinks(bool pFlag);

	/** Get the flag inducing link flipping at export time.
	  * \return Current state of the link flip flag.
	  */
	bool GetApplyFlipLinks();

	/** Get flip flags state.
	  * \return \c true if we need to flip either the UV or the links.
	  */
	bool GetApplyFlip() { return GetApplyFlipUV() || GetApplyFlipLinks(); }

	//@}

	//! Assignment operator.
    KFbxNurb& operator=(KFbxNurb& pNurb);

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

protected:

	KFbxNurb(KFbxSdkManager& pManager);
	virtual ~KFbxNurb();

	KString		GetTypeName();
	KStringList	GetTypeFlags();

	kUInt mUOrder, mVOrder;
	int mUCount, mVCount;
	int mUStep, mVStep;
	ENurbType mUType, mVType;
	
	double* mUKnotVector;
	double* mVKnotVector;
	
	int* mUMultiplicityVector;
	int* mVMultiplicityVector;

    ESurfaceMode mSurfaceMode;

	// Export flags.
	bool mApplyFlipUV;
	bool mApplyFlipLinks;

    // Error identifiers, these are only used internally.
	typedef enum 
	{
		eNurbTypeUnknown,
        eWrongNumberOfControlPoint,
        eWeightTooSmall,
        eUMultiplicityVectorError,
        eVMultiplicityVectorError,
        eUKnotVectorError,
        eVKnotVectorError,
		eErrorCount
	} EError;

	friend class KFbxReaderFbx;
	friend class KFbxWriterFbx;
	friend class KFbxWriterFbx6;
	friend class KFbxGeometryConverter;
	friend class KFbxSdkManager;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

typedef KFbxNurb* HKFbxNurb;

} // namespace
#endif // #ifndef _KFbxNurb_h


