#ifndef _KFbxPose_h
#define _KFbxPose_h

/**************************************************************************************

 Copyright  2003 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxplugins/kfbxtakenodecontainer.h>

#include <klib/kstring.h>
#include <klib/karrayul.h>
#include <kfbxmath/kfbxmatrix.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

/** This structure contains the description of a named pose.
  *
  */
typedef struct
{
	KString		mNodeName;		//! Identifier
	KFbxMatrix  mMatrix;		//! Transform matrix
	bool		mMatrixIsLocal; //! If true, the transform matrix above is defined in local coordinates.

} KFbxPoseInfo;

class KFbxScene;
class KFbxPoseProperties;

/** This class contains the description of a Pose manager.
  * \nosubgrouping
  * The KFbxPose object can be setup to hold "Bind Pose" data or "Rest Pose" data.
  *
  * The Bind Pose holds the transformation (translation, rotation and scaling) 
  * matrix of all the nodes implied in a link deformation. This includes the geometry
  * being deformed, the links deforming the geometry, and recursively all the
  * ancestors nodes of the link. The Bind Pose gives you the transformation of the nodes
  * at the moment of the binding operation when no deformation occurs.
  *
  * The Rest Pose is a snapshot of a node transformation. A Rest Pose can be used
  * to store the position of every node of a character at a certain point in
  * time. This pose can then be used as a reference position for animation tasks,
  * like editing walk cycles.
  *
  * One difference between the two modes is in the validation performed before
  * adding an item and the kind of matrix stored.
  * 
  * \par
  * In "Bind Pose" mode, the matrix is assumed to be defined in the global space,
  * while in "Rest Pose" the type of the matrix may be specified by the caller. So 
  * local system matrices can be used. Actually, because there is one such flag for
  * each entry (KFbxPoseInfo), it is possible to have mixed types in a KFbxPose elements.
  * It is therefore the responsability of the caller to check for the type of the retrieved
  * matrix and to do the appropriate conversions if required.
  *
  * \par
  * The validation of the data to be added consists of the following steps:
  *
  * \par
  *  1)	If this KFbxPose object stores "Bind Poses", then
  *     add a KFbxPoseInfo only if the pNodeName is not already
  *	    associated to another "Bind Pose". This check is done
  *     by visiting ALL the KFbxPose objects in the system. 
  *
  * \par
  *  The above test is only performed for the "Bind Pose" type. While
  *  the next one is always performed, no matter what kind of poses this
  *  KFbxPose object is setup to hold.
  *
  * \par
  *  2) If pNodeName is already inserted in the KFbxPose internal list,
  *	    then the passed matrix MUST be equal to the one already stored.
  *     If this is not the case, the Add method will return -1, indicating
  *     that no new KFbxPoseInfo has been created. 
  *
  * \par
  * If the Add method succeeds, it will return the index of the KFbxPoseInfo
  * structure that as been created and held by the KFbxPose object.
  *
  * \par
  * To ensure data integrity, the stored information can only be
  *	accessed using the provided methods (read-only). If an entry needs to be
  * modified, the caller has to remove the KFbxPoseInfo item by calling Remove(i)
  * and then Add a new one.
  *
  * \par
  * The internal list is not ordered and the search inside this is list is linear
  * (from the first element to ... the first match or the end of the list).
  *
  */
class KFBX_DLL KFbxPose : public KFbxTakeNodeContainer
{
public:
		/** Pose identifier flag.
		  * \return \c true if this object holds BindPose data.
		  */
		  bool        IsBindPose()     { return mType == 'b'; }

		/** Pose identifier flag.
		  * \return \c true if this object holds RestPose data.
		  */
	      bool        IsRestPose()     { return mType == 'r'; }

		 /** Get number of stored items.
		   * \return The number of items stored.
		   */
		  int		  GetCount() const { return mPoseInfo.GetCount(); }

		 /** Stores the pose transformation for the given node.
		   * \param pNodeName Name of the node for which the pose is stored.
		   * \param pMatrix Pose transform of the node.
		   * \param pLocalMatrix Flag to indicate if pMatrix is defined in Local or Global space.
		   * \return -1 if the function failed or the index of the stored item.
		   */
      	  int          Add(char* pNodeName, KFbxMatrix& pMatrix, bool pLocalMatrix = false);

         /** Remove the pIndexth item from the Pose object.
		   * \param pIndex Index of the item to be removed.
		   */
		  void          Remove(int pIndex);

		/** Get the node name.
		  * \param pIndex Index of the queried item.
		  * \return A pointer to the node name referenced.
		  * \remark If the index is invalid a pointer to an empty string is returned.
		  * \remark The returned pointer will become undefined if the KFbxPose object is destroyed.
		  */
		  char* GetNodeName(int pIndex)	const;

		/** Change the node name.
		  * \param pNewName New name.
		  * \param pIndex Index of the item to modify.
		  * \remark Changing the node name may seriously damage the pose consistency with
		  * the scene. It is the responsability of the caller to make sure that the new name
		  * corresponds to an existing node in the scene.
		  */
		  void ReplaceNodeName(int pIndex, char* pNewName);

	    /** Get the transform matrix.
		  * \param pIndex Index of the queried item.
		  * \return A reference to the pose matrix.
		  * \remark If the index is invalid a reference to an identiy matrix is returned.
		  * \remark The reference will become undefined if the KFbxPose object is destroyed.
		  */
    const KFbxMatrix&   GetMatrix(int pIndex)		const;

		/** Get the type of the matrix.
		  * \param pIndex Index of the queried item.
		  * \return \c true if the matrix is defined in the Local coordinate space and false otherwise.
		  * \remark If the KFbxPose object is configured to hold BindPose data, this method will always return \c false.
		  */
		  bool		    IsLocalMatrix(int pIndex);

		//@}
		/** 
		  * \name Search Section
		  */
		//@{

		/** Look in the KFbxPose object for the given node.
		  * \param pNodeName Name of the node we are looking for.
		  * \return -1 if the node is not in the list. Otherwise, the index of the
		  * corresponding KFbxPoseInfo element.
		  */
		  int Find(char* pNodeName);


		//@}
		/** 
		  * \name Utility Section
		  */
		//@{

		/** Get the list of Poses objects that contains the node with name pNodeName.
		  * \param pManager Is the manager owning the poses.
		  * \param pNodeName Name of the node we are exploring.
		  * \param pPosesList List of Bind/RestPoses that have the node.
		  * \param pIndexList Indices of the node in the corresponding poses lists.
		  * \return \c true if the node belongs to at least one Pose (either a BindPose or a RestPose).
		  * \remark The pPosesList and pIndexList are filled by this method.
		  * \remark The elements of the returned list must not be deleted since they still belong to the scene.
		  */
		static bool GetPosesContaining(KFbxSdkManager& pManager, char* pNodeName, 
									   KArrayTemplate<KFbxPose*>& pPoseList,
									   KArrayTemplate<int>& pIndex);

		/** Get the list of BindPose objects that contains the node with name pNodeName.
		  * \param pManager Is the manager owning the poses.
		  * \param pNodeName Name of the node we are exploring.
		  * \param pPosesList List of BindPoses that have the node.
		  * \param pIndexList Indices of the node in the corresponding poses lists.
		  * \return \c true if the node belongs to at least one BindPose.
		  * \remark The pPosesList and pIndexList are filled by this method.
		  * \remark The elements of the returned list must not be deleted since they still belong to the scene.
		  */
		static bool GetBindPoseContaining(KFbxSdkManager& pManager, char* pNodeName, 
										  KArrayTemplate<KFbxPose*>& pPoseList,
										  KArrayTemplate<int>& pIndex);

		/** Get the list of RestPose objects that contains the node with name pNodeName.
		  * \param pManager Is the manager owning the poses.
		  * \param pNodeName Name of the node we are exploring.
		  * \param pPosesList List of RestPoses that have the node.
		  * \param pIndexList Indices of the node in the corresponding poses lists.
		  * \return \c true if the node belongs to at least one RestPose.
		  * \remark The pPosesList and pIndexList are filled by this method.
		  * \remark The elements of the returned list must not be deleted since they still belong to the scene.
		  */
		static bool GetRestPoseContaining(KFbxSdkManager& pManager, char* pNodeName, 
										  KArrayTemplate<KFbxPose*>& pPoseList,
										  KArrayTemplate<int>& pIndex);


		/**
		  * \name Error Management
		  */
		//@{

		/** Retrieve error object.
		  *	\return Reference to error object.
		  */
		KError& GetError();

		/** \enum EError Error identifiers.
		  * - \e eERROR
		  * - \e eERROR_COUNT
		  */
		typedef enum 
		{
			eERROR,
			eERROR_COUNT
		} EError;	

		/** Get last error code.
		  *	\return Last error code.
		  */
		EError GetLastErrorID();

		/** Get last error string.
		  *	\return Textual description of the last error.
		  */
		char* GetLastErrorString();

		//@}

		/** Return the type ID of this class.
		  * \return Return KFbxObject::eGENERIC_NODE.
		  */
		virtual KFbxObject::ENameSpace GetNameSpace() const;


///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

protected:
	/** Create an object of type KFbxPose.
	  * \param pManager (need to exist for extended validation).
	  * \param pName Name of this KFbxPose.
	  * \param pIsBindPose If true, this object holds BindPose data.
	  */
      KFbxPose(KFbxSdkManager& pManager, KString& pName, bool pIsBindPose = false);

	/** Deletes this object. 
	  * All the references to NodeName and Matrix will become invalid.
	  */
      ~KFbxPose();

	// From KFbxObject
	void        Destruct();
	KString		GetTypeName();
	KStringList	GetTypeFlags();

	// Properties Handler
	KFbxPoseProperties* mPH;

	KError mError;

	// From KFbxTakeNodeContainer
	virtual void AddChannels(KFbxTakeNode *pTakeNode);

	virtual KObject*          GetPropertyHandler();
	virtual KFbxUserProperty* CreateKFbxUserProperty(KProperty* pProperty);

	//! Assignment operator.
    KFbxPose& operator=(KFbxPose& pPose);

	// returns false if pNodeName is already inserted in the list and the current matrix
	// is different from the stored one. Also, if this pose is a rest pose, check if
	// pNodeName belongs to other BindPoses (accessed through the scene pointer).
	// pos will contains the index of the KFbxPoseInfo if the parameters are already
	// stored in this object.
	bool ValidateParams(KString& pNodeName, KFbxMatrix& pMatrix, int& pPos);

	// Check only on this object's list.
	bool LocalValidateParams(KString& pNodeName, KFbxMatrix& pMatrix, int& pPos);

	friend class KFbxReaderFbx;
	friend class KFbxSdkManager;
	friend class KFbxWriterFbx6;

private:
	// don't give public access to the info otherwise we will loose the ability
	// to maintain data integrity.
    KFbxPoseInfo* GetItem(int pIndex);

private:
	char		mType;

	KArrayTemplate<KFbxPoseInfo*> mPoseInfo;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS 

};

typedef KFbxPose* HKFbxPose;

} // namespace
#endif // #ifndef _KFbxPose_h


