#ifndef _KFbxUserProperty_h
#define _KFbxUserProperty_h

/**************************************************************************************

 Copyright  2004 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

class KProperty;

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxColor;
class KFbxPropertyHandler;
class KFbxSdkManager;
class PropertyDataStorage;

/** Class to hold user properties.
  * \nosubgrouping
  */
class KFBX_DLL KFbxUserProperty
{
  public:

	/**
	  * \name Property Identification.
	  */
	//@{

	/** \enum EUserPropertyType Property types.
	  * - \e eUNIDENTIFIED	The property holds an unrecognized data type.
	  * - \e eBOOL			The property holds a \c true or \c false value.
	  * - \e eREAL			The property holds a floating point number value.
	  * - \e eCOLOR			The property holds a KFbxColor data type.
	  * - \e eINTEGER		The property holds an integer number value.
	  * - \e eVECTOR		The property holds three floating point number values.
	  * - \e eLIST          The property holds a list.
	  */
	typedef enum 
	{   
		eUNIDENTIFIED,
		eBOOL,
		eREAL,
		eCOLOR,
		eINTEGER,
		eVECTOR,
		eLIST
	} EUserPropertyType;

	/** Get the property internal name.
	  * \return Property internal name string.
	  */
    KString				GetName();

	/** Get the property label.
	  * \param pReturnNameIfEmpty If \c true, allow this method to return the internal name.
	  * \return The property label if set. Or the property internal name if the pReturnNameIfEmpty
	  * flag is set to \c true and the label has not been defined.
	  * \remarks Some applications may choose to ignore the label field and work uniquely with the
	  * internal name. Therefore, it should not be taken for granted that a label exists. Also, remember
      * that the label does not get saved in the FBX file. It only exist while the property object is
      * in memory.
	  */
	KString				GetLabel(bool pReturnNameIfEmpty=true);

	/** Get the property type.
	  * \return The property type identifier.
	  */
	EUserPropertyType	GetPropertyType();

	/** Get the property type in a human readable format.
	  * \param pType Property type identifier.
	  * \return Pointer to the property type string.
	  */
	static char*		GetPropertyTypeName(EUserPropertyType pType);

	/** Set a label to the property.
	  * \param pLabel Label string.
	  */
	void				SetLabel(KString pLabel);
	//@}

	/**
	  * \name Property Initialization.
	  */
	//@{

	/** \enum EFlags Property attribute flags.
	  * - \e eNO_FLAG      No special attributes set.
	  * - \e eANIMATABLE   An animation FCurve can be attached to this property.
	  * - \e eUSER		   User defined property.
	  */
	typedef enum
	{
		eNO_FLAG    = 0,
		eANIMATABLE = 1, 
		eUSER       = 1<<1,
		eTEMPORARY  = 1<<2  // System property, unknown by the SDK
	} EFlags;

	/** Change the attributes of the property.
	  * \param pFlag Property attribute identifier.
	  * \param pValue New state.
	  */
	void				ModifyFlag(EFlags pFlag, bool pValue);

	/** Get the property attribute state.
	  * \param pFlag Property attribute identifier.
	  * \return The currently set property attribute state.
	  */
	bool                GetFlag(EFlags pFlag);
	//@}

	/**
	  * \name Property Value.
	  */
	//@{

	/** Set the property value. 
	  * \param pValue Boolean property value.
	  */
	void				SetDefaultValue(bool pValue);

	/** Set the property value. 
	  * \param pValue Real property value.
	  */
	void				SetDefaultValue(double pValue);

	/** Set the property value. 
	  * \param pValue KFbxColor property value.
	  */
	void				SetDefaultValue(KFbxColor& pValue);

	/** Set the property value. 
	  * \param pValue Integer property value for \c eINTEGER and \c eLIST properties.
	  */
	void				SetDefaultValue(int pValue);

	/** Set the property value. 
	  * \param pValue1 First component of vector property value.
	  * \param pValue2 Second component of vector property value.
	  * \param pValue3 Third component of vector property value.
	  */
	void				SetDefaultValue(double pValue1, double pValue2, double pValue3);

	/** Add a string value at the end of the list.
	  * \param pStringValue Value of the string to be added.
	  * \return The index in the list where the string was added.
	  * \remarks This function is only valid when the property type is \c eLIST.
	  * Empty strings are not allowed.
	  */
	int                 AddListValueString(KString& pStringValue);

	/** Insert a string value at the specified index.
	  * \param pIndex Zero bound index.
	  * \param pStringValue Value of the string for the specified index.
	  * \remarks This function is only valid when the property type is \c eLIST.
	  * \c pIndex must be in the range [0, ListValueGetCount()].
	  * Empty strings are not allowed.
	  */
	void                InsertListValueString(int pIndex, KString& pStringValue);

	/** Get the number of elements in the list.
	  * \return The number of elements in the list.
	  * \remarks This function will return \c -1 if the property type is not \c eLIST.
	  */
	int                 ListValueGetCount();

	/** Set a string value for the specified index.
	  * \param pIndex Zero bound index.
	  * \param pStringValue Value of the string for the specified index.
	  * \remarks This function is only valid when the property type is \c eLIST.
	  * The function will assign the specified string to the specified index.
	  * A string value must exists at the specified index in order to be changed.
	  * Empty strings are not allowed.
	  */
	void                SetListValueString(int pIndex, KString& pStringValue);

	/** Remove the string value at the specified index.
	  * \param pIndex of the string value to be removed.
	  */
	void                RemoveListValueString(int pIndex);

	/** Get the property value.
	  * \param pValue The currently set property value.
	  */
	void				GetDefaultValue(bool& pValue);

	/** Get the property value.
	  * \param pValue The currently set property value.
	  */
	void				GetDefaultValue(double& pValue);

	/** Get the property value.
	  * \param pValue The currently set property value.
	  */
	void				GetDefaultValue(KFbxColor& pValue);

	/** Get the property value.
	  * \param pValue The currently set property value for \c eINTEGER and \c eLIST properties.
	  */
	void				GetDefaultValue(int& pValue);

	/** Get the property value.
	  * \param pValue1 The currently set property value.
	  * \param pValue2 The currently set property value.
	  * \param pValue3 The currently set property value.
	  */
	void				GetDefaultValue(double& pValue1, double& pValue2, double& pValue3);

	/** Get a string value for the specified index
	  * \param pIndex Zero bound index.
	  * \param pStringValue Value of the string for the specified index.
	  * \remarks This function is only valid when the property type is \c eLIST.
	  */
	KString             GetListValueString(int pIndex, KString& pStringValue);
	//@}


	/**
	  * \name Property Limits.
	  * Property limits are provided for convenience if some applications desire to
	  * bound the range of possible values for a given type property. Note that these
	  * limits are meaningless for the boolean type. It is the responsibility of the
	  * calling application to implement the necessary instructions to limit the property.
	  */
	//@{
	/** Set the minimum limit value of the property.
	  * \param pMin Minimum value allowed.
	  */
	void				SetMinLimit(double pMin);

	/** Get the minimum limit value of the property.
	  * \return Currently set minimum limit value.
	  */
	double				GetMinLimit();

	/** Set the maximum limit value of the property.
	  * \param pMax Maximum value allowed.
	  */
	void				SetMaxLimit(double pMax);
	/** Get the maximum limit value of the property.
	  * \return Currently set maximum limit value.
	  */
	double				GetMaxLimit();

	/** Set the minimum and maximum limit value of the property.
	  * \param pMin Minimum value allowed.
	  * \param pMax Maximum value allowed.
	  */
	void				SetLimits(double pMin, double pMax);
	//@}


	//! Equality operator.
	bool                operator==(KProperty* pKProperty);

  protected:
///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

	//! Constructor.
	 /* The pLabel parameter is defined for backward compatibility. Older FBX files
        saved it along with the other attributes. Now, the label attribute exists only while
        the property object is in memory and is lost when saving to the FBX file.
	  */
	KFbxUserProperty(KFbxSdkManager& pManager, char* pName, char* pTypeName, char* pLabel);
	KFbxUserProperty(KFbxSdkManager& pManager, char* pName, EUserPropertyType pType, char* pLabel, EFlags pFlags);
	KFbxUserProperty(KFbxSdkManager& pManager, KProperty* pProperty);

    //! Destructor.
    ~KFbxUserProperty();

	//! Assignment operator.
    KFbxUserProperty& operator=(KFbxUserProperty& pUserProperty);

	//! Internal management
	void SetPropertyHandler(KFbxPropertyHandler* pPropertyHandler);

	friend class KFbxReaderFbx;
	friend class KFbxSdkManager;
	
	friend class KFbxPropertyHandler;
	friend class KFbxNode;
	friend class KFbxTakeNodeContainer;

    friend class KFbxCharacter;

  private:
	KProperty*           mKProperty;
	KFbxPropertyHandler* mPropertyHandler;
	PropertyDataStorage* mData;

	KFbxSdkManager&		mManager;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS 

};

typedef KFbxUserProperty* HKFbxUserProperty;

} // namespace
#endif // _KFbxUserProperty_h