#ifndef NX_PHYSICS_NXSCENEDESC
#define NX_PHYSICS_NXSCENEDESC
/*----------------------------------------------------------------------------*\
|
|						Public Interface to NovodeX Technology
|
|							     www.novodex.com
|
\*----------------------------------------------------------------------------*/
/** \addtogroup physics
  @{
*/

/**
\brief Used to specify the broadphase collision detection behavior.

<b>Platform:</b>
\li PC SW: Yes
\li PPU  : No
\li PS3  : Yes
\li XB360: Yes

@see NxSceneDesc
*/
enum NxBroadPhaseType
	{
	NX_BROADPHASE_QUADRATIC,	//!< No acceleration structure (O(n^2)) [debug purpose]
	NX_BROADPHASE_FULL,			//!< Brute-force algorithm, performing the full job each time (O(n))
	NX_BROADPHASE_COHERENT,		//!< Incremental algorithm, using temporal coherence (O(n))

	NX_BROADPHASE_FORCE_DWORD = 0x7fffffff
	};

/**
\brief Used to specify the timestepping behavior.

<b>Platform:</b>
\li PC SW: Yes
\li PPU  : No (does not support sub stepping)
\li PS3  : Yes
\li XB360: Yes

@see NxSceneDesc
*/
enum NxTimeStepMethod
	{
	NX_TIMESTEP_FIXED,				//!< The simulation automatically subdivides the passed elapsed time into maxTimeStep-sized substeps.
	NX_TIMESTEP_VARIABLE,			//!< The simulation uses the elapsed time that the user passes as-is, substeps (maxTimeStep, maxIter) are not used.
	NX_NUM_TIMESTEP_METHODS
	};

enum NxHwPipelineSpec
	{
	NX_HW_RB_PIPELINE_HLP_ONLY  = 0,
	NX_HW_PIPELINE_FULL         = 1,
	NX_HW_PIPELINE_DEBUG        = 2
	};

enum NxHwSceneType
	{
	NX_HW_SCENE_TYPE_RB				= 0,	//!< Specifies a rigid body hardware scene.
	NX_HW_SCENE_TYPE_FLUID			= 1,	//!< Specifies a fluid hardware scene.
	NX_HW_SCENE_TYPE_FLUID_SOFTWARE	= 2		//!< Specifies a scene, running the fluid software reference. 
	};

enum NxSimulationType
	{
	NX_SIMULATION_SW	= 0,
	NX_SIMULATION_HW	= 1
	};

class NxUserNotify;
class NxUserTriggerReport;
class NxUserContactReport;
class NxBounds3;

// PT: those limits are not complete yet. They should include a lot more information than this.
/**
\brief Class used to retrieve limits(eg max number of bodies) for a scene.

<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes
*/
class NxSceneLimits
	{
	public:
	NxU32					maxNbActors;		//!< Expected max number of actors
	NxU32					maxNbBodies;		//!< Expected max number of bodies
	NxU32					maxNbStaticShapes;	//!< Expected max number of static shapes
	NxU32					maxNbDynamicShapes;	//!< Expected max number of dynamic shapes
	NxU32					maxNbJoints;		//!< Expected max number of joints

	NX_INLINE NxSceneLimits();
	};

NX_INLINE NxSceneLimits::NxSceneLimits()	//constructor sets to default
	{
	maxNbActors			= 0;
	maxNbBodies			= 0;
	maxNbStaticShapes	= 0;
	maxNbDynamicShapes	= 0;
	maxNbJoints			= 0;
	}

/**
\brief Descriptor class for scenes. See #NxScene.

@see NxScene NxPhysicsSDK.createScene maxBounds
*/

class NxSceneDesc
	{
	public:
	/**
	\brief Broad phase algorithm

	<b>Default:</b> NX_BROADPHASE_COHERENT

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes

	@see NxBroadPhaseType
	*/
	NxBroadPhaseType		broadPhase;

	/**
	\brief Gravity vector

	<b>Range:</b> force vector<br>
	<b>Default:</b> Zero

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes

	@see NxScene.setGravity()
	*/
	NxVec3					gravity;

	/**
	\brief Possible notification callback

	<b>Default:</b> NULL

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes

	@see NxUserNotify NxScene.setUserNotify() NxScene.getUserNotify()
	*/
	NxUserNotify*			userNotify;

	/**
	\brief Possible trigger callback

	<b>Default:</b> NULL

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes

	@see NxUserTriggerReport NxScene.setUserTriggerReport() NxScene.getUserTriggerReport()
	*/
	NxUserTriggerReport*	userTriggerReport;

	/**
	\brief Possible contact callback

	<b>Default:</b> NULL

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes

	@see NxUserContactReport NxScene.setUserContactReport() NxScene.getUserContactReport()
	*/
	NxUserContactReport*	userContactReport;

	/**
	\brief Maximum integration time step size

	<b>Range:</b> (0,inf)<br>
	<b>Default:</b> 1.0/60.0

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes
	*/
	NxReal					maxTimestep;

	/**
	\brief Maximum number of substeps to take

	<b>Default:</b> 8

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes
	*/
	NxU32					maxIter;

	/**
	\brief Integration method.

	<b>Default:</b> NX_TIMESTEP_FIXED

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes

	@see NxTimeStepMethod
	*/
	NxTimeStepMethod		timeStepMethod;

	/**
	\brief Max scene bounds.

	If scene bounds are provided (maxBounds in the descriptor), the SDK takes advantage of this information
	to accelerate scene-level collision queries (e.g. raycasting).

	<b>Range:</b> See #NxBounds3<br>
	<b>Default:</b> NULL

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes
	*/
	NxBounds3*				maxBounds;

	/**
	\brief Expected scene limits (or NULL)

	<b>Default:</b> NULL

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes

	@see NxSceneLimits
	*/
	NxSceneLimits*			limits;

	/**
	\brief Used to specify if a hardware or software simulation is used.

	<b>Default:</b> NX_SIMULATION_SW

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes

	@see NxSimulationType
	*/
	NxU32					simType;
	
	/**
	\brief Used to specify the hardware scene type.

	<b>Default:</b> NX_HW_SCENE_TYPE_RB

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxHwSceneType
	*/
	NxU32					hwSceneType;

	/**
	\brief Pipline specification.

	<b>Default:</b> NX_HW_RB_PIPELINE_HW

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxHwRBPipelineSpec NxHWFluidPipelineSpec
	*/
	NxU32					pipelineSpec;

	/**
	\brief Enable/disable default ground plane

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes

	<b>Default:</b> false
	*/
	bool					groundPlane;

	/**
	\brief Enable/disable 6 planes around maxBounds (if available)

	<b>Default:</b> false

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes

	@see maxBounds
	*/
	bool					boundsPlanes;

	/**
	\brief Enable/disable collision detection

	<b>Default:</b> true

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes
	*/
	bool					collisionDetection;

	/**
	\brief Will be copied to NxScene::userData

	<b>Default:</b> NULL

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes
	*/
	void*					userData;

	/**
	\brief constructor sets to default (no gravity, no ground plane, collision detection on).
	*/
	NX_INLINE NxSceneDesc();
	/**
	\brief (re)sets the structure to the default (no gravity, no ground plane, collision detection on).	
	*/
	NX_INLINE void setToDefault();
	/**
	\brief Returns true if the descriptor is valid.

	\return true if the current settings are valid (returns always true).
	*/
	NX_INLINE bool isValid() const;
	};

NX_INLINE NxSceneDesc::NxSceneDesc()	//constructor sets to default
	{
	setToDefault();
	}

NX_INLINE void NxSceneDesc::setToDefault()
	{
	broadPhase			= NX_BROADPHASE_COHERENT;
	gravity.zero();
	userNotify			= NULL;
	userTriggerReport	= NULL;
	userContactReport	= NULL;

	maxTimestep			= 1.0f/60.0f;
	maxIter				= 8;
	timeStepMethod		= NX_TIMESTEP_FIXED;

	maxBounds			= NULL;
	limits				= NULL;

	simType				= NX_SIMULATION_SW;
	hwSceneType			= NX_HW_SCENE_TYPE_RB;
	pipelineSpec		= NX_HW_PIPELINE_FULL;

	groundPlane			= false;
	boundsPlanes		= false;
	collisionDetection	= true;
	userData			= NULL;
	}

NX_INLINE bool NxSceneDesc::isValid() const
	{
	if(maxTimestep <= 0 || maxIter < 1 || timeStepMethod > NX_NUM_TIMESTEP_METHODS)
		return false;
	if(boundsPlanes && !maxBounds)
		return false;
	return true;
	}

/** @} */
#endif


//AGCOPYRIGHTBEGIN
///////////////////////////////////////////////////////////////////////////
// Copyright  2005 AGEIA Technologies.
// All rights reserved. www.ageia.com
///////////////////////////////////////////////////////////////////////////
//AGCOPYRIGHTEND

