#ifndef NX_COLLISION_NXSHAPEDESC
#define NX_COLLISION_NXSHAPEDESC
/*----------------------------------------------------------------------------*\
|
|						Public Interface to NovodeX Technology
|
|							     www.novodex.com
|
\*----------------------------------------------------------------------------*/
/** \addtogroup physics
  @{
*/

#include "NxShape.h"

/**
\brief Descriptor for #NxShape class. 

Used for saving and loading the shape state.

See the derived classes for the different shape types.

@see NxActor.createShape() NxSphereShapeDesc NxPlaneShapeDesc NxConvexShapeDesc NxTriangleMeshShapeDesc
NxCapsuleShapeDesc NxBoxShapeDesc
*/
class NxShapeDesc
	{
	protected:
	
	/**
	\brief The type of the shape (see NxShape.h).
	
	This gets set by the derived class' ctor, the user should not have to change it.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes
	*/
	const NxShapeType		type;
	public:

	/**
	\brief The pose of the shape in the coordinate frame of the owning actor.

	<b>Range:</b> rigid body transform<br>
	<b>Default:</b> Identity

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes

	@see NxShape.setLocalPose
	*/
	NxMat34					localPose;

	/**
	\brief A combination of ::NxShapeFlag values.

	<b>Default:</b> NX_SF_VISUALIZATION

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes

	@see NxShape.setFlag() NxShapeFlag
	*/
	NxU32					shapeFlags;

	/**
	\brief See the documentation for NxShape::setGroup().

	<b>Default:</b> 0

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes

	@see NxShape.setGroup()
	*/
	NxCollisionGroup		group;

	/**
	\brief The material index of the shape.

	<b>Default:</b> 0

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes

	@see NxScene.createMaterial() NxShape.setMaterial()
	*/
	NxMaterialIndex			materialIndex;
	
	/**
	\brief ccd Skeleton

	<b>Default:</b> NULL

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes

	@see NxShape.setCCDSkeleton() NxPhysicsSDK.createCCDSkeleton()
	*/	
	NxCCDSkeleton*			ccdSkeleton;
	
	/**
	\brief density of this individual shape when computing mass inertial properties for a rigidbody (unless a valid mass >0.0 is provided).
	
	See #NxActorDesc for a description of how the density is applied.

	<b>Range:</b> (0,inf)<br>
	<b>Default:</b> 1.0

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes
	*/
	NxReal					density;

	/**
	\brief mass of this individual shape when computing mass inertial properties for a rigidbody.  When mass<=0.0 then density and volume determine the mass.

	<b>Range:</b> (0,inf)<br>
	<b>Default:</b> 1.0

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes
	*/
	NxReal					mass;
	
	/**
	\brief The shape has been radially inflated by this much over the graphical representation.

	Two shapes will interpenetrate by the sum of their skin widths; this means that their graphical representations should just touch.
	The default skin width is the negative global parameter ::NX_MIN_SEPARATION_FOR_PENALTY. This is used if skinWidth is -1. A sum skin width
	of	zero for two bodies is not permitted because it is unstable. If you simulation jitters because resting bodies occasionally lose contact, 
	increase the sizes of your collision volumes and the skin width.

	Units: distance. 
	
	<b>Range:</b> (0,inf)<br>
	<b>Default:</b> -1.0

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes

	@see NxParameter
	*/
	NxReal					skinWidth;


	/**
	\brief Will be copied to NxShape::userData.

	<b>Default:</b> NULL

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes
	*/
	void*					userData;

	/**
	\brief Possible debug name. The string is not copied by the SDK, only the pointer is stored.

	<b>Default:</b> NULL

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes
	*/
	const char*				name;

#ifdef NX_SUPPORT_NEW_FILTERING
	/**
	\brief Groups bitmask for collision filtering

	<b>Default:</b> 0

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes
	*/
	NxGroupsMask			groupsMask;
#endif
	NX_INLINE virtual		~NxShapeDesc();
	
	/**
	\brief (re)sets the structure to the default.	
	*/
	NX_INLINE virtual	void setToDefault();
	
	/**
	\brief Returns true if the descriptor is valid.
	
	\return true if the current settings are valid
	*/
	NX_INLINE virtual	bool isValid() const;

	/**
	\brief Retrieves the shape type.

	\return The type of the shape. See #NxShapeType.
	*/
	NX_INLINE			NxShapeType	getType()	const	{ return type; }

	protected:
	/**
	constructor sets to default.

	\param[in] shapeType The type of shape this desc is created for.
	*/
	NX_INLINE				NxShapeDesc(NxShapeType shapeType);
	};

NX_INLINE NxShapeDesc::NxShapeDesc(NxShapeType t) : type(t)
	{
	setToDefault();
	}

NX_INLINE NxShapeDesc::~NxShapeDesc()
	{
	}

NX_INLINE void NxShapeDesc::setToDefault()
	{
	localPose.id();
	shapeFlags			= NX_SF_VISUALIZATION;
	group				= 0;
	materialIndex		= 0;
	ccdSkeleton		= NULL;
	skinWidth			= -1.0f;
	density				=  1.0f;
	mass				= -1.0f;  // by default we let the mass be determined by its density.  
	userData			= NULL;
	name				= NULL;
#ifdef NX_SUPPORT_NEW_FILTERING
	groupsMask.bits0	= 0;
	groupsMask.bits1	= 0;
	groupsMask.bits2	= 0;
	groupsMask.bits3	= 0;
#endif
	}

NX_INLINE bool NxShapeDesc::isValid() const
	{
	if(!localPose.isFinite())
		return false;
	if(group>=32)
		return false;	// We only support 32 different groups
	if(shapeFlags&0xffff0000)
		return false;	// Only 16-bit flags supported here
	if(type >= NX_SHAPE_COUNT)
		return false;
	if(materialIndex==0xffff)
		return false;	// 0xffff is reserved for internal usage
	if (skinWidth != -1 && skinWidth < 0)
		return false;
	return true;
	}

/** @} */
#endif


//AGCOPYRIGHTBEGIN
///////////////////////////////////////////////////////////////////////////
// Copyright  2005 AGEIA Technologies.
// All rights reserved. www.ageia.com
///////////////////////////////////////////////////////////////////////////
//AGCOPYRIGHTEND

