#ifndef NX_PHYSICS_NX_FLUIDACTOR
#define NX_PHYSICS_NX_FLUIDACTOR
/** \addtogroup fluids
  @{
*/
/*----------------------------------------------------------------------------*\
|
|						Public Interface to NovodeX Technology
|
|							     www.novodex.com
|
\*----------------------------------------------------------------------------*/
#include "Nxp.h"
#include "NxArray.h"
#include "NxBounds3.h"
#include "NxFluidDesc.h"
#include "NxPhysicsSDK.h"

class NxFluidEmitterDesc;
class NxFluidEmitter;

/**
\brief The fluid class represents the main module for the particle based fluid simulation.
SPH (Smoothed Particle Hydrodynamics) is used to animate the particles.

There are two kinds of particle interaction forces which govern the behaviour of the fluid:
<ol>
<li>
	Pressure forces: These forces result from particle densities higher than the
	"rest density" of the fluid.  The rest density is given by specifying the inter-particle
	distance at which the fluid is in its relaxed state.  Particles which are closer than
	the rest spacing are pushed away from each other.
<li>
	Viscosity forces:  These forces act on neighboring particles depending on the difference
	of their velocities.  Particles drag other particles with them which is used to simulate the
	viscous behaviour of the fluid.
</ol>
The fluid class manages a set of particles.
Particles can be created in two ways:
<ol>
<li>
	Particles can be added by the user directly.
<li>
	The user can add emitters to the fluid and configure the parameters of the emission.
	(See NxFluidEmitter)
</ol>
Particles can be removed in two ways as well:
<ol>
<li>
	The user can specify a lifetime for the particles.  When its lifetime expires, a particle is deleted.
<li>
	Shapes can be configured to act as drains.  When a particle intersects with a drain, the particle is deleted.
	(See NxShapeFlag)
</ol>
Particles collide with static and dynamic shapes.  The user can enable two-way interaction as well.
If he does, colliding particles exert impulses onto dynamic rigid bodies.  Particles are also
affected by the scene gravity and a user force, as well as global velocity damping.  In order to
render a fluid, the user can supply the fluid instance with a user buffer into which the particle
state is written after each simuation step.  Alternatively the user can configure the fluid to generate
a mesh surface from the particles (into an NxImplicitMesh).

For a good introduction to SPH fluid simulation,
see http://graphics.ethz.ch/~mattmuel/publications/sca03.pdf

\warning <b>Preliminary API, subject to change</b>

@see NxFluidDesc, NxFluidEmitter, NxFluidEmitterDesc, NxImplicitMesh, NxImplicitMeshDesc, NxMeshData, NxParticleData, NxShapeFlag
*/
class NxFluid
	{
	protected:
	NX_INLINE					NxFluid() : userData(NULL)	{}
	virtual						~NxFluid()	{}

	public:

/************************************************************************************************/

/** @name Emitters
*/
//@{

	/**
	\brief Creates an emitter for this fluid.
	
	NxFluidEmitterDesc::isValid() must return true.

	\warning <b>Preliminary API, subject to change</b>

	\param desc The fluid emitter desciptor. See #NxFluidEmitterDesc.
	\return The new fluid.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidEmitter
	*/
	virtual		NxFluidEmitter*		createEmitter(const NxFluidEmitterDesc& desc)				= 0;

	/**
	\brief Deletes the specified emitter.
	
	The emitter must belong to this fluid. Do not keep a reference to the deleted instance.

	\warning <b>Preliminary API, subject to change</b>

	\param emitter The emitter to release.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No
	*/
	virtual		void				releaseEmitter(NxFluidEmitter& emitter)							= 0;

	/**
	\brief Returns the number of emitters.

	\warning <b>Preliminary API, subject to change</b>

	\return The number of emitters.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No
	*/
	virtual		NxU32				getNbEmitters()									const	= 0;

	/**
	\brief Returns an array of emitter pointers with size getNbEmitters().

	\warning <b>Preliminary API, subject to change</b>

	\return An array of fluid emitter pointers.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No
	*/
	virtual		NxFluidEmitter**	getEmitters()									const	= 0;
//@}
/************************************************************************************************/

/** @name Implicit Mesh
*/
//@{

	/**
	\brief Creates a surface mesh for this fluid.
	
	NxImplicitMeshDesc::isValid() must return true. If a surface mesh already exists for this fluid, it's released.

	\warning <b>Preliminary API, subject to change</b>

	\param mesh The mesh descriptor. See #NxImplicitMeshDesc.
	\return The new implicit mesh.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No (not supported)
	\li PS3  : No
	\li XB360: No

	@see NxImplicitMesh
	*/
	virtual		NxImplicitMesh*		createSurfaceMesh(const NxImplicitMeshDesc& mesh)			= 0;

	/**
	\brief Deletes the surface mesh.

	\warning <b>Preliminary API, subject to change</b>

	Do not keep a reference to the deleted instance.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No (not supported)
	\li PS3  : No
	\li XB360: No
	*/
	virtual		void				releaseSurfaceMesh()									= 0;

	/**
	\warning <b>Preliminary API, subject to change</b>
	\brief Returns a pointer to the surface mesh of the fluid

	\return The surface mesh associated with the fluid.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No (not supported)
	\li PS3  : No
	\li XB360: No
	*/
	virtual		NxImplicitMesh*		getSurfaceMesh()								const	= 0;
//@}
/************************************************************************************************/

/** @name Particle Manipulation
*/
//@{

	/**
	\brief Adds particles to the simulation specified with the user buffer wrapper.
	
	The SDK only accesses the wrapped buffers until the function returns.

	\warning <b>Preliminary API, subject to change</b>

	\param pData Structure describing the particles to add.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes (max 671 new Particles per timestep)
	\li PS3  : No
	\li XB360: No

	@see NxParticleData
	*/
	virtual		void 				addParticles(const NxParticleData& pData)						= 0;

 	/**
	\brief Sets the wrapper for user buffers, which configure where particle data is written to.

	\warning <b>Preliminary API, subject to change</b>

	\param pData The descriptor for the buffers to write the particle data to.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxParticleData
	*/
	virtual		void 				setParticlesWriteData(const NxParticleData& pData)			= 0;

 	/**
	\brief Returns a copy of the wrapper which was set by setParticlesWriteData().

	\warning <b>Preliminary API, subject to change</b>

	\return The particle write data.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxParticleData  The descriptor for the buffers to write the particle data to.
	*/
	virtual		NxParticleData 		getParticlesWriteData()							const	= 0;
//@}
/************************************************************************************************/

/** @name Fluid Parameters
*/
//@{

	/**
	\brief Returns the fluid stiffness.

	\warning <b>Preliminary API, subject to change</b>

	\return The fluid stiffness.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.stiffness
	*/
	virtual		NxReal				getStiffness()									const	= 0;

	/**
	\brief Sets the fluid stiffness (must be positive).

	\warning <b>Preliminary API, subject to change</b>

	\param stiff The new fluid stiffness.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.stiffness
	*/
	virtual		void 				setStiffness(NxReal stiff)									= 0;

	/**
	\brief Returns the fluid viscosity.

	\warning <b>Preliminary API, subject to change</b>

	\return The viscosity  of the fluid.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.viscosity
	*/
	virtual		NxReal				getViscosity()									const	= 0;

	/**
	\brief Sets the fluid viscosity (must be positive).

	\warning <b>Preliminary API, subject to change</b>

	\param visc The new viscosity of the fluid.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.viscosity
	*/
	virtual		void 				setViscosity(NxReal visc)									= 0;

	/**
	\brief Returns the fluid damping.

	\warning <b>Preliminary API, subject to change</b>

	\return The fluid damping.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.damping
	*/
	virtual		NxReal				getDamping()									const	= 0;

	/**
	\brief Sets the fluid damping (must be nonnegative).

	\warning <b>Preliminary API, subject to change</b>

	\param damp The new fluid damping.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.damping
	*/
	virtual		void 				setDamping(NxReal damp)										= 0;

	/**
	\brief Returns the external force applied to each particle at each time step.

	\warning <b>Preliminary API, subject to change</b>

	\return The external force aaplied to particles.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.externalForce
	*/
	virtual		NxVec3				getExternalForce()								const	= 0;

	/**
	\brief Sets the external force applied to each particle at each time step.

	\warning <b>Preliminary API, subject to change</b>

	\param force External force to apply to particles.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.externalForce
	*/
	virtual		void 				setExternalForce(const NxVec3&force)							= 0;
//@}
/************************************************************************************************/

/** @name Collisions
*/
//@{

	/**
	\brief Returns the fluid collision restitution used for collision with static actors.

	\warning <b>Preliminary API, subject to change</b>

	\return The static collision restitution.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.staticCollisionRestitution
	*/
	virtual		NxReal				getStaticCollisionRestitution()					const	= 0;

	/**
	\brief Sets the fluid collision restitution used for collision with static actors.
	
	Must be between 0 and 1.

	\warning <b>Preliminary API, subject to change</b>

	\param rest The new static collision restitution.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.staticCollisionRestitution
	*/
	virtual		void 				setStaticCollisionRestitution(NxReal rest)					= 0;

	/**
	\brief Returns the fluid collision adhesion (friction) used for collision with static actors.

	\warning <b>Preliminary API, subject to change</b>

	\return The static collision adhesion.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.staticCollisionAdhesion
	*/
	virtual		NxReal				getStaticCollisionAdhesion()					const	= 0;

	/**
	\brief Sets the fluid collision adhesion (friction) used for collision with static actors.
	
	Must be between 0 and 1.

	\warning <b>Preliminary API, subject to change</b>

	\param adhesion The new static collision adhesion

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.staticCollisionAdhesion
	*/
	virtual		void 				setStaticCollisionAdhesion(NxReal adhesion)						= 0;

	/**
	\brief Returns the fluid collision restitution used for collision with dynamic actors.

	\warning <b>Preliminary API, subject to change</b>

	\return The dynamic collision resitution.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.dynamicCollisionRestitution
	*/
	virtual		NxReal				getDynamicCollisionRestitution()				const	= 0;

	/**
	\brief Sets the fluid collision restitution used for collision with dynamic actors.
	
	Must be between 0 and 1.

	\warning <b>Preliminary API, subject to change</b>

	\param rest The new dynamic collision restitution.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.dynamicCollisionRestitution
	*/
	virtual		void 				setDynamicCollisionRestitution(NxReal rest)					= 0;

	/**
	\brief Returns the fluid collision adhesion (friction) used for collision with dynamic actors.

	\warning <b>Preliminary API, subject to change</b>

	\return The dynamic collision adhesion.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.dynamicCollisionAdhesion
	*/
	virtual		NxReal				getDynamicCollisionAdhesion()					const	= 0;

	/**
	\brief Sets the fluid collision adhesion (friction) used for collision with dynamic actors.
	
	Must be between 0 and 1.

	\warning <b>Preliminary API, subject to change</b>
	
	\param adhesion The new dynamic collision adhesion.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.dynamicCollisionAdhesion
	*/
	virtual		void 				setDynamicCollisionAdhesion(NxReal adhesion)						= 0;

	/**
	\brief Returns the fluid's factor for smoothing impulse transfer used for collision with dynamic actors.

	\warning <b>Preliminary API, subject to change</b>

	\return The reaction smoothing.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No (not supported)
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.dynamicActorReactionSmoothing
	*/
	virtual		NxReal				getDynamicActorReactionSmoothing()				const	= 0;

	/**
	\brief Sets the fluid's factor for smoothing impulse transfer used for collision with dynamic actors.
	
	Must be between 0 and 1.

	\warning <b>Preliminary API, subject to change</b>

	\param smooth The new reaction smoothing.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No (not supported)
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.dynamicActorReactionSmoothing
	*/
	virtual		void 				setDynamicActorReactionSmoothing(NxReal smooth)				= 0;

	/**
	\warning <b>Preliminary API, subject to change</b>
	\brief Returns the fluid's factor for scaling impulse transfer used for collision with dynamic actors.
	
	\return The reaction scaling.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No (not supported)
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.dynamicActorReactionScaling
	*/
	virtual		NxReal				getDynamicActorReactionScaling()				const	= 0;

	/**
	\brief Sets the fluid's factor for scaling impulse transfer used for collision with dynamic actors.

	\warning <b>Preliminary API, subject to change</b>

	\param scale The new reaction scaling.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No (not supported)
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.dynamicActorReactionScaling
	*/
	virtual		void 				setDynamicActorReactionScaling(NxReal scale)				= 0;
//@}
/************************************************************************************************/


	/**
	\brief Sets actor flags.

	\warning <b>Preliminary API, subject to change</b>

	\param flag Member of #NxFluidFlag.
	\param val New flag value.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Partial (NX_FF_MESH_INCLUDE_SEPARATED not supported)
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.flags
	*/
	virtual		void				setFlag(NxFluidFlag flag, bool val)								= 0;

	/**
	\brief Returns actor flags.

	\warning <b>Preliminary API, subject to change</b>

	\param flag Member of #NxFluidFlag.
	\return The current flag value.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.flags
	*/
	virtual		NX_BOOL				getFlag(NxFluidFlag flag)							const	= 0;

	/**
	\brief Sets the fluids collision group, a shared group space with up to 32 groups (5 bit):
	
	Note that shape groups use the same address space.

	\warning <b>Preliminary API, subject to change</b>

	\param group The new fluid collision group.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.collisionGroup
	*/
	virtual		void				setCollisionGroup(NxCollisionGroup group)				= 0;

	/**
	\brief Returns the collision group set with setCollisionGroup(NxCollisionGroup group)

	\warning <b>Preliminary API, subject to change</b>

	\return The fluid collision group.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.collisionGroup
	*/
	virtual		NxU32				getCollisionGroup()								const	= 0;

#if NX_USE_FLUID_API

	/**
	\brief Sets the fluid group, a group space which is independent from the actor group space, (16 bit).

	\warning <b>Preliminary API, subject to change</b>

	\param group The fluid group.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.fluidGroup
	*/
	virtual		void				setFluidGroup(NxFluidGroup group)						= 0;

	/**
	\brief Gets the fluid group set by setFluidGroup().

	\warning <b>Preliminary API, subject to change</b>

	\return The fluid group.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.fluidGroup
	*/
	virtual		NxU32				getFluidGroup()									const	= 0;
#endif

/************************************************************************************************/

/** @name Particle Actions
*/
//@{

	/**
	\brief Sets the particle action which is applied when the particle separates from its neighbors.
	
	Separation does not occur if the particle system is simulated with no particle interactions.

	\warning <b>Preliminary API, subject to change</b>

	\param action The particle action.
	\param val True to execute the action.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.onSeparation
	*/
	virtual		void 				setOnSeparationAction(NxFluidParticleAction action, bool val)		= 0;

	/**
	\brief Returns the on-separation action which is set by setOnSeparationAction(NxFluidParticleAction, bool).

	\warning <b>Preliminary API, subject to change</b>

	\param action The particle action.
	\return True if the action is currently executed.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.onSeparation
	*/
	virtual		NX_BOOL				getOnSeparationAction(NxFluidParticleAction action)	const	= 0;
	
	/**
	\brief Sets the particle action which is applied when the particle collides with actors.

	\warning <b>Preliminary API, subject to change</b>

	\param action The particle action.
	\param val True to execute the action.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.onCollision
	*/
	virtual		void 				setOnCollision(NxFluidParticleAction action, bool val)				= 0;

	/**
	\brief Returns the on-collision action which is set by getOnCollision(NxFluidParticleAction).

	\warning <b>Preliminary API, subject to change</b>

	\param action The particle action.
	\return True if the action is currently executed.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.onCollision
	*/
	virtual		NX_BOOL				getOnCollision(NxFluidParticleAction action)			const	= 0;
	
	/**
	\brief Sets the particle action which is applied when the particle lifetime expires.

	\warning <b>Preliminary API, subject to change</b>

	\param action The particle action.
	\param val True to execute the action.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.onLifetimeExpired
	*/
	virtual		void 				setOnLifetimeExpired(NxFluidParticleAction action, bool val)		= 0;

	/**
	\brief Returns the on-expiration action which is set by setOnLifetimeExpired(NxFluidParticleAction, bool).

	\warning <b>Preliminary API, subject to change</b>

	\param action The particle action.
	\return True if the action is currently executed.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : No
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.onLifetimeExpired
	*/
	virtual		NX_BOOL				getOnLifetimeExpired(NxFluidParticleAction action)		const	= 0;
//@}
/************************************************************************************************/

/** @name Fluid Property Read Back
*/
//@{

	/**
	\brief Returns the maximum number of particles for this fluid.

	\warning <b>Preliminary API, subject to change</b>

	\return Max number of particles for this fluid.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.maxParticles
	*/
	virtual		NxU32 				getMaxParticles()							const	= 0;

	/**
	\brief Returns the kernel radius multiplier (the particle interact within a radius of
	getRestParticleDistance() * getKernelRadiusMultiplier() ).

	\warning <b>Preliminary API, subject to change</b>

	\return The kernel radius multiplier.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.kernelRadiusMultiplier
	*/
	virtual		NxReal				getKernelRadiusMultiplier()					const	= 0;

	/**
	\brief Returns the motion limit multiplier (the particle can move the maximal distance of 
	getRestParticleDistance() * getMotionLimitMultiplier() during one timestep).

	\warning <b>Preliminary API, subject to change</b>

	\return motion limit multiplier.

	@see getRestParticleDistance()
	*/
	virtual		NxReal				getMotionLimitMultiplier()					const	= 0;

	/**
	\brief Returns the fluid packet size used for parallelization of the fluid computations.

	\warning <b>Preliminary API, subject to change</b>

	\return The packet size multiplier.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No
	*/
	virtual		NxU32				getPacketSizeMultiplier()					const	= 0;

	/**
	\brief Returns the number of particles per meter in the relaxed state of the fluid.

	\warning <b>Preliminary API, subject to change</b>

	\return Rest particles per meter.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.restParticlesPerMeter
	*/
	virtual		NxReal				getRestParticlesPerMeter()					const	= 0;

	/**
	\brief Returns the density in the relaxed state of the fluid.

	\warning <b>Preliminary API, subject to change</b>

	\return Rest density.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc.restDensity
	*/
	virtual		NxReal				getRestDensity()							const	= 0;

	/**
	\brief Returns the inter-particle distance in the relaxed state of the fluid.

	This is the reciprocal of the value given by getRestParticlesPerMeter()

	\warning <b>Preliminary API, subject to change</b>

	\return Rest particle distance.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No
	*/
	virtual		NxReal				getRestParticleDistance()					const	= 0;

	/**
	\brief Returns the mass of a particle. This value is dependent on the rest inter-particle
	distance and the rest density of the fluid.

	\warning <b>Preliminary API, subject to change</b>

	\return Particle mass.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see getRestParticleDistance()
	@see getRestDensity()
	*/
	virtual		NxReal				getParticleMass()							const	= 0;

	/**
	\brief Returns the simulation method of the fluid.

	\warning <b>Preliminary API, subject to change</b>
	
	\return The simulation method.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidSimulationMethod
	*/
	virtual		NxU32				getSimulationMethod()						const	= 0;
	
	/**
	\brief Returns the collision method of the fluid.

	\warning <b>Preliminary API, subject to change</b>

	\return The collision method.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidCollisionMethod
	*/
	virtual		NxU32				getCollisionMethod()						const	= 0;

	/**
	\brief Returns the minimal (exact) world space axis aligned bounding box (AABB)
	including all simulated particles.

	\warning <b>Preliminary API, subject to change</b>

	\param dest Used to store the world bounds.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No
	*/
	virtual		void				getWorldBounds(NxBounds3& dest)				const	= 0;
//@}
/************************************************************************************************/


/** @name Fluid Descriptor Operations
*/
//@{

	/**
	\brief Loads the fluid descriptor.

	\param[in] desc The descriptor used to restore the state of the object.
	\return True on success.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc
	*/
	virtual		bool				loadFromDesc(const NxFluidDesc& desc)				= 0;

	/**
	\brief Saves the fluid descriptor.

	This method does not save out any emitters to the emitter array of the fluid desctriptor, 
	nor does it store particle data or any other user pointer. You can use NxFluid::saveEmittersToFluidDesc(NxFluidDesc &)
	to store the emitters of a fluid to a fluid descriptor.

	\param[out] desc The descriptor used to retrieve the state of the object.
	\return True on success.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No

	@see NxFluidDesc
	@see saveEmittersToDesc
	*/
	virtual		bool				saveToDesc(NxFluidDesc &desc)				const	= 0;

//@}
/************************************************************************************************/

	/**
	\brief Sets a name string for the object that can be retrieved with getName().
	
	This is for debugging and is not used by the SDK.  The string is not copied by the
	SDK, only the pointer is stored.

	\warning <b>Preliminary API, subject to change</b>

	\param name The new name.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No
	*/
	virtual	void			setName(const char* name)		= 0;

	/**
	\brief Returns the name string set with setName().

	\warning <b>Preliminary API, subject to change</b>

	\return The current name.

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No
	*/
	virtual	const char*		getName()			const	= 0;

/************************************************************************************************/

	/**
	\brief Helper function to save fluidEmitters to a fluidDescriptor.

	\warning <b>Preliminary API, subject to change</b>

	\return True if the resulting fluid descriptor is valid. 

	<b>Platform:</b>
	\li PC SW: No
	\li PPU  : Yes
	\li PS3  : No
	\li XB360: No
	*/
	NX_INLINE	bool		saveEmittersToFluidDesc(NxFluidDesc &desc);

/************************************************************************************************/


	//public variables:
	void*			userData;	//!< The user can set this to anything, usually to create a 1:1 relationship with a user object.
	};

NX_INLINE	bool		NxFluid::saveEmittersToFluidDesc(NxFluidDesc &desc)
	{
	NxU32				numEmitters = getNbEmitters();
	NxFluidEmitter**	emitter		= getEmitters();
	for(NxU32 i = 0; i < numEmitters; i++)
		{
		NxFluidEmitterDesc emitterDesc;
		emitter[i]->saveToDesc(emitterDesc);
		desc.emitters.pushBack(emitterDesc);
		}
		return desc.isValid();
	}

/** @} */
#endif


//AGCOPYRIGHTBEGIN
///////////////////////////////////////////////////////////////////////////
// Copyright  2005 AGEIA Technologies.
// All rights reserved. www.ageia.com
///////////////////////////////////////////////////////////////////////////
//AGCOPYRIGHTEND

