#ifndef NX_FLUIDS_NXFLUIDACTORDESC
#define NX_FLUIDS_NXFLUIDACTORDESC
/** \addtogroup fluids
  @{
*/
/*----------------------------------------------------------------------------*\
|
|						Public Interface to NovodeX Technology
|
|							     www.novodex.com
|
\*----------------------------------------------------------------------------*/

#include "fluids/NxMeshData.h"
#include "fluids/NxParticleData.h"
#include "fluids/NxFluidEmitterDesc.h"
#include "fluids/NxImplicitMeshDesc.h"

#include "NxArray.h"

/**
Particles can be treated in two ways: either they are simulated considering
interparticular forces (SPH), or they are simulated independently.
In the latter case (with the simulation method set to NX_F_NO_PARTICLE_INTERACTION),
you still get collision between particles and static/dynamic shapes, damping,
acceleration due to gravity, and the user force.

\warning <b>Preliminary API, subject to change</b>
*/
enum NxFluidSimulationMethod
	{
	/**
	\brief Enable simulation of inter particle forces.
	*/
	NX_F_SPH						= (1<<0),	

	/**
	\brief Do not simulate inter particle forces.
	*/
	NX_F_NO_PARTICLE_INTERACTION	= (1<<1),

	/**
	\brief Alternate between SPH and simple particles
	*/
	NX_F_MIXED_MODE					= (1<<2),
	};

/**
There are several interesting particle events during the simulation:
<ul>
	<li>Particle separates completely from other particles.
	<li>Particle collides with static or dynamic environment.
	<li>Particle runs out of lifetime.
	<li>Particle exceeds certain speed.
	<li>Particle drained.
	<li>Particle stands still.
</ul>
On one hand these events are stored as flags and can be retrieved from the simulation
along with other particle states such as position, velocity, and lifetime. On the other
hand it's possible that the simulation takes certain actions when particles enter or
leave these states:
<ul>
<li>
TURN_DEAD means that the particle is removed from the simulation.
<li>
TURN_SIMPLE means that the particle is now simulated without considering any particle neighbors.
This will be useful (e.g.) to remove particles which are separated from the
simulation or turn them into simple particles, which will lower the computational
cost for the SPH simulation as well as for the surface mesh generation.
</ul>

\warning <b>Preliminary API, subject to change</b>
*/
enum NxFluidParticleAction
	{
	NX_F_TURN_DEAD		= (1<<1),
	NX_F_TURN_SIMPLE	= (1<<2),
	};

/**
The NxFluid instance can be selected for collision with both static and dynamic shapes.
One can also define whether the NxFluid collision with a dynamic actor should have any
impact on the momentum of the actor or not.

\warning <b>Preliminary API, subject to change</b>
*/
enum NxFluidCollisionMethod
	{
	NX_F_STATIC					= (1<<0),	
	NX_F_DYNAMIC 				= (1<<1),
	NX_F_DYNAMIC_ACTOR_REACTION	= (1<<2),
	};

/**
Fluid flags.
NX_FF_VISUALIZATION enables debug visualization for the NxFluid.
NX_FF_MESH_IGNORE_SEPARATED specifies that single separated particles should not be included
in the surface generation

\warning <b>Preliminary API, subject to change</b>
*/
enum NxFluidFlag
	{
	NX_FF_VISUALIZATION							= (1<<0),
	NX_FF_MESH_INCLUDE_SEPARATED				= (1<<1),	
	};

/**
\brief Describes an NxFluid.

\warning <b>Preliminary API, subject to change</b>
*/
class NxFluidDesc
	{
	public:

	/**
	\brief Array of emitter descriptors that describe emitters which emit fluid into this
	fluid actor.

	A fluid actor can have any number of emitters.

	\warning <b>Preliminary API, subject to change</b>

	@see NxFluidEmitter
	*/
	NxArray<NxFluidEmitterDesc>	emitters;
	
	/**
	\brief 	Describes the particles which are added to the fluid initially.

	The pointers to the buffers are invalidated after the initial particles are generated.

	\warning <b>Preliminary API, subject to change</b>

	@see NxParticleData
	*/
	NxParticleData				initialParticleData;
	
	/**
	\brief Sets the maximal number of particles for the fluid used in the simulation.
	
	If more particles are added directly, or more particles are emitted into the
	fluid after this limit is reached, they are simply ignored.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxU32						maxParticles;

	/**
	\brief The particle resolution given as particles per linear meter measured when the fluid is
	in its rest state (relaxed).
	
	Even if the particle system is simulated without particle interactions, this parameter defines the 
	emission density of the emitters.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxReal						restParticlesPerMeter;

	/**
	\brief Target density for the fluid (water is about 1000).
	
	Even if the particle system is simulated without particle interactions, this parameter defines 
	indirectly in combination with restParticlesPerMeter the mass of one particle 
	( mass = restDensity/(restParticlesPerMeter^3) ).
	
	The particle mass has an impact on the repulsion effect on emitters and actors.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxReal						restDensity;

	/**
	\brief Radius of sphere of influence for particle interaction.
	
	This parameter is relative to the rest spacing of the particles, which is defined by the parameter
	restParticlesPerMeter
	
	( radius = kernelRadiusMultiplier/restParticlesPerMeter ).

	\warning <b>Preliminary API, subject to change</b>

	@see restParticlesPerMeter
	*/
	NxReal						kernelRadiusMultiplier;

	/**
	\brief Maximal distance a particle is allowed to travel within one timestep.

	This parameter is relative to the rest spacing of the particles, which is defined by the parameter
	restParticlesPerMeter

	( maximal travel distance = motionLimitMultiplier/restParticlesPerMeter ).

	\warning <b>Preliminary API, subject to change</b>

	@see restParticlesPerMeter
	*/
	NxReal						motionLimitMultiplier;

	/**
	\brief This parameter controls the parallelization of the fluid.
	
	The spatial domain is divided into so called packets, equal sized cubes, alligned in a grid.
	
	The parameter given defines the edge length of such a packet. This parameter is relative to the interaction 
	radius of the particles, given as restParticlesPerMeter*kernelRadiusMultiplier.

	It has to be a power of two.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxU32						packetSizeMultiplier;

	/**
	\brief The stiffness of the particle interaction related to the pressure.
	
	This factor linearly scales the force which acts on particles which are closer to each other than 
	the rest spacing.
	
	Setting	this parameter appropriately is crucial for the simulation.  The right value depends on many factors
	such as viscosity, damping, and kernelRadiusMultiplier.  Values which are too high will result in an
	unstable simulation, whereas too low values will make the fluid appear "springy" (the fluid
	acts more compressible).

	Must be positive.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxReal						stiffness;

	/**
	\brief 	The viscosity of the fluid defines its viscous behaviour.
	
	Higher values will result in a honey-like behaviour.  Viscosity is an effect which depends on the 
	relative velocity of neighboring particles; it reduces the magnitude of the relative velocity.
	
	Must be positive.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxReal						viscosity;

	/**
	\brief Velocity damping constant, which is globally applied to each particle.
	
	It generally reduces the velocity of the particles. Setting the damping to 0 will leave the 
	particles unaffected.

	Must be nonnegative.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxReal						damping;
	/**
	\brief Body force (N/m^3) applied to all particles at all time steps.

	Useful to simulate smoke or fire.
	Acceleration = body force / rest density.
	For example,
	with a rest density of 1000.0 a body force of <0, 10000.0, 0>
	counteracts a gravity of <0,-10,0>.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxVec3						externalForce;

	/**
	\brief Defines the restitution coefficient used for collisions of the fluid particles with static shapes.

	Must be between 0 and 1.
	
	A value of 0 causes the colliding particle to get a zero velocity component in the
	direction of the surface normal of the static shape at the collision location; i.e.
	it will not bounce.
	
	A value of 1 causes a particle's velocity component in the direction of the surface normal to invert;
	i.e. the particle bounces off the surface with the same velocity magnitude as it had before collision.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxReal						staticCollisionRestitution;
	
	/**
	\brief Defines the "friction" of the fluid regarding the surface of a static shape.

	Must be between 0 and 1.
	
	A value of 1 will cause the particle to lose its velocity tangential to
	the surface normal of the shape at the collision location; i.e. it will not slide
	along the surface.
	
	A value of 0 will preserve the particle's velocity in the tangential surface
	direction; i.e. it will slide without resistance on the surface.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxReal						staticCollisionAdhesion;
	
	/**
	\brief Defines the restitution coefficient used for collisions of the fluid particles with dynamic shapes.
	
	Must be between 0 and 1.

	\warning <b>Preliminary API, subject to change</b>

	@see staticCollisionRestitution
	*/
	NxReal						dynamicCollisionRestitution;
	
	/**
	\brief Defines the "friction" of the fluid regarding the surface of a dynamic shape.

	Must be between 0 and 1.

	\warning <b>Preliminary API, subject to change</b>

	@see staticCollisionAdhesion
	*/
	NxReal						dynamicCollisionAdhesion;

	/**
	\brief Defines the fluid's factor for smoothing impulse transfer for bidirectional interaction with dynamic 
	actors.
	
	Due to the approximation of a fluid using discrete particles, the collision between a fluid volume 
	and a rigid body decomposes into a series of discrete particle rigid body collisions. In order to get 
	a smooth impulse transfer the impulses are smoothed over time.

	Must be between 0 and 1.
	
	A value of 0 means no smoothing (i.e. impulses of past timesteps have no influence on the current
	applied impulses) and a value of 1 will cause a smoothing where the impulses
	of the last timestep have the same weight as the impulses of the current timestep.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxReal						dynamicActorReactionSmoothing;

	/**
	\brief Defines the fluid's factor for scaling impulse transfer, used for collision with dynamic actors.

	Sometimes it's hard to pick the right fluid properties as restDensity and restParticlesPerMeter
	and appropriate rigid body masses to get the desired level of interaction between rigid bodies and the fluid.
	
	This parameter allows scaling of the interaction effect. A value of 1 has no effect.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxReal						dynamicActorReactionScaling;

	/**
	\brief NxFluidSimulationMethod flags. Defines whether or not particle interactions are considered 
	in the simulation.

	\warning <b>Preliminary API, subject to change</b>

	@see NxFluidSimulationMethod
	*/
	NxU32						simulationMethod;


	/**
	\brief NxFluidCollisionMethod flags. Selects whether static collision and/or dynamic collision 
	with the environment is performed.
	
	If NX_ACTOR_REACTION is set, fluid particles act on rigid body actors. Collisions can be overruled
	on a per-actor basis.

	\warning <b>Preliminary API, subject to change</b>

	@see NxFluidCollisionMethod
	*/
	NxU32						collisionMethod;

	/**
	\brief These fields use the NxFluidParticleAction flags; they represent particle events which can each be 
	configured with a particle action.

	\warning <b>Preliminary API, subject to change</b>

	@see NxFluidParticleAction
	*/
	NxU32						onSeparation;
	NxU32						onCollision;
	NxU32						onLifetimeExpired;	
	
	/**
	\brief The surface mesh descriptor used to create an NxImplicitMesh which serves fluid mesh generation.

	If NULL, no surface mesh will be generated.

	\warning <b>Preliminary API, subject to change</b>

	@see NxImplicitMeshDesc
	*/
	NxImplicitMeshDesc*			surfaceMeshDesc;
	
	/**
	\brief Defines the user data buffers which are used to store particle data, which can be used for rendering.

	\warning <b>Preliminary API, subject to change</b>

	@see NxParticleData
	*/
	NxParticleData				particlesWriteData;

	/**
	\brief Flags which define behavior when colliding with actors.

	\warning <b>Preliminary API, subject to change</b>

	@see NxFluidFlag
	*/
	NxU32						flags;

#if NX_USE_FLUID_API
	/**
	\brief The fluids group.
	
	It is possible to assign each fluid to a group. This is a different
	set of groups from the collision groups (used on the shape level) despite the similar name.
	Here up to 0xffff different groups are permitted. This group space is independent from the
	actor group space. Using the fluid group one can set contact reporting flags between fluids and
	actors belonging to a pair of groups.
	Only one flag is supported: NX_FPF_NOTIFY_ON_COLLISION	

	\warning <b>Preliminary API, subject to change</b>

	@see NxFluidContactPairFlag
	*/
	NxFluidGroup				fluidGroup;
#endif

	/**
	\brief The fluid collision group.

	It is possible to assign each fluid to a collision group.  This collision group is used to specify
	whether collisions should be detected between fluid shape pairs	belonging to a given pair of groups.
	Initially all pairs are enabled.

	Collision detection between a shape and a fluid occurs if
	<tt>getGroupCollisionFlag(shape->getGroup(), fluid->getCollisionGroup()) && isEnabledPair(a,b)</tt>
	is true.

	The shape collision groups and fluid collision groups share the same address space.
	NxCollisionGroup is an integer between 0 and 31.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxCollisionGroup			collisionGroup;	

	void*						userData;	//!< Will be copied to NxFluid::userData
	const char*					name;		//!< Possible debug name. The string is not copied by the SDK, only the pointer is stored.

	NX_INLINE ~NxFluidDesc();
	/**
	\brief Constructor sets to default.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NX_INLINE NxFluidDesc();	
	/**
	\brief (Re)sets the structure to the default.	

	\warning <b>Preliminary API, subject to change</b>
	*/
	NX_INLINE void setToDefault();
	/**
	\brief Returns true if the current settings are valid

	\warning <b>Preliminary API, subject to change</b>
	*/
	NX_INLINE bool isValid() const;
	};

NX_INLINE NxFluidDesc::NxFluidDesc()
	{
	setToDefault();
	}

NX_INLINE NxFluidDesc::~NxFluidDesc()
	{
	}

NX_INLINE void NxFluidDesc::setToDefault()
	{
	emitters					.clear();

	maxParticles				= 0xffff;
	restParticlesPerMeter		= 50.0f;
    restDensity					= 1000.0f;
    kernelRadiusMultiplier		= 1.2f;
	motionLimitMultiplier		= 3.0f * kernelRadiusMultiplier;
	packetSizeMultiplier		= 256;
	stiffness					= 20.0f;
    viscosity					= 6.0f;
    damping						= 0.0f;
	externalForce.zero();
    staticCollisionRestitution	= 1.0f;
	staticCollisionAdhesion		= 0.05f;
	dynamicCollisionRestitution	= 1.0f;
	dynamicCollisionAdhesion	= 0.5f;
	dynamicActorReactionSmoothing = 0.75f;
	dynamicActorReactionScaling = 0.5f;

	simulationMethod			= NX_F_SPH;
	collisionMethod				= NX_F_STATIC|NX_F_DYNAMIC;
	
	onSeparation				= 0;
	onCollision					= 0;
	onLifetimeExpired			= 0;

	surfaceMeshDesc				= NULL;
	particlesWriteData			.setToDefault();

	flags						= NX_FF_VISUALIZATION;
	
	userData					= NULL;
	name						= NULL;
	}

NX_INLINE bool NxFluidDesc::isValid() const
	{
	if (kernelRadiusMultiplier < 1.0f) return false;
	if (restDensity <= 0.0f) return false;
	if (restParticlesPerMeter <= 0.0f) return false;

	if (packetSizeMultiplier == 0) return false;
	if (packetSizeMultiplier & ( packetSizeMultiplier - 1 ) ) return false; 

	if (motionLimitMultiplier <= 0.0f) return false;
	if (motionLimitMultiplier > packetSizeMultiplier*kernelRadiusMultiplier) return false;

	if (stiffness <= 0.0f) return false;
	if (viscosity <= 0.0f) return false;

	if (!(simulationMethod & NX_F_NO_PARTICLE_INTERACTION ^ simulationMethod & NX_F_SPH ^ NX_F_MIXED_MODE)) return false;
	
	if (damping < 0.0f) return false;
	if (dynamicCollisionAdhesion < 0.0f || dynamicCollisionAdhesion > 1.0f) return false;
	if (dynamicCollisionRestitution < 0.0f || dynamicCollisionRestitution > 1.0f) return false;
	if (staticCollisionAdhesion < 0.0f || staticCollisionAdhesion > 1.0f) return false;
	if (staticCollisionRestitution < 0.0f || staticCollisionRestitution > 1.0f) return false;
	if (dynamicActorReactionSmoothing  < 0.0f || dynamicActorReactionSmoothing  > 1.0f) return false;
	
	if (onCollision && !(onCollision & NX_F_TURN_DEAD ^ onCollision & NX_F_TURN_SIMPLE)) return false;
	if (onSeparation && !(onSeparation & NX_F_TURN_DEAD ^ onSeparation & NX_F_TURN_SIMPLE)) return false;
	if (onLifetimeExpired && !(onLifetimeExpired & NX_F_TURN_DEAD ^ onLifetimeExpired & NX_F_TURN_SIMPLE)) return false;

	if (!initialParticleData.isValid()) return false;
	if (!particlesWriteData.isValid()) return false;
	if (surfaceMeshDesc && !surfaceMeshDesc->isValid()) return false;

	if (maxParticles > 0xffff) return false;
	if (emitters.size() > 0xffff)

	for (unsigned i = 0; i < emitters.size(); i++)
		if (!emitters[i].isValid()) return false;

	return true;
	}
/** @} */
#endif


//AGCOPYRIGHTBEGIN
///////////////////////////////////////////////////////////////////////////
// Copyright  2005 AGEIA Technologies.
// All rights reserved. www.ageia.com
///////////////////////////////////////////////////////////////////////////
//AGCOPYRIGHTEND

