#ifndef NX_FLUIDS_NXPARTICLEDATA
#define NX_FLUIDS_NXPARTICLEDATA
/** \addtogroup fluids
  @{
*/

/*----------------------------------------------------------------------------*\
|
|						Public Interface to NovodeX Technology
|
|							     www.novodex.com
|
\*----------------------------------------------------------------------------*/

/**
\brief Particle flags are used to specify the state particles are in.

\warning <b>Preliminary API, subject to change</b>
*/
enum NxParticleFlag
	{
	NX_FP_DIED						= (1<<0),	
	NX_FP_LIFETIME_EXPIRED			= (1<<1),
	NX_FP_DRAINED					= (1<<2),
	NX_FP_SEPARATED					= (1<<3),
	};

/**
\brief Descriptor-like user-side class describing a set of fluid particles.

NxParticleData is used to submit particles to the simulation and
to retrieve information about particles in the simulation. 

Each particle is described by its position, velocity, lifetime, density, and a set of (NxParticleFlag) flags.
The memory for the particle data is allocated by the application,
making this class a "user buffer wrapper".

\warning <b>Preliminary API, subject to change</b>
*/
class NxParticleData
	{
	public:

	/**
	\brief Specifies the maximal number of particle elements the buffers can hold.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxU32					maxParticles;
    
	/**
	\brief Points to the user-allocated memory holding the number of elements stored in the buffers. 
	
	If the SDK writes to a given particle buffer, it also sets the numbers of elements written. If 
	this is set to NULL, the SDK can't write to the given buffer.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxU32*					numParticlesPtr;

	/**
	\brief The pointer to the user-allocated buffer for particle positions.

	A position consists of three consecutive 32-bit floats. If set to NULL, positions are not read or written to.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxF32*					bufferPos;
	
	/**
	\brief The pointer to the user-allocated buffer for particle velocities.

	A velocity consists of three consecutive 32-bit 
	floats. If set to NULL, velocities are not read or written to.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxF32*					bufferVel;
	
	/**
	\brief The pointer to the user-allocated buffer for particle lifetimes.

	A particle lifetime consists of one 32-bit 
	float. If set to NULL, lifetimes are not read or written to.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxF32*					bufferLife;
    
	/**
	\brief The pointer to the user-allocated buffer but for particle densities.

	A particle density consists of one 32-bit float. If set to NULL, densities are not read or written to.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxF32*					bufferDensity;

	/**
	\brief The pointer to the user-allocated buffer for particle flags of the type NxParticleFlag.

	The flags are represented by one 32-bit unsigned integer. If set to NULL, flags are not read or written to.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxU32*					bufferFlags;


	/**
	\brief The separation (in bytes) between consecutive particle positions.

	The position of the first particle is found at location <tt>bufferPos</tt>;
	the second is at <tt>bufferPos + bufferPosByteStride</tt>;
	and so on.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxU32					bufferPosByteStride;

	/**
	\brief The separation (in bytes) between consecutive particle velocities.

	The velocity of the first particle is found at location <tt>bufferVel</tt>;
	the second is at <tt>bufferVel + bufferVelByteStride</tt>;
	and so on.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxU32					bufferVelByteStride;

	/**
	\brief The separation (in bytes) between consecutive particle lifetimes.

	The lifetime of the first particle is found at location <tt>bufferLife</tt>;
	the second is at <tt>bufferLife + bufferLifeByteStride</tt>;
	and so on.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxU32					bufferLifeByteStride;

	/**
	\brief The separation (in bytes) between consecutive particle densities.

	The density of the first particle is found at location <tt>bufferDensity</tt>;
	the second is at <tt>bufferDensity + bufferDensityByteStride</tt>;
	and so on.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxU32					bufferDensityByteStride;

	/**
	\brief The separation (in bytes) between consecutive particle flags.

	The flags for the first particle are found at location <tt>bufferFlags</tt>;
	the second are at <tt>bufferFlags + bufferFlagsByteStride</tt>;
	and so on.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxU32					bufferFlagsByteStride;

	const char*				name;			//!< Possible debug name. The string is not copied by the SDK, only the pointer is stored.

	NX_INLINE ~NxParticleData();
	/**
	\brief (Re)sets the structure to the default.	

	\warning <b>Preliminary API, subject to change</b>
	*/
	NX_INLINE void setToDefault();
	/**
	\brief Returns true if the current settings are valid

	\warning <b>Preliminary API, subject to change</b>
	*/
	NX_INLINE bool isValid() const;

	/**
	\brief Constructor sets to default.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NX_INLINE	NxParticleData();
	};

NX_INLINE NxParticleData::NxParticleData()
	{
	setToDefault();
	}

NX_INLINE NxParticleData::~NxParticleData()
	{
	}

NX_INLINE void NxParticleData::setToDefault()
	{
	maxParticles			= 0;
	numParticlesPtr			= NULL;
	bufferPos				= NULL;
	bufferVel				= NULL;
	bufferLife				= NULL;
	bufferDensity			= NULL;
	bufferFlags				= NULL;
	bufferPosByteStride		= 0;
	bufferVelByteStride		= 0;
	bufferLifeByteStride	= 0;
	bufferDensityByteStride = 0;
	bufferFlagsByteStride	= 0;
	name					= NULL;
	}

NX_INLINE bool NxParticleData::isValid() const
	{
	if (numParticlesPtr && !(bufferPos || bufferVel || bufferLife || bufferDensity || bufferFlags)) return false;
	if ((bufferPos || bufferVel || bufferLife || bufferDensity || bufferFlags) && !numParticlesPtr) return false;
	if (bufferPos && !bufferPosByteStride) return false;
	if (bufferVel && !bufferVelByteStride) return false;
	if (bufferLife && !bufferLifeByteStride) return false;
	if (bufferDensity && !bufferDensityByteStride) return false;
	if (bufferFlags && !bufferFlagsByteStride) return false;
    
	return true;
	}

/** @} */
#endif


//AGCOPYRIGHTBEGIN
///////////////////////////////////////////////////////////////////////////
// Copyright  2005 AGEIA Technologies.
// All rights reserved. www.ageia.com
///////////////////////////////////////////////////////////////////////////
//AGCOPYRIGHTEND

