/****************************************************************************\
*                                                                            *
* 20.02.2006                                                                 *
* CGuiMgr - Header File                                                      *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
* CGuiMgr nutzt ein "Left-Handed"-Koordinatensystem:                         *
* Ursprung liegt links oben und positive Z-Werte zeigen in Richtung Benutzer *
*                                                                            *
\****************************************************************************/

#ifndef __CGUIMGR_H__
#define __CGUIMGR_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/IManagedObject.h>
#include <Input/IInputMgr.h>
#include <Display/Windows/IWindow.h>
#include <Math/CMatrix44.h>
#include <Renderer/IRenderer.h>
#include <Timer/CTimer.h>
#include "IGuiObject.h"
#include "CGuiContainer.h"
#include "CGuiButton.h"
#include "CGuiText.h"
#include "CGuiWnd.h"

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

enum EShortcutEvent
{
    SE_MOUSE_CLICK,
    SE_KB_ENTER,
    SE_FOCUS,
};

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CGuiMgr : public IManagedObject, private IInputHandler, private IWindowHandler
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CGuiMgr() {Clear();}
        
        //Creation (VScreen-Dimensionen geben die Gre des virtuellen Koordinatensystems fr die GUI an)
        bool Create(const TSmartPointer<IWindow> &pWindow, const TSmartPointer<IInputMgr> &pInputMgr,
                    const TSmartPointer<CTimer> &pTimer, word wVScreenWidth, word wVScreenHeight);

        void Update();
        void Render();

        //GUI state control
        void SetRootObject(const TSmartPointer<IGuiObject> &pObject);
        inline TSmartPointer<IGuiObject> GetRootObject() const {return m_pRootObject;}
        void GrabInput(const TSmartPointer<IGuiObject> &pObject);
        void ReleaseInput(const TSmartPointer<IGuiObject> &pObject);
        void SetFocus(const TSmartPointer<IGuiObject> &pObject);

        //State request
        inline void GetVScreenSize(word &wWidth, word &wHeight) const {wWidth = m_wVScreenWidth; wHeight = m_wVScreenHeight;}
        inline CVector2f GetCursorPos() const {return m_vCursorPos;}
/*
        void RegisterShortcut(dword dwKey, const TSmartPointer<IGuiObject> &pObject, EShortcutEvent event);
        void UnregisterShortcut(dword dwKey, const TSmartPointer<IGuiObject> &pObject);
*/

        //Factory functions
        TSmartPointer<CGuiContainer> CreateContainer() const;
        TSmartPointer<CGuiWnd> CreateWnd(const std::string &sTexture,
										const CVector2f &uv1 = CVector2f(0.0f, 0.0f),
										const CVector2f &uv2 = CVector2f(1.0f, 1.0f),
										const CVector4f &color = CVector4f(1.0f, 1.0f, 1.0f, 1.0f),
										EBlendingMode blendingMode = EBM_NONE) const;
		TSmartPointer<CGuiButton> CreateButton(const std::string &sTexture = "Data\\buttonTex.tga") const;
        TSmartPointer<CGuiText> CreateText() const;

		//Info
		inline void EnableInfoRender(bool bEnable){m_bRenderInfo = bEnable;}

        void Destroy();

        virtual ~CGuiMgr() {Destroy();}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        void Clear();
        void UpdateCursorPos();

        virtual void OnWndSize(word wWidth, word wHeight);
        virtual bool OnMouseMove(sdword sdwX, sdword sdwY, sdword sdwZ);
        virtual bool OnMouseKeyDown(byte bKey);
        virtual bool OnMouseKeyUp(byte bKey);
        virtual bool OnChar(char c);
        virtual bool OnKeyDown(dword dwKey);
        virtual bool OnKeyUp(dword dwKey);

        //Private copy constructor and assignment operator to prevent mistakes:
        CGuiMgr(const CGuiMgr &);
        CGuiMgr &operator =(const CGuiMgr &);
      
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        word m_wVScreenWidth, m_wVScreenHeight;
        float m_fWidthScale, m_fHeightScale;
        CVector2f m_vCursorPos;

        CMatrix44 m_Projection;

        TSmartPointer<IGuiObject> m_pRootObject;
        TSmartPointer<IGuiObject> m_pFocusObject;
        typedef std::vector< TSmartPointer<IGuiObject> > GuiObjectList;
        GuiObjectList m_InputGrabbers;      //Stack (!) von Objekten, die den Input fr sich alleine beanspruchen. Nur das oberste Objekt bekommt Input-Events

        TSmartPointer<IWindow> m_pWindow;
        TSmartPointer<IInputMgr> m_pInputMgr;
        TSmartPointer<CTimer> m_pTimer;

        //Gui Font (could create multiple...)
        TSmartPointer<IFont> m_pGuiFont;

        //Gui sounds
        TSmartPointer<ISound> m_pButtonRolloverSound;
        TSmartPointer<ISound> m_pButtonClickSound;

        //For faster access
        TSmartPointer<IRenderer> m_pRenderer;

        //Debug: For FPS
		char   m_sFps[32];
		double m_oldFpsUpdateTime, m_newFpsUpdateTime;
        TSmartPointer<CGuiText> m_pFpsText;

        //Debug: For Reps
		char   m_sRendered[32];
        TSmartPointer<CGuiText> m_pRepsRendered;
		
		bool m_bRenderInfo;
};

#endif
