/****************************************************************************\
*                                                                            *
* 16.02.2006                                                                 *
* CW32Window - Source File                                                   *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <map>

#include "CW32Window.h"

#define W32WNDCLASS_NAME "RCW32Window"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
CW32Window::WindowMap CW32Window::sm_WindowMap;
bool CW32Window::sm_bWndClassRegistered = false;

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CW32Window::Create(HINSTANCE hInstance, const std::string &sTitle,
                                word wWidth, word wHeight, bool bFullscreen,
                                sword swXPos, sword swYPos, byte bBitsPerPixel)
{
    assert(!IsCreated() && "Error: CW32Window::Create called for an already existing window");

    m_hInstance = hInstance;

    if(!sm_bWndClassRegistered)
    {
        WNDCLASSEX wc;
        wc.cbSize = sizeof(wc);
        wc.style = 0;
        wc.lpfnWndProc = StaticWindowProc;
        wc.cbClsExtra = 0;
        wc.cbWndExtra = 0;
        wc.hInstance = hInstance;
        wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
        wc.hIconSm = LoadIcon(NULL, IDI_APPLICATION);
        wc.hCursor = LoadCursor(NULL, IDC_ARROW);
        wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
        wc.lpszMenuName = NULL;
        wc.lpszClassName = W32WNDCLASS_NAME;

        if(!RegisterClassEx(&wc))
        {
            LOG(TL_USER | TL_ERROR, "Error: RegisterClassEx failed in CW32Window::Create!");
            return false;
        }

        sm_bWndClassRegistered = true;
    }

    HDC curIC = CreateIC("DISPLAY", "", "", NULL);

    m_iOldWidth = GetDeviceCaps(curIC, HORZRES);
    m_iOldHeight  = GetDeviceCaps(curIC, VERTRES);
    m_iOldDepth  = GetDeviceCaps(curIC, BITSPIXEL);
    m_iOldRefresh = GetDeviceCaps(curIC, VREFRESH);

    DeleteDC(curIC);

    if(bFullscreen)
    {
        DEVMODE dmScreenSettings;
        ZeroMemory(&dmScreenSettings, sizeof(dmScreenSettings));
        dmScreenSettings.dmSize = sizeof(dmScreenSettings);
        dmScreenSettings.dmPelsWidth	= wWidth;
        dmScreenSettings.dmPelsHeight	= wHeight;
        dmScreenSettings.dmBitsPerPel	= bBitsPerPixel;
        dmScreenSettings.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;

        if(ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL)
        {
            bFullscreen = false;
        }
    }

    DWORD dwStyle = WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS; // | WS_VISIBLE;

    if(!bFullscreen)
    {
        dwStyle |= WS_CAPTION | WS_SYSMENU | WS_THICKFRAME | WS_MAXIMIZEBOX; //  | WS_MINIMIZEBOX
    }

    RECT rc;
    SetRect(&rc, 0, 0, wWidth, wHeight);
    AdjustWindowRect(&rc, dwStyle, false);

    if(!(m_hWnd = CreateWindowEx(0,//WS_EX_TOPMOST,
                                 W32WNDCLASS_NAME,
                                 sTitle.c_str(),
                                 dwStyle,
                                 swXPos,
                                 swYPos,
                                 rc.right - rc.left,
                                 rc.bottom - rc.top,
                                 NULL,
                                 NULL,
                                 hInstance,
                                 NULL)))
    {
        LOG(TL_USER | TL_ERROR, "Error: CreateWindowEx failed in CW32Window::Create!");
        return false;
    }

    sm_WindowMap.insert(WindowMapValType(m_hWnd, this));

    //Status merken
    m_bFullscreen = bFullscreen;
    m_bBitsPerPixel = bBitsPerPixel;
    m_wDefaultWidth = m_wWidth = wWidth;
    m_wDefaultHeight = m_wHeight = wHeight;

    return true;
}

void CW32Window::SetPos(sword swPosX, sword swPosY)
{
    if(!m_bFullscreen)
    {
        SetWindowPos(m_hWnd, 0, swPosX, swPosY, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);
        m_swDefaultXPos = swPosX;
        m_swDefaultYPos = swPosY;
    }
}

void CW32Window::SetSize(word wWidth, word wHeight)
{
    if(!m_bFullscreen)
    {
        SetWindowPos(m_hWnd, 0, 0, 0, wWidth, wHeight, SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);
        m_wDefaultWidth = wWidth;
        m_wDefaultHeight = wHeight;
    }
}
void CW32Window::SetTitle(const std::string &sTitle)
{
    SetWindowText(m_hWnd, sTitle.c_str());
}

void CW32Window::SetFullscreen(bool bFullscreen, word wWidth, word wHeight)
{
    HandlerListIterator i(m_WindowHandlers.begin());
    while(i != m_WindowHandlers.end())
    {
        (*i)->OnPreFullscreen(bFullscreen);
        ++i;
    }

    m_wWidth = wWidth;
    m_wHeight = wHeight;

    if(bFullscreen)
    {
        DEVMODE dmScreenSettings;
        ZeroMemory(&dmScreenSettings, sizeof(dmScreenSettings));
        dmScreenSettings.dmSize = sizeof(dmScreenSettings);
        dmScreenSettings.dmPelsWidth	= wWidth;
        dmScreenSettings.dmPelsHeight	= wHeight;
        dmScreenSettings.dmBitsPerPel	= m_bBitsPerPixel;
        dmScreenSettings.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;

        if(ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL)
        {
            LOG(TL_ERROR, "Errror: Failed to enter fullscreen mode");
            bFullscreen = false;
        }
        else
        {
            SetWindowLong(m_hWnd, GWL_STYLE, WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS);
            SetWindowPos(m_hWnd, HWND_TOP, 0, 0, wWidth, wHeight, SWP_SHOWWINDOW | SWP_FRAMECHANGED);
        }
    }
    if(!bFullscreen)
    {
        if(m_bFullscreen)
        {
            RestoreMode();
        }
        DWORD dwStyle = WS_POPUP | WS_CLIPCHILDREN;
        SetWindowLong(m_hWnd, GWL_STYLE, WS_POPUP | WS_CLIPCHILDREN | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME | WS_MINIMIZEBOX | WS_MAXIMIZEBOX);
        SetWindowPos(m_hWnd, HWND_TOP, m_swDefaultXPos, m_swDefaultYPos, wWidth, wHeight, SWP_SHOWWINDOW | SWP_FRAMECHANGED);
    }

    m_bFullscreen = bFullscreen;

    i = m_WindowHandlers.begin();
    while(i != m_WindowHandlers.end())
    {
        (*i)->OnPostFullscreen(bFullscreen);
        ++i;
    }
}

void CW32Window::Show(bool bShow)
{
    if(bShow)
    {
        ShowWindow(m_hWnd, SW_SHOW);
    }
    else
    {
        ShowWindow(m_hWnd, SW_HIDE);
    }
}

void CW32Window::Destroy()
{
    if(m_hWnd)
    {
        WindowMapIterator i;

        i = sm_WindowMap.find(m_hWnd);

        assert(i != sm_WindowMap.end() && "Error: Window not found in CW32Window::Destroy");

        sm_WindowMap.erase(i);

        DestroyWindow(m_hWnd);

        if(sm_WindowMap.empty())
        {
            UnregisterClass(W32WNDCLASS_NAME, m_hInstance);
            sm_bWndClassRegistered = false;
        }
    }

    IWindow::Destroy();

    Clear();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CW32Window::Clear()
{
    m_hWnd = NULL;
    m_hInstance = NULL;
    m_bDestroy = false;
}

void CW32Window::RestoreMode()
{
    dword i = 0;
    DEVMODE devmode;

    while(EnumDisplaySettings(NULL, i, &devmode))
    {
        if (devmode.dmPelsWidth == m_iOldWidth && devmode.dmPelsHeight == m_iOldHeight && devmode.dmBitsPerPel == m_iOldDepth && devmode.dmDisplayFrequency == m_iOldRefresh)
            break;
        ++i;
    }

    ChangeDisplaySettings(&devmode, CDS_FULLSCREEN);
}

LRESULT CALLBACK CW32Window::StaticWindowProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    WindowMapIterator i;

    i = sm_WindowMap.find(hWnd);
    if(i == sm_WindowMap.end())
    {
        return DefWindowProc(hWnd, msg, wParam, lParam);
    }

    return (*i).second->WindowProc(msg, wParam, lParam);
}

LRESULT CW32Window::WindowProc(UINT msg, WPARAM wParam, LPARAM lParam)
{
    HandlerListIterator i(m_WindowHandlers.begin());
    switch(msg)
    {
        case WM_ACTIVATEAPP:
            while(i != m_WindowHandlers.end())
            {
                (*i)->OnWndActivate(wParam != 0);
                 ++i;
            }
            return 0;
        case WM_CHAR:
            while(i != m_WindowHandlers.end())
            {
                (*i)->OnWndChar((char) wParam);
                ++i;
            }
            return 0;
        case WM_DESTROY:
            while(i != m_WindowHandlers.end())
            {
                m_bDestroy = true;
                (*i)->OnWndDestroy();
                ++i;
            }
            return 0;
        case WM_SIZE:
            m_wWidth = LOWORD(lParam);  //LoWord=Width, HiWord=Height
            m_wHeight = HIWORD(lParam);
            while(i != m_WindowHandlers.end())
            {
                (*i)->OnWndSize(m_wWidth, m_wHeight);
                ++i;
            }
            return 0;
        case WM_SYSCOMMAND:
            if(wParam == SC_MAXIMIZE)
            {
                SetFullscreen(true, m_wDefaultWidth, m_wDefaultHeight);
                return 0;
            }
            if(wParam == SC_RESTORE)
            {
                SetFullscreen(false, m_wDefaultWidth, m_wDefaultHeight);
                return 0;
            }
    }

    return DefWindowProc(m_hWnd, msg, wParam, lParam);
}
