/****************************************************************************\
*                                                                            *
* 15.04.2006                                                                 *
* CTerrainBuilder - Source File                                              *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CTerrainBuilder.h"
#include "CSettings.h"
#include "ITextureLoader.h"
#include "ModelStructs.h"
#include "CTerrainInfo.h"
#include <CEngineFactory.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CTerrainBuilder::BuildTerrain(const std::string &sLevelName, CTerrainInfo *pTerrainInfo,
                                   std::vector< TSmartPointer<CGeometryInfo> > &geometryInfos) const
{
    assert((bool) m_pTextureLoader && "Error: m_pTextureLoader is NULL in CTerrainBuilder::BuildTerrain");

    std::string sValue;

    //Get required parameters
    float fXSize, fZSize, fYScale;
    word wSegments;
    if(!CSettings::InstancePtr()->GetValue(sLevelName, "XSize", sValue))
    {
        LOG(TL_ERROR, "Error: Couldn't get \"XSize\"-Value from \"%s\" in CTerrainBuilder::BuildTerrain", sLevelName.c_str());
        return false;
    }
    fXSize = (float) strtod(sValue.c_str(), NULL);
    if(!CSettings::InstancePtr()->GetValue(sLevelName, "ZSize", sValue))
    {
        LOG(TL_ERROR, "Error: Couldn't get \"ZSize\"-Value from \"%s\" in CTerrainBuilder::BuildTerrain", sLevelName.c_str());
        return false;
    }
    fZSize = (float) strtod(sValue.c_str(), NULL);
    if(!CSettings::InstancePtr()->GetValue(sLevelName, "YScale", sValue))
    {
        LOG(TL_ERROR, "Error: Couldn't get \"YScale\"-Value from \"%s\" in CTerrainBuilder::BuildTerrain", sLevelName.c_str());
        return false;
    }
    fYScale = (float) strtod(sValue.c_str(), NULL);
    if(!CSettings::InstancePtr()->GetValue(sLevelName, "Segments", sValue))
    {
        LOG(TL_ERROR, "Error: Couldn't get \"Segments\"-Value from \"%s\" in CTerrainBuilder::BuildTerrain", sLevelName.c_str());
        return false;
    }
    wSegments = (word) strtoul(sValue.c_str(), NULL, 10);

    //Get Heightmap value and load the map
    if(!CSettings::InstancePtr()->GetValue(sLevelName, "Heightmap", sValue))
    {
        LOG(TL_ERROR, "Error: Couldn't get \"Heightmap\"-Value from \"%s\" in CTerrainBuilder::BuildTerrain", sLevelName.c_str());
        return false;
    }
    TSmartPointer<CImageInfo> pImageInfo = new CImageInfo;
    if(!m_pTextureLoader->LoadImage(sValue, pImageInfo))
    {
        LOG(TL_ERROR, "Error: m_pTextureLoader->LoadImage(%s) failed in CTerrainBuilder::BuildTerrain", sValue.c_str());
        return false;
    }
    if((pImageInfo->m_Width - 1)% wSegments || (pImageInfo->m_Height - 1) % wSegments)
    {
        LOG(TL_ERROR, "Error: Heightmap size (%u x %u) should be: (a multiple of %u) - 1", pImageInfo->m_Width, pImageInfo->m_Height, wSegments);
        return false;
    }

    pImageInfo->m_Bpp /= 8;

    //Get first terrain texture
    STexture tex1;
    if(!CSettings::InstancePtr()->GetValue(sLevelName, "TerrainTexture1", tex1.fileName))
    {
        LOG(TL_ERROR, "Error: Couldn't get \"TerrainTexture1\"-Value from \"%s\" in CTerrainBuilder::BuildTerrain", sLevelName.c_str());
        return false;
    }
    tex1.e_applyMode = eMODULATE;
	//Get second terrain texture
	STexture tex2;
	if(!CSettings::InstancePtr()->GetValue(sLevelName, "TerrainTexture2", tex2.fileName))
	{
		LOG(TL_ERROR, "Error: Couldn't get \"TerrainTexture2\"-Value from \"%s\" in CTerrainBuilder::BuildTerrain", sLevelName.c_str());
		return false;
	}
	tex2.e_applyMode = eMODULATE;

    //The created mesh per cell loos like this:
    // (5 vetices, 4 triangles)
    // x-----x
    // |\ A /|
    // | \ / |
    // |D x B|
    // | / \ |
    // |/ C \|
    // x-----x
    // If this is changed, CTerrainInfo NEEDS TO BE UPDATED TOO!
    
    //ToDo: SOME (A LOT?) OF OPTIMIZATION CAN HAPPEN HERE :-) (Profile first!!!)

    //Calculate size of each cell
    float fCellXSize = fXSize / (pImageInfo->m_Width - 1);
    float fCellZSize = fZSize / (pImageInfo->m_Height - 1);

    dword dwNumVertices = pImageInfo->m_Width * pImageInfo->m_Height + //All edge vertices
                          (pImageInfo->m_Width - 1) * (pImageInfo->m_Height - 1); //All Vertices in the center of the cells

    //Create buffer for all vetices
    CVector3f *pPos = new CVector3f[dwNumVertices]; 
    //Create buffer for uvs
    CVector2f *pUvs = new CVector2f[2 * dwNumVertices];
    //Calculate all vertices
    word w, h;
    dword dwOffset = 0;
    //First all edge vertices
    for(h = 0; h < pImageInfo->m_Height; ++h)
    {
        for(w = 0; w < pImageInfo->m_Width; ++w)
        {
             //We use only the first color value (red? blue?))
            pPos[dwOffset].Set(fCellXSize * w,
                               pImageInfo->m_pData[dwOffset * pImageInfo->m_Bpp] * fYScale,
                               (fCellZSize * h));
			pUvs[2 * dwOffset].Set(float(w)/float(pImageInfo->m_Width), float(h)/float(pImageInfo->m_Height));
            pUvs[2 * dwOffset + 1].Set((float)(w % 2), (float)(h % 2));
            ++dwOffset;
        }
    }

    //Now all center vertices
    dword dwBaseOffset = 0;
    for(h = 0; h < pImageInfo->m_Height - 1; ++h)
    {
        for(w = 0; w < pImageInfo->m_Width - 1; ++w)
        {
            pPos[dwOffset].Set(fCellXSize * w + fCellXSize / 2.0f,
                               (pPos[dwBaseOffset].m_Y +
                                    pPos[dwBaseOffset + 1].m_Y +
                                    pPos[dwBaseOffset + pImageInfo->m_Width].m_Y +
                                    pPos[dwBaseOffset + pImageInfo->m_Width + 1].m_Y) / 4.0f,
                               (fCellZSize * h + fCellZSize / 2.0f));
			pUvs[2 * dwOffset].Set((0.5f + w)/float(pImageInfo->m_Width), (0.5f + h)/float(pImageInfo->m_Height));
            pUvs[2 * dwOffset + 1].Set(0.5f, 0.5f);
            ++dwOffset;
            ++dwBaseOffset;
        }
        ++dwBaseOffset; //Increment once more per row, because the inner loop runs only pImageInfo->m_Width - 1 times
    }

    //Create index buffer
    dword dwNumIndices = 3 * 4 * (pImageInfo->m_Width - 1) * (pImageInfo->m_Height - 1);
    dword *pIndices = new dword[dwNumIndices];

    dwBaseOffset = 0;
    dword dwCenterOffset;
    for(h = 0; h < pImageInfo->m_Height - 1; ++h)
    {
        for(w = 0; w < pImageInfo->m_Width - 1; ++w)
        {
            dwOffset = w + (h  * pImageInfo->m_Width);
            dwCenterOffset = pImageInfo->m_Width * pImageInfo->m_Height + dwOffset - h;
            //Create the indices for this point (counter-clockwise)
            //A
            pIndices[dwBaseOffset++] = dwOffset;
            pIndices[dwBaseOffset++] = dwCenterOffset;
            pIndices[dwBaseOffset++] = dwOffset + 1;
            //B
            pIndices[dwBaseOffset++] = dwOffset + 1;
            pIndices[dwBaseOffset++] = dwCenterOffset;
            pIndices[dwBaseOffset++] = dwOffset + pImageInfo->m_Width + 1;
            //C
            pIndices[dwBaseOffset++] = dwOffset + pImageInfo->m_Width + 1;
            pIndices[dwBaseOffset++] = dwCenterOffset;
            pIndices[dwBaseOffset++] = dwOffset + pImageInfo->m_Width;
            //D
            pIndices[dwBaseOffset++] = dwOffset + pImageInfo->m_Width;
            pIndices[dwBaseOffset++] = dwCenterOffset;
            pIndices[dwBaseOffset++] = dwOffset;
        }
    }

    //Cook the mesh
    CPhysXMesh *pMesh = CEngineFactory::InstancePtr()->GetPhysicsMgr()->CookTerrain(dwNumVertices, dwNumIndices, pPos, pIndices, 1000);
    if(!pMesh)
    {
        LOG(TL_ERROR, "Error: CookTerrain failed in CTerrainBuilder::BuildTerrain");
        return false;
    }

    //Fill terrain info
    if(pTerrainInfo)
    {
        if(!pTerrainInfo->Create(pImageInfo->m_Width, pImageInfo->m_Height, pImageInfo->m_pData, pImageInfo->m_Bpp, fCellXSize, fCellZSize, fYScale, pMesh))
        {
            LOG(TL_ERROR, "Error: pTerrainInfo->Create failed in CTerrainBuilder::BuildTerrain");
            return false;
        }
    }

    //Create buffer for triangle normals
    CVector3f *pTriNormals = new CVector3f[4 * (pImageInfo->m_Width - 1) * (pImageInfo->m_Height - 1)];
    //Calculate triangle normals
    dwOffset = 0;     
    dwBaseOffset = 0;
    dwCenterOffset = pImageInfo->m_Width * pImageInfo->m_Height; //Set dwCenterOffset to the offset of the first center vetex
    for(h = 0; h < pImageInfo->m_Height - 1; ++h)
    {
        for(w = 0; w < pImageInfo->m_Width - 1; ++w)
        {
            //A
            pTriNormals[dwOffset] = (pPos[dwCenterOffset] - pPos[dwBaseOffset]) * 
									(pPos[dwBaseOffset + 1] - pPos[dwBaseOffset]);
            pTriNormals[dwOffset].Normalize();
            ++dwOffset;
            //B
            pTriNormals[dwOffset] = (pPos[dwCenterOffset] - pPos[dwBaseOffset + 1]) * 
									(pPos[dwBaseOffset + pImageInfo->m_Width + 1] - pPos[dwBaseOffset + 1]);
            pTriNormals[dwOffset].Normalize();
            ++dwOffset;
            //C
            pTriNormals[dwOffset] = (pPos[dwBaseOffset + pImageInfo->m_Width + 1] - pPos[dwBaseOffset + pImageInfo->m_Width]) * 
									(pPos[dwCenterOffset] - pPos[dwBaseOffset + pImageInfo->m_Width]);
            pTriNormals[dwOffset].Normalize();
            ++dwOffset;
            //D
            pTriNormals[dwOffset] = (pPos[dwBaseOffset + pImageInfo->m_Width] - pPos[dwBaseOffset]) * 
									(pPos[dwCenterOffset] - pPos[dwBaseOffset]);
            pTriNormals[dwOffset].Normalize();
            ++dwOffset;

            ++dwBaseOffset;
            ++dwCenterOffset;
        }
        ++dwBaseOffset; //Increment once more per row, because the inner loop runs only pImageInfo->m_Width - 1 times
    }

    //Create buffer for vertex normals
    CVector3f *pNormals = new CVector3f[pImageInfo->m_Width * pImageInfo->m_Height + //All edge vertices
                                       (pImageInfo->m_Width - 1) * (pImageInfo->m_Height - 1)]; //All Vertices in the center of the cells
    //Calculate Vertex normals
    //First all edge vertices
    dwOffset = 0;
    float fColor = 0.0f;
    for(h = 0; h < pImageInfo->m_Height; ++h)
    {
        for(w = 0; w < pImageInfo->m_Width; ++w)
        {
            dwBaseOffset = 4 * (w + h * (pImageInfo->m_Width - 1)); //Offset of the A-Triangle of the "next" cell
            pNormals[dwOffset].Set(0.0f, 0.0f, 0.0f);
            if(h != 0)
            {
                if(w != 0)
                {
                    //TopLeft B+C
                    pNormals[dwOffset] += pTriNormals[(dwBaseOffset - ((pImageInfo->m_Width - 1) * 4)) - 3];
                    pNormals[dwOffset] += pTriNormals[(dwBaseOffset - ((pImageInfo->m_Width - 1) * 4)) - 2];
                }
                if(w != pImageInfo->m_Width - 1)
                {
                    //TopRight C+D
                    pNormals[dwOffset] += pTriNormals[(dwBaseOffset - ((pImageInfo->m_Width - 1) * 4)) + 2];
                    pNormals[dwOffset] += pTriNormals[(dwBaseOffset - ((pImageInfo->m_Width - 1) * 4)) + 3];
                }
            }
            if(h != pImageInfo->m_Height - 1)
            {
                if(w != 0)
                {
                    //BottomLeft A+B
                    pNormals[dwOffset] += pTriNormals[dwBaseOffset - 4];
                    pNormals[dwOffset] += pTriNormals[dwBaseOffset - 3];
                }
                if(w != pImageInfo->m_Width - 1)
                {
                    //BottomRight A+D
                    pNormals[dwOffset] += pTriNormals[dwBaseOffset];
                    pNormals[dwOffset] += pTriNormals[dwBaseOffset + 3];
                }
            }

			pNormals[dwOffset].Normalize();
			
            ++dwOffset;
        }
    }

    //Now all center vertices
    dwBaseOffset = 0;
    for(h = 0; h < pImageInfo->m_Height - 1; ++h)
    {
        for(w = 0; w < pImageInfo->m_Width - 1; ++w)
        {
            //A
            pNormals[dwOffset] = pTriNormals[dwBaseOffset];
            ++dwBaseOffset;
            //B
            pNormals[dwOffset] += pTriNormals[dwBaseOffset];
            ++dwBaseOffset;
            //C
            pNormals[dwOffset] += pTriNormals[dwBaseOffset];
            ++dwBaseOffset;
            //D
            pNormals[dwOffset] += pTriNormals[dwBaseOffset];
            ++dwBaseOffset;

            pNormals[dwOffset].Normalize();
			
            ++dwOffset;
        }
    }

    //Calculate size of each segment (= each geometry info)
    word wSegmentWidth = (pImageInfo->m_Width - 1) / wSegments;
    word wSegmentHeight = (pImageInfo->m_Height - 1) / wSegments;

    //Create landscape material
    SMaterial mat;
    ZeroMemory(&mat, sizeof(mat));
    //Debug
    mat.ambientC.Set(0.0f, 0.0f, 0.0f, 0.0f);
    mat.diffuseC.Set(1.0f, 1.0f, 1.0f, 1.0f);
	mat.useLighting = true;
    //End debug

    //Resize the vector to fit all infos
    geometryInfos.resize(wSegments * wSegments);
    //Create the segments
    SGeoGroup *pGroup;
    dwBaseOffset = 0; //Abuse old offset variable for geoinfo offset
    for(word y = 0; y < wSegments; ++y)
    {
        for(word x = 0; x < wSegments; ++x)
        {
            //Create new CGeometryInfo for this segment
            geometryInfos[dwBaseOffset] = new CGeometryInfo;
            pGroup = geometryInfos[dwBaseOffset]->CreateGroup(2);
            pGroup->m_matIndex = geometryInfos[dwBaseOffset]->Insert(mat);
            pGroup->m_pTexIndices[0] = geometryInfos[dwBaseOffset]->Insert(tex1);
			pGroup->m_pTexIndices[1] = geometryInfos[dwBaseOffset]->Insert(tex2);
			pGroup->m_eApplyModes.push_back(tex1.e_applyMode);
			pGroup->m_eApplyModes.push_back(tex2.e_applyMode);

            for(word h = 0; h < wSegmentHeight; ++h)
            {
                for(word w = 0; w < wSegmentWidth; ++w)
                {
                    dwOffset = w + x * wSegmentWidth + (h + y * wSegmentHeight) * pImageInfo->m_Width;
                    dwCenterOffset = pImageInfo->m_Width * pImageInfo->m_Height + dwOffset - (h + y * wSegmentHeight);

                    //Create the cell for this point (counter-clockwise)
                    //A
                    pGroup->Insert(pPos[dwOffset], pNormals[dwOffset], &pUvs[2* dwOffset]);
                    pGroup->Insert(pPos[dwCenterOffset], pNormals[dwCenterOffset], &pUvs[2 * dwCenterOffset]);
                    pGroup->Insert(pPos[dwOffset + 1], pNormals[dwOffset + 1], &pUvs[2 * (dwOffset + 1)]);
                    //B
                    pGroup->Insert(pPos[dwOffset + 1], pNormals[dwOffset + 1], &pUvs[2 * (dwOffset + 1)]);
                    pGroup->Insert(pPos[dwCenterOffset], pNormals[dwCenterOffset], &pUvs[2 * dwCenterOffset]);
                    pGroup->Insert(pPos[dwOffset + pImageInfo->m_Width + 1], pNormals[dwOffset + pImageInfo->m_Width + 1], &pUvs[2 * (dwOffset + pImageInfo->m_Width + 1)]);
                    //C
                    pGroup->Insert(pPos[dwOffset + pImageInfo->m_Width + 1], pNormals[dwOffset + pImageInfo->m_Width + 1], &pUvs[2 * (dwOffset + pImageInfo->m_Width + 1)]);
                    pGroup->Insert(pPos[dwCenterOffset], pNormals[dwCenterOffset], &pUvs[2 * dwCenterOffset]);
                    pGroup->Insert(pPos[dwOffset + pImageInfo->m_Width], pNormals[dwOffset + pImageInfo->m_Width], &pUvs[2 * (dwOffset + pImageInfo->m_Width)]);
                    //D
                    pGroup->Insert(pPos[dwOffset + pImageInfo->m_Width], pNormals[dwOffset + pImageInfo->m_Width], &pUvs[2 * (dwOffset + pImageInfo->m_Width)]);
                    pGroup->Insert(pPos[dwCenterOffset], pNormals[dwCenterOffset], &pUvs[2 * dwCenterOffset]);
                    pGroup->Insert(pPos[dwOffset], pNormals[dwOffset], &pUvs[2 * dwOffset]);
               }
            }
            ++dwBaseOffset; //Increment abused variable
        }
    }

    SAFE_DELETE_ARRAY(pNormals);
    SAFE_DELETE_ARRAY(pTriNormals);
    SAFE_DELETE_ARRAY(pIndices);
    SAFE_DELETE_ARRAY(pUvs);
    SAFE_DELETE_ARRAY(pPos);

    return true;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/