/****************************************************************************\
*                                                                            *
* 01.06.2006                                                                 *
* COGLCGProgram - Source File                                               *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "COGLCGProgram.h"
#include "Renderer/COGLTriangleGroup.h"

#pragma comment(lib, "cg.lib")
#pragma comment(lib, "cgGL.lib")

//STATIC
CGcontext COGLCGProgram::s_context = NULL;
std::map<std::string,SProgramInfo *> COGLCGProgram::s_VertexPrograms;
std::map<std::string,SProgramInfo *> COGLCGProgram::s_FragmentPrograms;

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

// Rounds off a value to zero if it's VERY close to zero
#define ROUNDOFF(x) (x < 0.0001f && x > -0.0001f) ? 0.0f : x

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

void cgErrorCallback(){

	CGerror err = cgGetError();

    if (err != CG_NO_ERROR) {
		LOG(TL_ERROR, "CG Error: %s" ,cgGetErrorString(err));
	}
}

COGLCGProgram::COGLCGProgram(COGLTriangleGroup *pTri, COGLRenderer *pRenderer){
	if(!s_context){
		s_context = cgCreateContext();
		cgSetErrorCallback(cgErrorCallback);
	}

	m_shaderType        = eSHADER_NOT_SET;  
	m_pVertexProgInfo   = NULL;
	m_pFragmentProgInfo = NULL;
	m_pRenderer         = pRenderer;
	m_pGeometry         = pTri;
}

SProgramInfo * COGLCGProgram::CompileProgram(EProgramType eType, const std::string & fileName, const std::string & entryFunc){

	std::map<std::string,SProgramInfo *>::iterator tIt;

	switch(eType){
		case ePROGRAM_FRAGMENT:
			tIt = s_FragmentPrograms.find(fileName);
			if(tIt != s_FragmentPrograms.end()){
				(*tIt).second->IncReference();
				return (*tIt).second;
			}
			break;
		case ePROGRAM_VERTEX:
			tIt = s_VertexPrograms.find(fileName);
			if(tIt != s_VertexPrograms.end()){
				(*tIt).second->IncReference();
				return (*tIt).second;
			}
			break;
	}

	
	SProgramInfo * newInfo = new SProgramInfo(); 
	newInfo->m_fileName = fileName;

	switch (eType){
		case ePROGRAM_VERTEX:
			newInfo->m_Profile = cgGLGetLatestProfile(CG_GL_VERTEX);
			newInfo->m_eType = ePROGRAM_VERTEX;
			break;
		case ePROGRAM_FRAGMENT:
			newInfo->m_Profile = cgGLGetLatestProfile(CG_GL_FRAGMENT);
			newInfo->m_eType = ePROGRAM_FRAGMENT;
			break;
	}

	cgGLSetOptimalOptions(newInfo->m_Profile);
    
	newInfo->m_Program = cgCreateProgramFromFile(s_context,CG_SOURCE,fileName.c_str(),newInfo->m_Profile,entryFunc.c_str(),NULL); 
	
	if(newInfo->m_Program){
		cgGLLoadProgram(newInfo->m_Program);

		switch (eType){
			case ePROGRAM_VERTEX:
				s_VertexPrograms.insert(std::make_pair(fileName,newInfo));
				break;
			case ePROGRAM_FRAGMENT:
				s_FragmentPrograms.insert(std::make_pair(fileName,newInfo));
				break;
		}

		newInfo->IncReference();
		return newInfo;
	
	}else{
	
		delete newInfo;
		return NULL;

	}
}

COGLCGProgram::~COGLCGProgram(){

	if(m_pVertexProgInfo){
		if(m_pVertexProgInfo->Release()){
			
			std::map<std::string,SProgramInfo *>::iterator tIt;
			tIt = s_VertexPrograms.find(m_pVertexProgInfo->m_fileName);
			
			delete (*tIt).second;
			s_VertexPrograms.erase(tIt);
		}
	}

	if(m_pFragmentProgInfo){
		if(m_pFragmentProgInfo->Release()){

			std::map<std::string,SProgramInfo *>::iterator tIt;
			tIt = s_FragmentPrograms.find(m_pFragmentProgInfo->m_fileName);

			delete (*tIt).second;
			s_FragmentPrograms.erase(tIt);
		}
	}

	if( !(s_FragmentPrograms.size() || s_VertexPrograms.size())){
		cgDestroyContext(s_context);
	}
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/

void COGLCGProgram::CalculateVertexTangents(dword vertexCount,dword *pIndices,CVector3f *pCoords,CVector3f * pNormals,CVector2f * pUvs,CVector4f * pTangent){

	CVector3f *tan1 = new CVector3f[vertexCount*2];
	CVector3f *tan2 = tan1 + vertexCount;
	ZeroMemory(tan1, vertexCount * sizeof(CVector3f) * 2);

	int vIndex1;
	int vIndex2;
	int vIndex3;
	CVector3f v1v2, v1v3, sdir, tdir;
	CVector2f s1s2, s1s3;
	float r;

	for(int i = 0 ; i < (int)vertexCount ; i+=3){
		vIndex1 = pIndices[i];
		vIndex2 = pIndices[i+1];
		vIndex3 = pIndices[i+2];

		v1v2 = pCoords[vIndex2] - pCoords[vIndex1];
		v1v3 = pCoords[vIndex3] - pCoords[vIndex1];
		s1s2 = pUvs[vIndex2]    - pUvs[vIndex1];
		s1s3 = pUvs[vIndex3]    - pUvs[vIndex1];

		//TODO - catch NAN
		r = 1.0F / (s1s2.m_X * s1s3.m_Y - s1s3.m_X * s1s2.m_Y);

		sdir.Set((s1s3.m_Y * v1v2.m_X - s1s2.m_Y * v1v3.m_X) * r, 
			     (s1s3.m_Y * v1v2.m_Y - s1s2.m_Y * v1v3.m_Y) * r,
			     (s1s3.m_Y * v1v2.m_Z - s1s2.m_Y * v1v3.m_Z) * r);
		tdir.Set((s1s2.m_X * v1v3.m_X - s1s3.m_X * v1v2.m_X) * r, 
			     (s1s2.m_X * v1v3.m_Y - s1s3.m_X * v1v2.m_Y) * r,
			     (s1s2.m_X * v1v3.m_Z - s1s3.m_X * v1v2.m_Z) * r); 

		tan1[vIndex1] += sdir;
		tan1[vIndex2] += sdir;
		tan1[vIndex3] += sdir;

		tan2[vIndex1] += tdir;
		tan2[vIndex2] += tdir;
		tan2[vIndex3] += tdir;
	}

	int vertInd;
	CVector3f n,t,res,zero;
	float handedness;

	
	for (long i = 0; i < (int)vertexCount ; i++)
	{
		vertInd = pIndices[i];
		n = pNormals[vertInd];
		t = tan1[vertInd];

		// Gram-Schmidt orthogonalize
		res = t - (n*(n.Dot(t)));
		
		if(res != zero)
			res.Normalize();
		else
			res.Set(1.0f,0.0f,0.0f);
		
		handedness = ((n * t).Dot(tan2[vertInd]) < 0.0F) ? -1.0F : 1.0F;

		pTangent[i].Set(res.m_X,res.m_Y,res.m_Z,handedness);
	}
	delete [] tan1;

}