/****************************************************************************\
*                                                                            *
* 18.03.2006                                                                 *
* COGLModelFactory - Source File                                                *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Renderer/COGLModelFactory.h>
#include <Renderer/COGLStockGeometry.h>
#include <Renderer/IRenderer.h>
#include <Renderer/COGLRenderer.h>
#include <CEngineFactory.h>

#include <glew.h>

#include <Renderer/COGLRenderInfo.h>
#include <Renderer/coglcg/COGLCGSimpleBump.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
COGLModelFactory::COGLModelFactory(TSmartPointer<IModelLoader> pModelLoader,
                                   TSmartPointer<ITextureLoader> pTextureLoader,
                                   TSmartPointer<CTerrainBuilder> pTerrainBuilder)
                                   : IModelFactory(pModelLoader,pTextureLoader, pTerrainBuilder)
{
}

COGLModelFactory::~COGLModelFactory(){
	m_currentTextures.clear();
	m_currentMG.clear();
}


/************************\
* getStockObject()       *
\************************/
TSmartPointer<IStockGeometry> COGLModelFactory::getStockObject(ESTOCK_OBJECT eType, const std::string & texFileName,
													 float topLeftX, float topLeftY, float bottomRightX, float bottomRightY,
													 const CVector4f *color, EBlendingMode blendingMode)
{

	bool useTexture = false;
	word newDispListNmb = glGenLists(1);
	
    COGLTexture *newTexture = NULL;
	
    if(!(texFileName.size() == 0))
    {
        newTexture = new COGLTexture();
		newTexture->m_target = GL_TEXTURE_2D;
		newTexture->m_bMipMap = false;
		newTexture->m_minFilter = GL_LINEAR;
		newTexture->m_maxFilter = GL_LINEAR;

		if(CreateTexture(newTexture,texFileName)){
            useTexture = true;
        }else{
            SAFE_DELETE(newTexture);
        }
    }

	glNewList(newDispListNmb,GL_COMPILE);
	
	if(useTexture)
	{
		glActiveTexture(GL_TEXTURE0);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D,newTexture->getTexBufID());
	}

	switch(eType){

		case eQUAD:
			glBegin(GL_TRIANGLES);
				glColor4fv((float*)color);
				if(useTexture)glTexCoord2f(topLeftX,topLeftY);
				glVertex3f(0.0f, 0.0f, 0.0f);
				if(useTexture)glTexCoord2f(topLeftX,bottomRightY);
				glVertex3f(0.0f, 1.0f, 0.0f);
				if(useTexture)glTexCoord2f(bottomRightX,topLeftY);
				glVertex3f(1.0f, 0.0f, 0.0f);
				if(useTexture)glTexCoord2f(topLeftX,bottomRightY);
				glVertex3f(0.0f, 1.0f, 0.0f);
				if(useTexture)glTexCoord2f(bottomRightX,bottomRightY);
				glVertex3f(1.0f, 1.0f, 0.0f);
				if(useTexture)glTexCoord2f(bottomRightX,topLeftY);
				glVertex3f(1.0f, 0.0f, 0.0f);
			glEnd();
			break;		
		case eCIRCLE:
			LOG(TL_ERROR, "Error: eCIRCLE not implemented in COGLModelFactory::getStockObject");
	}

	if(useTexture){
		glDisable(GL_TEXTURE_2D);
		SAFE_DELETE(newTexture);
	}

	glEndList();

	return new COGLStockGeometry((COGLRenderer *)((IRenderer *)CEngineFactory::InstancePtr()->GetRenderer()),this,newDispListNmb, blendingMode);
}


/************************\
* getModel()             *
\************************/
TSmartPointer<IModelGeometry> COGLModelFactory::getModel(const std::string & modelFileName)
{
	
	std::string sModelName(modelFileName);
	//lowercase all characters
	transform(sModelName.begin(), sModelName.end(),	//source
			   sModelName.begin(),							//destination
			   tolower);									//operation

	//Backslashes durch slashes ersetzen
	for(dword i = 0; i < sModelName.length(); ++i)
	{
		if(sModelName[i] == '\\')
			sModelName[i] = '/';
	}

	std::map<std::string,TSmartPointer<COGLModelGeometry> >::iterator it;
	//check if it is already loaded
	it = m_currentMG.find(sModelName);
	
	if(it != m_currentMG.end()){
		return (IModelGeometry*)(COGLModelGeometry*)(*it).second;
	}else{

		LOG(TL_DEBUG | TL_STDOUT, "Loading new Model : %s",sModelName.c_str());
	
		//check if the loader can load it
		if(m_pModelLoader->Load(sModelName)){

			TSmartPointer<CGeometryInfo> pNewGeoInfo = new CGeometryInfo();
			if(!m_pModelLoader->GetGeometryInfo(pNewGeoInfo)){
				pNewGeoInfo = NULL;
				return NULL;
			}

			TSmartPointer<COGLModelGeometry> pNewMG = BuildMG(pNewGeoInfo);
			if(!pNewMG){
				//cleanup
				pNewGeoInfo = NULL;
				pNewMG = NULL;
				return NULL;
			}else{
				//delete the intermediate structure 
				pNewGeoInfo = NULL;

                //safe the MG
				m_currentMG.insert(std::make_pair(sModelName,pNewMG));
				//return it
				return (IModelGeometry*)(COGLModelGeometry*)pNewMG;
			}
		
		}else{
			return NULL; 
		}
	}
}


/************************\
* CreateTerrain()        *
\************************/
bool COGLModelFactory::CreateTerrain(const std::string &sLevel, CTerrainInfo *pTerrainInfo,
									 std::vector< TSmartPointer<IModelGeometry> > &models)
{
	std::vector< TSmartPointer<CGeometryInfo> > pGeoInfos;
	if(!m_pTerrainBuilder->BuildTerrain(sLevel, pTerrainInfo, pGeoInfos))
	{
		LOG(TL_ERROR, "Error: m_pTerrainBuilder->BuildTerrain failed for level %s", sLevel.c_str());
		return false;
	}

	models.reserve(pGeoInfos.size());
	std::vector< TSmartPointer<CGeometryInfo> >::iterator it;
	
	TSmartPointer<COGLModelGeometry> pNewTerrainPartMG;

	//clear the Reference to the current TerrainModelGeometries
	m_vCurrentTerrainMG.clear();

	for(it = pGeoInfos.begin(); it != pGeoInfos.end(); ++it)
	{
		pNewTerrainPartMG = BuildMG((*it));
		models.push_back((IModelGeometry*)(COGLModelGeometry*)pNewTerrainPartMG);
		m_vCurrentTerrainMG.push_back(pNewTerrainPartMG);
	}
	return true;
}


/************************\
* CreateEnv()            *
\************************/
bool COGLModelFactory::CreateEnv(const std::string *texNames,byte nmb)
{
	COGLTexture newTexture;
	std::map<std::string,TSmartPointer<COGLModelGeometry> >::iterator it;

	switch(nmb){
		//Sphere
		case 1:
			newTexture.m_target    = GL_TEXTURE_2D;
			newTexture.m_bMipMap   = true;
			newTexture.m_minFilter = GL_LINEAR_MIPMAP_LINEAR;
			newTexture.m_maxFilter = GL_LINEAR;

			if(CreateTexture(&newTexture,texNames[0])){
				//set to sphere
				m_currentEnvironment.setSphere(newTexture.getTexBufID());

				COGLTriangleGroup * cTriGroup;

				//for every model stored
				for(it = m_currentMG.begin() ; it != m_currentMG.end() ; it++){

					//for every TriangleGroup in the Model
					for(int i = 0 ; i < (int)(*it).second->m_TriangleGroups.size() ; i++){

						cTriGroup = (*it).second->getTriangleGroup(i);

						//create Reflection Layer
						float reflectivity = (*it).second->getMaterial(cTriGroup->getMaterialIndex())->m_reflectivity;

						if( reflectivity > 0){

							COGLReflectionMapTexLayer *rmtl = cTriGroup->createGlobalReflTexLayer();
							rmtl->setSphere(newTexture.getTexBufID());
							rmtl->setReflectivity(reflectivity);

							//recompile
							if(cTriGroup->getRenderInfo()->m_eCacheMode == OGL_CACHE_MODE_EXECUTE){
								cTriGroup->getRenderInfo()->m_eCacheMode = OGL_CACHE_MODE_DO_COMPILE;
								((COGLRenderer *)(IRenderer *)CEngineFactory::InstancePtr()->GetRenderer())->CompileList(cTriGroup,(*it).second);
								cTriGroup->getRenderInfo()->m_eCacheMode = OGL_CACHE_MODE_EXECUTE;
							}
						}
					}
				}
				return true;
			}else{
				return false;
			}
			//Cube
		case 6:
			for(int i = 0 ; i< nmb ; i++){
				newTexture.m_target    = GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT + i;
				newTexture.m_bMipMap   = false;
				newTexture.m_minFilter = GL_LINEAR;
				newTexture.m_maxFilter = GL_LINEAR;

				if(!CreateTexture(&newTexture,texNames[i])){
					LOG(TL_ERROR,"Could not establish CUBE_MAP_TEXTURE_OBJECTS in COGLModelFactory::CreateEnv()");
					return false;
				}
			}
			//set to cube
			m_currentEnvironment.setCube();

			COGLTriangleGroup * cTriGroup;

			//for every model stored
			for(it = m_currentMG.begin() ; it != m_currentMG.end() ; it++){

				//for every TriangleGroup in the Model
				for(int i = 0 ; i < (int)(*it).second->m_TriangleGroups.size() ; i++){

					cTriGroup = (*it).second->getTriangleGroup(i);

					//create Reflection Layer
					float reflectivity = (*it).second->getMaterial(cTriGroup->getMaterialIndex())->m_reflectivity;

					if( reflectivity > 0){

						COGLReflectionMapTexLayer *rmtl = cTriGroup->createGlobalReflTexLayer();
						cTriGroup->createGlobalReflTexLayer()->setCube();
						rmtl->setReflectivity(reflectivity);

						//recompile
						if(cTriGroup->getRenderInfo()->m_eCacheMode == OGL_CACHE_MODE_EXECUTE){
							cTriGroup->getRenderInfo()->m_eCacheMode = OGL_CACHE_MODE_DO_COMPILE;
							((COGLRenderer *)(IRenderer *)CEngineFactory::InstancePtr()->GetRenderer())->CompileList(cTriGroup,(*it).second);
							cTriGroup->getRenderInfo()->m_eCacheMode = OGL_CACHE_MODE_EXECUTE;
						}
					}
				}
			}
			break;

		default:
			LOG(TL_ERROR,"Must specify eighter 1 or 6 Textures for Environment Mapping in COGLModelFactory::CreateEnv()");
			return false;
	}
	return true;
}


/**************************\
* RecompileAll()           * 
\**************************/
dword COGLModelFactory::RecompileAll(EOGLRenderMode eRenderMode){

	dword nmbFailedVBO     = 0;
	dword nmbFailedCompile = 0;
	dword nmbCreatedVBO    = 0;
	dword nmbCompiledList  = 0;

	EOGLRenderMode eMode;

	//CURRENT MODELS

	for(mg_it = m_currentMG.begin() ; mg_it != m_currentMG.end() ; ++mg_it){

		//for every TriangleGroup in the Model
		for(int i = 0 ; i < (int)(*mg_it).second->m_TriangleGroups.size() ; i++){

			//set the rendermode to the original mode
			if(eRenderMode == OGL_RENDER_MODE_UNKNOWN){
				eMode = (*mg_it).second->getTriangleGroup(i)->getRenderInfo()->m_eRenderMode;
			}else{
				eMode = eRenderMode;
				if(eMode == OGL_RENDER_MODE_VERTEXBUFFEROBJECT){
					if(!(*mg_it).second->getTriangleGroup(i)->getRenderInfo()->m_bIsVertexBuffered){
						if(!(*mg_it).second->getTriangleGroup(i)->Build(eMode,true)){
							nmbFailedVBO++;
						}else{
							nmbCreatedVBO++;
						}
					}
				}
			}
				
			if(!(*mg_it).second->getTriangleGroup(i)->Build(eMode,true)){
				LOG(TL_ERROR,"Error Creating RenderMode Override in CModelFactory::RecompileAll()");
			}

			if((*mg_it).second->getTriangleGroup(i)->getRenderInfo()->m_eCacheMode != OGL_CACHE_MODE_NOT_COMPILABLE){
				//compile
				if((*mg_it).second->getTriangleGroup(i)->Compile(eMode,true)){
					nmbCompiledList++;
				}else{
					nmbFailedCompile++;
				}
			}
		}
	}

	//CURRENT TERRAIN

	for(mgT_it = m_vCurrentTerrainMG.begin() ; mgT_it != m_vCurrentTerrainMG.end() ; ++mgT_it){

		//for every TriangleGroup in the Model
		for(int i = 0 ; i < (int)(*mgT_it)->m_TriangleGroups.size() ; i++){

			//set the rendermode to the original mode
			if(eRenderMode == OGL_RENDER_MODE_UNKNOWN){
				eMode = (*mgT_it)->getTriangleGroup(i)->getRenderInfo()->m_eRenderMode;
			}else{
				eMode = eRenderMode;
				if(eMode == OGL_RENDER_MODE_VERTEXBUFFEROBJECT){
					if(!(*mgT_it)->getTriangleGroup(i)->getRenderInfo()->m_bIsVertexBuffered){
						if(!(*mgT_it)->getTriangleGroup(i)->Build(eMode,true)){
							nmbFailedVBO++;
						}else{
							nmbCreatedVBO++;
						}
					}
				}
			}

			if(!(*mgT_it)->getTriangleGroup(i)->Build(eMode,true)){
				LOG(TL_ERROR,"Error Creating RenderMode Override in CModelFactory::RecompileAll()");
			}

			//compile
			if((*mgT_it)->getTriangleGroup(i)->getRenderInfo()->m_eCacheMode != OGL_CACHE_MODE_NOT_COMPILABLE){
				if((*mgT_it)->getTriangleGroup(i)->Compile(eMode,true)){
					nmbCompiledList++;
				}else{
					nmbFailedCompile++;
				}
			}

		}
	}

	LOG(TL_DEBUG,"nmbFailedVBO: %d\nnmbFailedCompile: %d\nnmbCreatedVBO: %d\nnmbCompiledList: %d",nmbFailedVBO,nmbFailedCompile,nmbCreatedVBO,nmbCompiledList);
	
	return nmbFailedCompile + nmbFailedVBO;
}

/**************************\
* Recompile()              * 
\**************************/
dword COGLModelFactory::Recompile(EOGLRenderMode eRenderMode){

	dword nmbFailedVBO     = 0;
	dword nmbFailedCompile = 0;
	dword nmbCreatedVBO    = 0;
	dword nmbCompiledList  = 0;

	EOGLRenderMode eMode;

	//CURRENT MODELS

	for(mg_it = m_currentMG.begin() ; mg_it != m_currentMG.end() ; ++mg_it){

		//for every TriangleGroup in the Model
		for(int i = 0 ; i < (int)(*mg_it).second->m_TriangleGroups.size() ; i++){

			//set the rendermode to the original mode
			if(eRenderMode == OGL_RENDER_MODE_UNKNOWN){
				eMode = (*mg_it).second->getTriangleGroup(i)->getRenderInfo()->m_eRenderMode;
			}else{
				eMode = eRenderMode;
				if(eRenderMode == OGL_RENDER_MODE_VERTEXBUFFEROBJECT){
					if(!(*mg_it).second->getTriangleGroup(i)->getRenderInfo()->m_bIsVertexBuffered){
						if(!(*mg_it).second->getTriangleGroup(i)->Build(eMode,true)){
							nmbFailedVBO++;
						}else{
							nmbCreatedVBO++;
						}
					}
				}
			}
					
			if((*mg_it).second->getTriangleGroup(i)->getRenderInfo()->m_eCacheMode == OGL_CACHE_MODE_EXECUTE){

				if(!(*mg_it).second->getTriangleGroup(i)->Build(eMode,true)){
					LOG(TL_ERROR,"Error Creating RenderMode Override in CModelFactory::RecompileAll()");
				}

				//compile
				if((*mg_it).second->getTriangleGroup(i)->getRenderInfo()->m_eCacheMode != OGL_CACHE_MODE_NOT_COMPILABLE){
					if((*mg_it).second->getTriangleGroup(i)->Compile(eMode,true)){
						nmbCompiledList++;
					}else{
						nmbFailedCompile++;
					}
				}

			}
		}
	}

	//CURRENT TERRAIN

	for(mgT_it = m_vCurrentTerrainMG.begin() ; mgT_it != m_vCurrentTerrainMG.end() ; ++mgT_it){

		//for every TriangleGroup in the Model
		for(int i = 0 ; i < (int)(*mgT_it)->m_TriangleGroups.size() ; i++){

			//set the rendermode to the original mode
			if(eRenderMode == OGL_RENDER_MODE_UNKNOWN){
				eMode = (*mgT_it)->getTriangleGroup(i)->getRenderInfo()->m_eRenderMode;
			}else{
				eMode = eRenderMode;
				if(eMode == OGL_RENDER_MODE_VERTEXBUFFEROBJECT){
					if(!(*mgT_it)->getTriangleGroup(i)->getRenderInfo()->m_bIsVertexBuffered){
						if(!(*mgT_it)->getTriangleGroup(i)->Build(eMode,true)){
							nmbFailedVBO++;
						}else{
							nmbCreatedVBO++;
						}
					}
				}
			}

			if((*mgT_it)->getTriangleGroup(i)->getRenderInfo()->m_eCacheMode == OGL_CACHE_MODE_EXECUTE){

				if(!(*mgT_it)->getTriangleGroup(i)->Build(eMode,true)){
					LOG(TL_ERROR,"Error Creating RenderMode Override in CModelFactory::RecompileAll()");
				}

				//compile
				if((*mgT_it)->getTriangleGroup(i)->getRenderInfo()->m_eCacheMode != OGL_CACHE_MODE_NOT_COMPILABLE){
					if((*mgT_it)->getTriangleGroup(i)->Compile(eMode,true)){
						nmbCompiledList++;
					}else{
						nmbFailedCompile++;
					}
				}

			}
		}
	}

	LOG(TL_DEBUG,"nmbFailedVBO: %d\nnmbFailedCompile: %d\nnmbCreatedVBO: %d\nnmbCompiledList: %d",nmbFailedVBO,nmbFailedCompile,nmbCreatedVBO,nmbCompiledList);

	return nmbFailedCompile + nmbFailedVBO;
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
COGLModelGeometry * COGLModelFactory::BuildMG(CGeometryInfo *pGeoInfo)
{
	//Create the ModelGeometry
	//***********************************************************************************
	COGLModelGeometry *pModelGeometry = new COGLModelGeometry();

    if(!pModelGeometry->Create((COGLRenderer *)((IRenderer *)CEngineFactory::InstancePtr()->GetRenderer()),this, pGeoInfo))
	{
		SAFE_DELETE(pModelGeometry);
		return NULL;
	}
	
	//copy Material Data
	//***********************************************************************************
	dword nmb;
	COGLMaterial * oglMaterialPtr;
	std::vector<SMaterial>::iterator itM;

	for(nmb = 0, itM = pGeoInfo->m_Materials.begin(); itM != pGeoInfo->m_Materials.end(); ++itM, ++nmb)
	{
		oglMaterialPtr = pModelGeometry->getMaterial(nmb);

		oglMaterialPtr->m_ambientC[0]   = (float) (*itM).ambientC[0];
		oglMaterialPtr->m_ambientC[1]   = (float) (*itM).ambientC[1];
		oglMaterialPtr->m_ambientC[2]   = (float) (*itM).ambientC[2];
		oglMaterialPtr->m_ambientC[3]   = (float) (*itM).ambientC[3];
		oglMaterialPtr->m_diffuseC[0]   = (float) (*itM).diffuseC[0];
		oglMaterialPtr->m_diffuseC[1]   = (float) (*itM).diffuseC[1];
		oglMaterialPtr->m_diffuseC[2]   = (float) (*itM).diffuseC[2];
		oglMaterialPtr->m_diffuseC[3]   = (float) (*itM).diffuseC[3];
		oglMaterialPtr->m_selfIllumC[0] = (float) (*itM).selfIllumC[0];
		oglMaterialPtr->m_selfIllumC[1] = (float) (*itM).selfIllumC[1];
		oglMaterialPtr->m_selfIllumC[2] = (float) (*itM).selfIllumC[2];
		oglMaterialPtr->m_selfIllumC[3] = (float) (*itM).selfIllumC[3];
		oglMaterialPtr->m_specularC[0]  = (float) (*itM).specularC[0];
		oglMaterialPtr->m_specularC[1]  = (float) (*itM).specularC[1];
		oglMaterialPtr->m_specularC[2]  = (float) (*itM).specularC[2];
		oglMaterialPtr->m_specularC[3]  = (float) (*itM).specularC[3];

		//materialProperties specified by extra Group Node
		oglMaterialPtr->m_exp           = (float) (*itM).exp;
		oglMaterialPtr->m_opacity       = (float) (*itM).opacity;
		oglMaterialPtr->m_reflectivity  = (float) (*itM).reflectivity;
		oglMaterialPtr->m_useLighting   =         (*itM).useLighting;
	}

	//copy Texture Data
	//***********************************************************************************
	sword ind;
	std::vector<STexture>::iterator it;

	for( ind = 0 , it = pGeoInfo->m_Textures.begin() ; it != pGeoInfo->m_Textures.end() ; ++it , ++ind ){
		pModelGeometry->getTexture(ind)->m_target = GL_TEXTURE_2D;
		//mipmap default - trilinear filtering
		if(true){
			pModelGeometry->getTexture(ind)->m_bMipMap   = true;
			pModelGeometry->getTexture(ind)->m_minFilter = GL_LINEAR_MIPMAP_LINEAR;
			pModelGeometry->getTexture(ind)->m_maxFilter = GL_LINEAR;
		//no mipmap - bilinear filtering
		}else{
			pModelGeometry->getTexture(ind)->m_bMipMap   = false;
			pModelGeometry->getTexture(ind)->m_minFilter = GL_LINEAR;
			pModelGeometry->getTexture(ind)->m_maxFilter = GL_LINEAR;
		}
		if(!CreateTexture(pModelGeometry->getTexture(ind),(*it).fileName)){
			LOG(TL_ERROR,"Could not create Texture : ",pModelGeometry->getTexture(ind)->getFilename().c_str());
		};
	}
	
	//copy TriangleGroup Data
	//***********************************************************************************
	COGLTriangleGroup * oglTriangleGroupPtr;
	CVector3f         * oglCoordPtr;
	CVector3f         * oglNormalPtr;
	CVector4f         * oglColorPtr;
	dword             * oglIndexArrayPtr;
	
	std::vector<SGeoGroup *>::iterator itG;
	for(nmb = 0, itG = pGeoInfo->m_pGeoGroups.begin() ; itG != pGeoInfo->m_pGeoGroups.end() ; ++itG ,++nmb )
	{
		oglTriangleGroupPtr = pModelGeometry->getTriangleGroup(nmb);
		oglCoordPtr         = oglTriangleGroupPtr->getCoordArray();
		oglNormalPtr        = oglTriangleGroupPtr->getNormalArray();
		oglIndexArrayPtr    = oglTriangleGroupPtr->getIndexArray();
		        		
		dword numVertices  = (dword)(*itG)->m_Vertices.size();
		dword numIndices   = (dword)(*itG)->m_Indices.size();
		bool  hasVC        =        (*itG)->m_Colors.size() > 0;
		word  numTexLayers = (word) (*itG)->m_bNumTexLayers;
		dword i;byte j;

		//set MaterialIndex-----------------------------------------------------------------
		oglTriangleGroupPtr->setMaterialIndex((*itG)->m_matIndex);
				
		//copy Indices----------------------------------------------------------------------
		for(i = 0 ; i < numIndices ; ++i){
			oglIndexArrayPtr[i] = (*itG)->m_Indices[i];
		}

		//copy Coord and Normals------------------------------------------------------------
		for(i = 0 ; i < numVertices ; ++i){
			oglCoordPtr[i]  = (*itG)->m_Vertices[i];
			oglNormalPtr[i] = (*itG)->m_Normals[i];
		}

		//copy Colors-----------------------------------------------------------------------
		if(hasVC){
			oglColorPtr = oglTriangleGroupPtr->getVertexColorArray();
			for(i = 0 ; i < numVertices ; ++i){
				oglColorPtr[i] = (*itG)->m_Colors[i];
			}
		}

		//copy TextureIndices and Uvs-------------------------------------------------------
		CVector2f *oglUvPtr;
		for(j = 0 ; j < numTexLayers ; ++j){
			oglTriangleGroupPtr->getTexLayer(j)->setTextureIndex((*itG)->m_pTexIndices[j]);
			
			switch((*itG)->m_eApplyModes[j]){
				case eREPLACE:
					oglTriangleGroupPtr->getTexLayer(j)->setBlendMode(e_TBM_REPLACE);
				break;
				case eMODULATE:
					oglTriangleGroupPtr->getTexLayer(j)->setBlendMode(e_TBM_MODULATE);
				break;
				case eADDITIVE:
					oglTriangleGroupPtr->getTexLayer(j)->setBlendMode(e_TBM_ADD);
				break;
                
				default:
					oglTriangleGroupPtr->getTexLayer(j)->setBlendMode(e_TBM_REPLACE);
			}
			
			oglUvPtr = oglTriangleGroupPtr->getTexLayer(j)->getUvArray();
			for(i = 0; i < numVertices ; ++i){
				oglUvPtr[i] = (*itG)->m_pUvs[j][i];
			}
		}

		//assign the reflection layer if environment present---------------------------------
		oglMaterialPtr = pModelGeometry->getMaterial((*itG)->m_matIndex);
		
		if(oglMaterialPtr->m_reflectivity > 0){
			
			COGLReflectionMapTexLayer *rmtl = oglTriangleGroupPtr->createGlobalReflTexLayer();
			switch(m_currentEnvironment.m_Type){
				case eRefl_Sphere:
					rmtl->setSphere(m_currentEnvironment.m_TexBufID);
					rmtl->setReflectivity(oglMaterialPtr->m_reflectivity);
					break;
				case eRefl_Cube:
					rmtl->setCube();
					rmtl->setReflectivity(oglMaterialPtr->m_reflectivity);
					break;
			}
		}
	
		//get Shader Info---------------------------------------------------------------------
		SShader *tmp;
		bool shaderAssigned = false;
		
		//check the if a Shader is specified
		if(pGeoInfo->m_Materials[oglTriangleGroupPtr->getMaterialIndex()].shader.shaderFileName.size() != 0){
			
			tmp = &pGeoInfo->m_Materials[oglTriangleGroupPtr->getMaterialIndex()].shader;
			
			//simpleBump
			if(tmp->shaderFileName.compare("simpleBump.cg") == 0){
				if((int)tmp->params.size() >= 1)
				if( tmp->params[0].paramType == eParam_String)
				if( tmp->params[0].name.compare("texNormal") == 0){

					if(numTexLayers >= 1){
				
						COGLCGSimpleBump *bump = new COGLCGSimpleBump(oglTriangleGroupPtr,(COGLRenderer *)(IRenderer *)CEngineFactory::InstancePtr()->GetRenderer());
							
						COGLTexture newTexture;
						newTexture.m_target    = GL_TEXTURE_2D;
						newTexture.m_bMipMap   = true;
						newTexture.m_minFilter = GL_LINEAR_MIPMAP_LINEAR;
						newTexture.m_maxFilter = GL_LINEAR;	

						std::string shaderPath = "Data/shader/";
						std::string *texFileName = (std::string *)tmp->params[0].value;
						texFileName->append(".tga");
						shaderPath.append(*texFileName);

						if(CreateTexture(&newTexture,shaderPath)){
							if(bump->Create(newTexture.m_glTexBufID,pModelGeometry->getTexture(oglTriangleGroupPtr->getTexLayer(0)->getTextureIndex())->getTexBufID())){
								oglTriangleGroupPtr->getRenderInfo()->setCGShader(bump);
								shaderAssigned = true;
							}
						}else{
							delete bump;
						}
					}
				}
			}
		}

		if(!shaderAssigned){

			//ASSIGN THE SHADINGSTATE----------------------------------------------------------------
			if(pModelGeometry->m_Materials[oglTriangleGroupPtr->getMaterialIndex()]->m_useLighting){
				oglTriangleGroupPtr->getRenderInfo()->m_shadingState = OGL_RENDER_STATE_USE_LIGHTING;
			}else{
				if(hasVC){
					oglTriangleGroupPtr->getRenderInfo()->m_shadingState = OGL_RENDER_STATE_USE_VERTEXCOLORS;	
				}else{
					oglTriangleGroupPtr->getRenderInfo()->m_shadingState = OGL_RENDER_STATE_USE_COLOR;
				}
			}
			if(numTexLayers > 0){
				oglTriangleGroupPtr->getRenderInfo()->m_shadingState |= OGL_RENDER_STATE_USE_TEXTURING;
			}

			//SET THE RENDERMODE AND CACHEMODE--------------------------------------------------------

			bool           cached = true;
			EOGLRenderMode eMode  = OGL_RENDER_MODE_VERTEXARRAY;

			if(cached){
				oglTriangleGroupPtr->Compile(eMode,false);
			}else{
				oglTriangleGroupPtr->Build(eMode,false);
			}
        		
		}else{

			//ASSIGN THE SHADINGSTATE----------------------------------------------------------------
			oglTriangleGroupPtr->getRenderInfo()->m_shadingState = OGL_RENDER_STATE_USE_COLOR;
			
			//SET THE RENDERMODE --------------------------------------------------------------------
			oglTriangleGroupPtr->BuildShaderMode(false);
			
			//SET THE CACHEMODE----------------------------------------------------------------------
			oglTriangleGroupPtr->getRenderInfo()->m_eCacheMode = OGL_CACHE_MODE_NOT_COMPILABLE;

		}

		//CHECK GLOBAL OVERRIDES 
        COGLGlobalRenderSettings *gS = (COGLGlobalRenderSettings *)(IGlobalRenderSettings *)CEngineFactory::InstancePtr()->GetRenderer()->GetGlobalSettings();

		if(gS != NULL){

			//check GLOBAL RENDERMODE----------------------------------------------------------------
			if(gS->m_bGlobalRenderModeStateIsActive){
				if(!oglTriangleGroupPtr->Build(gS->m_RenderModeOverride,true)){
					LOG(TL_ERROR,"New TriangleGroup - Error Creating TriGroup in Global RenderMode in COGLModelFactory::BuildMG()");
				}else{
					LOG(TL_DEBUG,"New TriangleGroup - Creating TriGroup in Global RenderMode in COGLModelFactory::BuildMG()");
				}
			}
			
			//check GLOBAL CACHEMODE-----------------------------------------------------------------
			if(!shaderAssigned){
				if(gS->m_bGlobalCacheModeStateIsActive){
					if(gS->m_CacheModeOverride == OGL_CACHE_MODE_EXECUTE){
						if(!oglTriangleGroupPtr->Compile(oglTriangleGroupPtr->getRenderInfo()->m_eRenderMode,true)){
							LOG(TL_ERROR,"New TriangleGroup - Error Compiling Global CMO in COGLModelFactory::BuildMG()");
						}else{
							LOG(TL_ERROR,"New TriangleGroup - Compiling Global CMO in COGLModelFactory::BuildMG()");
						}
					}
				}
			}

		}
	}
	//return
	return pModelGeometry;
}


/************************\
* createTexture()        *
\************************/
bool COGLModelFactory::CreateTexture(COGLTexture *tex, const std::string &fileName){

	//check if a texture with this filename is already loaded
	std::map<std::string,TSmartPointer<CImageInfo> >::iterator tIt;
	
	tIt = m_currentTextures.find(fileName);

	if(tIt != m_currentTextures.end())
    {
		tex->setFileName((*tIt).second->fileName);
		tex->setTexBufID((*tIt).second->m_ID);
		return true;
	}else{

		//check if the tex Object was initialized
		if(!(tex->m_target && tex->m_maxFilter && tex->m_minFilter)){
			LOG(TL_ERROR,"Texture Properties not specified in COGLModelFactory::createTexture()");
			return false;
		}

		TSmartPointer<CImageInfo> pImageInfo = new CImageInfo();
		pImageInfo->fileName = fileName;

		//load the image Data
		if(m_pTextureLoader->LoadImage(pImageInfo->fileName,pImageInfo)){

			//create textureObject out of it
			if( ((COGLRenderer *)(IRenderer *)CEngineFactory::InstancePtr()->GetRenderer())->CreateTexture(pImageInfo,tex) ){

				//save the data
				pImageInfo->m_ID = tex->getTexBufID();
				pImageInfo->m_bMipMaps = tex->m_bMipMap;
				m_currentTextures.insert(std::make_pair(pImageInfo->fileName,pImageInfo));
				return true;

			}
		}
		pImageInfo = NULL;
		return false;
	}
}

void COGLModelFactory::SetGlobalTextureQuality(EOGLTextureQuality eQuality){

	for(mT_it = m_currentTextures.begin() ; mT_it != m_currentTextures.end() ; ++mT_it){
		
		glBindTexture(GL_TEXTURE_2D,(*mT_it).second->m_ID);
		
		switch(eQuality){
			case OGL_TQ_LINEAR:
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
				break;
			case OGL_TQ_BILINEAR:
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
				break;
			case OGL_TQ_TRILINEAR:
				if((*mT_it).second->m_bMipMaps){
					glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
					glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
				}
				break;
		}
	}
}

void COGLModelFactory::SetGlobalMipMapping(bool bEnable){

	for(mT_it = m_currentTextures.begin() ; mT_it != m_currentTextures.end() ; ++mT_it){

		glBindTexture(GL_TEXTURE_2D,(*mT_it).second->m_ID);

		if(bEnable){
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAX_LEVEL,1000);
		}else{
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAX_LEVEL,0);
		}
	}
}