/****************************************************************************\
*                                                                            *
* 18.03.2006                                                                 *
* COGLRenderer - Source File                                                 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "COGLRenderer.h"

#include <glew.h>

#include <Display/Windows/IWindow.h>
#include <Renderer/COGLModelGeometry.h>

#include <Renderer/COGLRenderInfo.h>

#pragma comment(lib, "glew32.lib")
#pragma comment(lib, "OpenGL32.lib")
#pragma comment(lib, "Glu32.lib")

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool COGLRenderer::Create(const TSmartPointer<IWindow> &pWindow)
{
    assert((bool) pWindow && "Error: pWindow is NULL in COGLRenderer::Create");

    m_pWindow = pWindow;

	int iPixelFormat;
    PIXELFORMATDESCRIPTOR pfd =				// pfd Tells Windows How We Want Things To Be
    {
        sizeof(PIXELFORMATDESCRIPTOR),		// Size Of This Pixel Format Descriptor
            1,								// Version Number
            PFD_DRAW_TO_WINDOW |			// Format Must Support Window
            PFD_SUPPORT_OPENGL |			// Format Must Support OpenGL
            PFD_DOUBLEBUFFER,				// Must Support Double Buffering
            PFD_TYPE_RGBA,					// Request An RGBA Format
            m_pWindow->GetBitsPerPixel(),	// Select Our Color Depth
            0, 0, 0, 0, 0, 0,				// Color Bits Ignored
            0,								// No Alpha Buffer
            0,								// Shift Bit Ignored
            0,								// No Accumulation Buffer
            0, 0, 0, 0,						// Accumulation Bits Ignored
            16,								// 16Bit Z-Buffer (Depth Buffer)  
            0,								// No Stencil Buffer
            0,								// No Auxiliary Buffer
            PFD_MAIN_PLANE,					// Main Drawing Layer
            0,								// Reserved
            0, 0, 0							// Layer Masks Ignored
    };

    m_hDC = ::GetDC(m_pWindow->GetWindowHandle());

    if(!m_hDC)
    {
        LOG(TL_ERROR, "Error: ::GetDC failed in COGLRenderer::Create");
        return false;
	}

    iPixelFormat = ::ChoosePixelFormat(m_hDC, &pfd);

    if(!iPixelFormat)
    {
        LOG(TL_ERROR, "Error: ::ChoosePixelFormat failed in COGLRenderer::Create");
        return false;
    }

    if(!::SetPixelFormat(m_hDC, iPixelFormat, &pfd))
    {
        LOG(TL_ERROR, "Error: ::SetPixelFormat failed in COGLRenderer::Create");
        return false;
    }

    m_hGLRC = ::wglCreateContext(m_hDC);

    if(!m_hGLRC)
    {
        LOG(TL_ERROR, "Error: ::wglCreateContext failed in COGLRenderer::Create");
        return false;
    }

    if(!::wglMakeCurrent(m_hDC, m_hGLRC))
    {
        LOG(TL_ERROR, "Error: ::wglMakeCurrent failed in COGLRenderer::Create");
        return false;
    }

	//Info
	glGetString(GL_EXTENSIONS);
	glGetString(GL_VERSION);
	glGetString(GL_VENDOR);

	//Initialize Glew
	GLenum err = glewInit();
	if (GLEW_OK != err) 
    {
		LOG(TL_ERROR,"glewInit() Failed in COGLRenderer.Create()");
		LOG(TL_ERROR,"Error: %s", glewGetErrorString(err));
	}
	//check for Vertex_Buffer_Object Support
	if (GLEW_ARB_vertex_buffer_object){
		m_bVertexBufferSupported = true;
	}
	//check for Texture_Cube_Map Support
	if(GLEW_EXT_texture_cube_map){
		m_bTextureCubeMapSupported = true;
	}

	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	//Depth
	EnableZTest(true);
	EnableZWrite(true);
	glDepthFunc(GL_LEQUAL);
	//ShadingModel
	glShadeModel(GL_SMOOTH);
	//Culling
    glCullFace(GL_BACK);
    glFrontFace(GL_CCW);
    glEnable(GL_CULL_FACE);
	//Debug : Light
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER,GL_TRUE);
	glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL,GL_SEPARATE_SPECULAR_COLOR);
	glEnable(GL_LIGHT0);
	m_vLightDiffuseColor = CVector4f(1.0f,1.0f,1.0f,1.0f);
	m_vLightDiffuseColor = CVector4f(0.0f,0.0f,0.0f,0.0f);
	//Matrix
	SetMatrixMode(EMM_PROJECTION);
	//Hint
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	//StipplePattern
	if(!m_pPattern){
		
		m_pPattern = new byte[32*32];
		
		dword rowskip = 0;
		for(int i = 0; i < 16 ; i++){
			rowskip = i*64;
			for(int j = 0; j < 32; j++){
				m_pPattern[rowskip + j] = 0x11;
			}
			for(int k = 32; k < 64 ; k++){
				m_pPattern[rowskip + k] = 0x88;
			}
		}
	}
	glPolygonStipple(m_pPattern);

	//Blending
	m_bBlendEnableFlag = true;

	//Register Handler
    m_pWindow->RegisterHandler(this);
    return true;
}

TSmartPointer<IFont> COGLRenderer::GetFontObject(const std::string &sFontFace, byte bSize, bool bBold, bool bItalic)
{	
	FontObjectList::iterator it;
	for(it = m_FontObjects.begin(); it != m_FontObjects.end(); ++it){
		if((*it)->m_bBold == bBold    && (*it)->m_bItalic == bItalic &&
           (*it)->m_fontSize == bSize && (*it)->m_fontFace.compare(sFontFace) == 0)
		return (TSmartPointer<IFont>)(*it);
	}
		
	int iWeight = ( bBold ? FW_BOLD : FW_REGULAR );
	
    HFONT hFont = ::CreateFont(
		-bSize,							// Height Of Font
        0,								// Width Of Font
        0,								// Angle Of Escapement
        0,								// Orientation Angle
        iWeight,						// Font Weight
        bItalic,						// Italic
        FALSE,							// Underline
        FALSE,							// Strikeout
        ANSI_CHARSET,					// Character Set Identifier
        OUT_TT_PRECIS,					// Output Precision
        CLIP_DEFAULT_PRECIS,			// Clipping Precision
        ANTIALIASED_QUALITY,			// Output Quality
        FF_DONTCARE | DEFAULT_PITCH,    // Family And Pitch
        sFontFace.c_str());				// Font Name
	
	if(!hFont)
    {
        LOG(TL_ERROR, "Error: Couldn't create font %s (%u, %x, %x)", sFontFace.c_str(), (word) bSize, (word) bBold, (word) bItalic);
        return NULL;
	}

	TSmartPointer<COGLFont> fontPtr(new COGLFont(this));
		
	fontPtr->m_fontFace  = sFontFace;
	fontPtr->m_bBold     = bBold;
	fontPtr->m_bItalic   = bItalic;
	fontPtr->m_fontSize  = bSize;

	int iFirstChar = int(' ');
	fontPtr->m_Quantity = 256 - iFirstChar; //Create full extended ascii list (except the first 31 control chars)
	fontPtr->m_Start = ::glGenLists(fontPtr->m_Quantity);
	fontPtr->m_Base = fontPtr->m_Start - iFirstChar;
    HFONT hOldFont;
    hOldFont = (HFONT)::SelectObject(m_hDC, hFont);
    ::wglUseFontBitmaps(m_hDC, iFirstChar, fontPtr->m_Quantity, fontPtr->m_Start);
    ::SelectObject(m_hDC, hOldFont);
	::DeleteObject(hFont);

	m_FontObjects.push_back(fontPtr);

	return (TSmartPointer<IFont>)fontPtr;
}

void COGLRenderer::RenderText(COGLFont *pFont, SText *pText)
{
	GLboolean bDepthTest = glIsEnabled(GL_DEPTH_TEST);
	GLboolean bLighting = glIsEnabled(GL_LIGHTING);
	GLboolean bTexture2D = glIsEnabled(GL_TEXTURE_2D);
	if(bDepthTest)
		glDisable(GL_DEPTH_TEST);
	if(bLighting)
		glDisable(GL_LIGHTING);
	if(bTexture2D)
		glDisable(GL_TEXTURE_2D);
	SetMatrixMode(EMM_MODELVIEW);
	SaveMatrix();
	glColor4fv(pText->m_textColor);
	glRasterPos3f(pText->m_vPos.m_X, pText->m_vPos.m_Y + pFont->m_fontSize, pText->m_vPos.m_Z);
	glListBase(pFont->m_Base);
	glCallLists((GLsizei)pText->m_text.size(), GL_UNSIGNED_BYTE, pText->m_text.c_str());
	RestoreMatrix();
	if(bDepthTest)
		glEnable(GL_DEPTH_TEST);
	if(bLighting)
		glEnable(GL_LIGHTING);
	if(bTexture2D)
		glEnable(GL_TEXTURE_2D);
}


void COGLRenderer::RenderStockObject(COGLStockGeometry *obj){

	if(m_bBlendEnableFlag){
		if(obj->m_BlendingMode == EBM_ALPHA)
		{
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		}
		else if(obj->m_BlendingMode == EBM_COLOR)
		{
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_COLOR, GL_ONE_MINUS_SRC_COLOR);
		}
		glCallList(obj->m_dispListNmb);
		if(obj->m_BlendingMode == EBM_ALPHA || obj->m_BlendingMode == EBM_COLOR)
		{
			glDisable(GL_BLEND);
		}
	}else{
		glCallList(obj->m_dispListNmb);
	}

}


bool COGLRenderer::CreateTexture(const CImageInfo * const pImageInfo, COGLTexture * tex){

	GLuint texBufID = 0;

	glGenTextures(1,&texBufID);

	if(tex->m_target != GL_TEXTURE_2D){
		glBindTexture(GL_TEXTURE_CUBE_MAP_ARB,texBufID);
	}else{
		glBindTexture(GL_TEXTURE_2D,texBufID);
	}

	//mipmap or normal 
	if(tex->m_bMipMap){

		if (pImageInfo->m_bAlpha){
			gluBuild2DMipmaps(tex->m_target, GL_RGBA8, pImageInfo->m_Width, pImageInfo->m_Height, GL_BGRA_EXT, GL_UNSIGNED_BYTE,pImageInfo->m_pData);
		}else{
			gluBuild2DMipmaps(tex->m_target, GL_RGB8, pImageInfo->m_Width, pImageInfo->m_Height, GL_BGR_EXT, GL_UNSIGNED_BYTE,pImageInfo->m_pData);
		}

	}else{

		if (pImageInfo->m_bAlpha){
			glTexImage2D(tex->m_target,0,GL_RGBA8, pImageInfo->m_Width,pImageInfo->m_Height,0, GL_BGRA_EXT, GL_UNSIGNED_BYTE,pImageInfo->m_pData);
		}else{
			glTexImage2D(tex->m_target,0,GL_RGB8, pImageInfo->m_Width,pImageInfo->m_Height,0, GL_BGR_EXT, GL_UNSIGNED_BYTE,pImageInfo->m_pData);
		}

	}

	dword glError;

	if(tex->m_target == GL_TEXTURE_2D){
		if(!glIsTexture(texBufID)){
			return false;
		}
	}else{
		if( (glError = glGetError()) != GL_NO_ERROR){
			LOG(TL_ERROR, "OpenGL-Error: \"%s\"", (const char *)gluErrorString(glError));
			ReportErrors();
			//debug
			return true;
		}
	}

	glTexParameteri(tex->m_target, GL_TEXTURE_MIN_FILTER, tex->m_minFilter);
	glTexParameteri(tex->m_target, GL_TEXTURE_MAG_FILTER, tex->m_maxFilter);

	tex->setFileName(pImageInfo->fileName);
	tex->setTexBufID(texBufID);

	return true;
}

bool COGLRenderer::ComplieVBO(COGLTriangleGroup *obj){

	if(!m_bVertexBufferSupported){
		return false;
	}

	//vertexBuffer
	glGenBuffersARB(1, &obj->m_pCoordArray.m_ID);
	glBindBufferARB(GL_ARRAY_BUFFER_ARB,obj->m_pCoordArray.m_ID);
	glBufferDataARB(GL_ARRAY_BUFFER_ARB,obj->m_NumVertices*3*sizeof(float),obj->m_pCoordArray.m_DataArray,GL_STATIC_DRAW_ARB);

	if(obj->m_RenderInfo->m_shadingState & OGL_RENDER_STATE_USE_LIGHTING){
		glGenBuffersARB(1, &obj->m_pNormalArray.m_ID);
		glBindBufferARB(GL_ARRAY_BUFFER_ARB,obj->m_pNormalArray.m_ID);
		glBufferDataARB(GL_ARRAY_BUFFER_ARB,obj->m_NumVertices*3*sizeof(float),obj->m_pNormalArray.m_DataArray,GL_STATIC_DRAW_ARB);
	}

	if(obj->m_RenderInfo->m_shadingState &  OGL_RENDER_STATE_USE_VERTEXCOLORS){
		glGenBuffersARB(1, &obj->m_pVertexColorArray.m_ID);
		glBindBufferARB(GL_ARRAY_BUFFER_ARB,obj->m_pVertexColorArray.m_ID);
		glBufferDataARB(GL_ARRAY_BUFFER_ARB,obj->m_NumVertices*4*sizeof(float),obj->m_pVertexColorArray.m_DataArray,GL_STATIC_DRAW_ARB);
	}

	if(obj->m_RenderInfo->m_shadingState &  OGL_RENDER_STATE_USE_TEXTURING){
		for(int i = 0 ; i < (int)obj->m_TextureLayers.size() ; i++){
			glGenBuffersARB(1, &obj->m_TextureLayers[i]->m_pUvs.m_ID);
			glBindBufferARB(GL_ARRAY_BUFFER_ARB,obj->m_TextureLayers[i]->m_pUvs.m_ID);
			glBufferDataARB(GL_ARRAY_BUFFER_ARB,obj->m_NumVertices*2*sizeof(float),obj->m_TextureLayers[i]->m_pUvs.m_DataArray,GL_STATIC_DRAW_ARB);
		}
	}

	//indices
	glGenBuffersARB(1, &obj->m_pIndexArray.m_ID);
	glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB,obj->m_pIndexArray.m_ID);
	glBufferDataARB(GL_ELEMENT_ARRAY_BUFFER_ARB,obj->m_NumIndices*sizeof(dword),obj->m_pIndexArray.m_DataArray,GL_STATIC_DRAW_ARB);

	//unbind
	glBindBufferARB(GL_ARRAY_BUFFER_ARB,0);
	glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB,0);
	
	return true;
}

bool COGLRenderer::CompileList(COGLTriangleGroup *pTriGroup, COGLModelGeometry *pMg){

	m_pActiveMG       = pMg;
	m_pActiveTriGroup = pTriGroup;

    if(!glIsList(m_pActiveTriGroup->m_RenderInfo->m_dispListNmb)){
		m_pActiveTriGroup->m_RenderInfo->m_dispListNmb = glGenLists(1);
	}

	glNewList(m_pActiveTriGroup->m_RenderInfo->m_dispListNmb,GL_COMPILE);

		switch(m_pActiveTriGroup->m_RenderInfo->m_eCurrentRenderMode){
			case OGL_RENDER_MODE_VERTEXARRAY:
				PushState();
					RenderVertexArrays();
				PopState();
				break;
			case OGL_RENDER_MODE_MEGASLOW:
				PushState();
					RenderMegaSlow();
				PopState();
				break;
			case OGL_RENDER_MODE_VERTEXBUFFEROBJECT:
				PushState();
					RenderVertexBufferObjects();
				PopState();
				break;
			default:
				glEndList();
				glDeleteLists(m_pActiveTriGroup->m_RenderInfo->m_dispListNmb,1);
				LOG(TL_ERROR, "Error: Unknown model typ in COGLRenderer::Compile()");
				return false;
		}

	glEndList();

	return (glIsList(m_pActiveTriGroup->m_RenderInfo->m_dispListNmb) != 0);
}

void COGLRenderer::Render(COGLModelGeometry * obj){

	m_pActiveMG = obj;

	for(m_i = 0 ; m_i < (int)obj->m_TriangleGroups.size() ; ++m_i ){

		m_pActiveTriGroup = obj->getTriangleGroup(m_i);

		//CHECK IF GLOBAL CACHEMODE OVERRIDE IS SET
		if(m_pGlobalRenderSettings->m_bGlobalCacheModeStateIsActive){
			
			//if ------ALL MODELS------ should be rendered as DisplayLists and the CURRENT MODEL can be rendered as a Displaylist
			if(m_pGlobalRenderSettings->m_CacheModeOverride == OGL_CACHE_MODE_EXECUTE && m_pActiveTriGroup->m_RenderInfo->m_eCurrentCacheMode == OGL_CACHE_MODE_EXECUTE){
						
				//The Global RenderMode is assumed to be compiled into the List
				if(!m_pGlobalRenderSettings->m_bGlobalRenderStateIsActive){
					glCallList(m_pActiveTriGroup->m_RenderInfo->m_dispListNmb);
					continue;
				}else{

					PushState();
						RenderVertexArrays();
					PopState();

				}

			//if ------NO MODELS------ models should be rendered as a DisplayList
			}else{

				//CHECK IF GLOBAL RENDERMODE OVERRIDE IS SET
				if(m_pGlobalRenderSettings->m_bGlobalRenderModeStateIsActive){

					switch(m_pGlobalRenderSettings->m_RenderModeOverride){
						case OGL_RENDER_MODE_VERTEXARRAY:
							PushState();
								RenderVertexArrays();
							PopState();
							break;
						case OGL_RENDER_MODE_MEGASLOW:
							PushState();	
								RenderMegaSlow();
							PopState();
							break;
						case OGL_RENDER_MODE_VERTEXBUFFEROBJECT:
							PushState();
								RenderVertexBufferObjects();
							PopState();
							break;
						default:
							LOG(TL_ERROR, "Error: Unknown GLOBAL RenderMode in COGLRenderer::Render()");
					}

				}else{

					switch(m_pActiveTriGroup->m_RenderInfo->m_eRenderMode){
						case OGL_RENDER_MODE_VERTEXARRAY:
							PushState();
								RenderVertexArrays();
							PopState();
							break;
						case OGL_RENDER_MODE_MEGASLOW:
							PushState();
								RenderMegaSlow();
							PopState();
							break;
						case OGL_RENDER_MODE_VERTEXBUFFEROBJECT:
							PushState();
								RenderVertexBufferObjects();
							PopState();
							break;
						case OGL_RENDER_MODE_SHADER:
							RenderShader();
							break;
						default:
							LOG(TL_ERROR, "Error: Unknown RenderMode in COGLRenderer::Render()");
					}
					
				}
			}

		}else{

			//CHECK THE CACHE MODE FOR THIS MODEL
			switch(m_pActiveTriGroup->m_RenderInfo->m_eCacheMode){

				case OGL_CACHE_MODE_EXECUTE:

				if(!m_pGlobalRenderSettings->m_bGlobalRenderStateIsActive){
					glCallList(m_pActiveTriGroup->m_RenderInfo->m_dispListNmb);
					continue;
				}else{

					PushState();
						RenderVertexArrays();
					PopState();

				}

			}

			//CHECK IF GLOBAL RENDERMODE OVERRIDE IS SET
			if(m_pGlobalRenderSettings->m_bGlobalRenderModeStateIsActive){

				switch(m_pGlobalRenderSettings->m_RenderModeOverride){
						case OGL_RENDER_MODE_VERTEXARRAY:
							PushState();
								RenderVertexArrays();
							PopState();
							break;
						case OGL_RENDER_MODE_MEGASLOW:
							PushState();
								RenderMegaSlow();
							PopState();
							break;
						case OGL_RENDER_MODE_VERTEXBUFFEROBJECT:
							PushState();
								RenderVertexBufferObjects();
							PopState();
							break;
						default:
							LOG(TL_ERROR, "Error: Unknown GLOBAL RenderMode in COGLRenderer::Render()");
				}
			
			}else{

				switch(m_pActiveTriGroup->m_RenderInfo->m_eRenderMode){
						case OGL_RENDER_MODE_VERTEXARRAY:
							PushState();
								RenderVertexArrays();
							PopState();
							break;
						case OGL_RENDER_MODE_MEGASLOW:
							PushState();
								RenderMegaSlow();
							PopState();
							break;
						case OGL_RENDER_MODE_VERTEXBUFFEROBJECT:
							PushState();
								RenderVertexBufferObjects();
							PopState();
							break;
						case OGL_RENDER_MODE_SHADER:
							RenderShader();
							break;
						default:
							LOG(TL_ERROR, "Error: Unknown RenderMode in COGLRenderer::Render()");
				}

			}
		}
	}
}


void COGLRenderer::PushState(){

	static float amb[4] = {0.4f,0.4f,0.4f,1.0f};

	m_pActiveCoordPtr   = m_pActiveTriGroup->m_pCoordArray.m_DataArray;
	m_pActiveIndexArray = m_pActiveTriGroup->m_pIndexArray.m_DataArray;
	m_ActiveState       = m_pActiveTriGroup->m_RenderInfo->m_shadingState;

	if(!m_pGlobalRenderSettings->m_bGlobalRenderStateIsActive){

		switch(m_ActiveState){

			case OGL_SHADE_STATE_LIGHTING_TEXTURING:

				//Lighting
				glEnable(GL_LIGHTING);

				m_pActiveMat       = m_pActiveMG->m_Materials[m_pActiveTriGroup->m_MaterialIndex];
				m_pActiveNormalPtr = m_pActiveTriGroup->m_pNormalArray.m_DataArray;

				//glMaterialfv(GL_FRONT,GL_AMBIENT,(float*)&(m_pActiveMat->m_ambientC));
				glMaterialfv(GL_FRONT,GL_DIFFUSE,(float*)&(m_pActiveMat->m_diffuseC));
				glMaterialfv(GL_FRONT,GL_SPECULAR,(float*)&(m_pActiveMat->m_specularC));
				glMateriali(GL_FRONT,GL_SHININESS,(int)m_pActiveMat->m_exp);

				//DEBUG
				glMaterialfv(GL_FRONT,GL_AMBIENT,amb);

				
					
				//Texturing
				m_ActiveNmbOfTexLayers = (int)m_pActiveTriGroup->m_TextureLayers.size();

				for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; m_l++ ){
					glActiveTexture(GL_TEXTURE0 + m_l);
					glEnable(GL_TEXTURE_2D);
					glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,m_pActiveTriGroup->m_TextureLayers[m_l]->m_texureBlendMode);
					glBindTexture(GL_TEXTURE_2D,m_pActiveMG->m_Textures[m_pActiveTriGroup->m_TextureLayers[m_l]->m_textureIndex]->m_glTexBufID);
				}

			break;

			case OGL_SHADE_STATE_COLOR_TEXTURING:

				//White Color
				glColor4f(1.0f,1.0f,1.0f,1.0f);

				//Texturing
				m_ActiveNmbOfTexLayers = (int)m_pActiveTriGroup->m_TextureLayers.size();

				for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; m_l++ ){
					glActiveTexture(GL_TEXTURE0 + m_l);
					glEnable(GL_TEXTURE_2D);
					glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,m_pActiveTriGroup->m_TextureLayers[m_l]->m_texureBlendMode);
					glBindTexture(GL_TEXTURE_2D,m_pActiveMG->m_Textures[m_pActiveTriGroup->m_TextureLayers[m_l]->m_textureIndex]->m_glTexBufID);
				}

			break;

			case OGL_SHADE_STATE_VERTEXCOLOR_TEXTURING:

				//VertexColor Pointer
				m_pActiveColorPtr = m_pActiveTriGroup->m_pVertexColorArray.m_DataArray;
				
				///Texturing
				m_ActiveNmbOfTexLayers = (int)m_pActiveTriGroup->m_TextureLayers.size();

				for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; m_l++ ){
					glActiveTexture(GL_TEXTURE0 + m_l);
					glEnable(GL_TEXTURE_2D);
					glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,m_pActiveTriGroup->m_TextureLayers[m_l]->m_texureBlendMode);
					glBindTexture(GL_TEXTURE_2D,m_pActiveMG->m_Textures[m_pActiveTriGroup->m_TextureLayers[m_l]->m_textureIndex]->m_glTexBufID);
				}

			break;

			case OGL_SHADE_STATE_LIGHTING:

				//Lighting
				glEnable(GL_LIGHTING);

				m_pActiveMat       = m_pActiveMG->m_Materials[m_pActiveTriGroup->m_MaterialIndex];
				m_pActiveNormalPtr = m_pActiveTriGroup->m_pNormalArray.m_DataArray;

				//glMaterialfv(GL_FRONT,GL_AMBIENT,(float*)&(m_pActiveMat->m_ambientC));
				glMaterialfv(GL_FRONT,GL_DIFFUSE,(float*)&(m_pActiveMat->m_diffuseC));
				glMaterialfv(GL_FRONT,GL_SPECULAR,(float*)&(m_pActiveMat->m_specularC));
				glMateriali(GL_FRONT,GL_SHININESS,(int)m_pActiveMat->m_exp);

				//DEBUG
				glMaterialfv(GL_FRONT,GL_AMBIENT,amb);

			break;

			case OGL_SHADE_STATE_VERTEXCOLOR:

				//VertexColor Pointer
				m_pActiveColorPtr = m_pActiveTriGroup->m_pVertexColorArray.m_DataArray;

			break;

			case OGL_SHADE_STATE_COLOR:

				//White Color
				glColor4f(1.0f,1.0f,1.0f,1.0f);

			break;
		}

		//check ReflectionLayer
		if(m_pActiveTriGroup->m_pGlobalReflTexLayer){

			if(m_ActiveState & OGL_RENDER_STATE_USE_TEXTURING){
				glActiveTexture(GL_TEXTURE0 + m_l + 1);
			}else{
				glActiveTexture(GL_TEXTURE0);
			}

			glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D,m_pActiveTriGroup->m_pGlobalReflTexLayer->m_TexBufID);

			glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_COMBINE);
			glTexEnvi(GL_TEXTURE_ENV,GL_COMBINE_RGB,GL_INTERPOLATE);
			
			glTexEnvfv(GL_TEXTURE_ENV,GL_TEXTURE_ENV_COLOR,(float*)&m_pActiveTriGroup->m_pGlobalReflTexLayer->m_mixColor);
			glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE0_RGB,GL_PREVIOUS);
			glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE1_RGB,GL_TEXTURE);
			glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE2_RGB,GL_CONSTANT);

			glTexEnvi(GL_TEXTURE_ENV,GL_OPERAND2_RGB,GL_SRC_ALPHA);

			//DEBUG
			//glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_ADD);

			glTexGeni(GL_S,GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
			glTexGeni(GL_T,GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
			glEnable(GL_TEXTURE_GEN_S);
			glEnable(GL_TEXTURE_GEN_T);
		}


	}else{

		m_ActiveState = 0;
		
		if(m_ActiveState & OGL_RENDER_STATE_USE_TEXTURING && (m_pGlobalRenderSettings->m_GlobalRenderState & OGL_RENDER_STATE_USE_TEXTURING)){

			m_ActiveState |= OGL_RENDER_STATE_USE_TEXTURING;
			
			m_ActiveNmbOfTexLayers = (int)m_pActiveTriGroup->m_TextureLayers.size();

			for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; m_l++ ){
				glActiveTexture(GL_TEXTURE0 + m_l);
				glEnable(GL_TEXTURE_2D);
				glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,m_pActiveTriGroup->m_TextureLayers[m_l]->m_texureBlendMode);
				glBindTexture(GL_TEXTURE_2D,m_pActiveMG->m_Textures[m_pActiveTriGroup->m_TextureLayers[m_l]->m_textureIndex]->m_glTexBufID);
			}

			//blend in the reflectionLayer 
			if(m_pActiveTriGroup->m_pGlobalReflTexLayer){

				glActiveTexture(GL_TEXTURE0 + m_l + 1);
				
				glEnable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D,m_pActiveTriGroup->m_pGlobalReflTexLayer->m_TexBufID);

				glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_COMBINE);
				glTexEnvi(GL_TEXTURE_ENV,GL_COMBINE_RGB,GL_INTERPOLATE);
				glTexEnvfv(GL_TEXTURE_ENV,GL_TEXTURE_ENV_COLOR,(float*)&m_pActiveTriGroup->m_pGlobalReflTexLayer->m_mixColor);

				glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE0_RGB,GL_PREVIOUS);
				glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE1_RGB,GL_TEXTURE);
				glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE2_RGB,GL_CONSTANT);

				glTexEnvi(GL_TEXTURE_ENV,GL_OPERAND2_RGB,GL_SRC_ALPHA);

				glTexGeni(GL_S,GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
				glTexGeni(GL_T,GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
				glEnable(GL_TEXTURE_GEN_S);
				glEnable(GL_TEXTURE_GEN_T);
			}
		}

		if(m_ActiveState & OGL_RENDER_STATE_USE_LIGHTING && (m_pGlobalRenderSettings->m_GlobalRenderState & OGL_RENDER_STATE_USE_LIGHTING)){

			m_ActiveState |= OGL_RENDER_STATE_USE_LIGHTING;

			glEnable(GL_LIGHTING);

			m_pActiveMat = m_pActiveMG->m_Materials[m_pActiveTriGroup->m_MaterialIndex];
			m_pActiveNormalPtr = m_pActiveTriGroup->m_pNormalArray.m_DataArray;

			//glMaterialfv(GL_FRONT,GL_AMBIENT,(float*)&(m_pActiveMat->m_ambientC));
			glMaterialfv(GL_FRONT,GL_DIFFUSE,(float*)&(m_pActiveMat->m_diffuseC));
			glMaterialfv(GL_FRONT,GL_SPECULAR,(float*)&(m_pActiveMat->m_specularC));
			//glMateriali(GL_FRONT,GL_SHININESS,(int)m_pActiveMat->m_exp);

			//DEBUG
			glMaterialfv(GL_FRONT,GL_AMBIENT,amb);

		}else{
            
			if(m_ActiveState & OGL_RENDER_STATE_USE_VERTEXCOLORS && (m_pGlobalRenderSettings->m_GlobalRenderState & OGL_RENDER_STATE_USE_VERTEXCOLORS)){
					
				m_ActiveState |= OGL_RENDER_STATE_USE_VERTEXCOLORS;

				m_pActiveColorPtr = m_pActiveTriGroup->m_pVertexColorArray.m_DataArray;

			}else{

				m_ActiveState |= OGL_RENDER_STATE_USE_COLOR;

				glColor4fv((float *)&m_pGlobalRenderSettings->m_globalDrawColor);
			
			}
		}
	}
}

void COGLRenderer::PopState(){

	if(!m_pGlobalRenderSettings->m_bGlobalRenderStateIsActive){

		switch(m_ActiveState){

			case OGL_SHADE_STATE_LIGHTING_TEXTURING:

				glDisable(GL_LIGHTING);

				for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; m_l++){
					glActiveTexture(GL_TEXTURE0 + m_l);
					glDisable(GL_TEXTURE_2D);
				}

			break;

			case OGL_SHADE_STATE_COLOR_TEXTURING:

				for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; m_l++){
					glActiveTexture(GL_TEXTURE0 + m_l);
					glDisable(GL_TEXTURE_2D);
				}

			break;

			case OGL_SHADE_STATE_VERTEXCOLOR_TEXTURING:

				for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; m_l++){
					glActiveTexture(GL_TEXTURE0 + m_l);
					glDisable(GL_TEXTURE_2D);
				}

			break;

			case OGL_SHADE_STATE_LIGHTING:

				glDisable(GL_LIGHTING);

			break;

			case OGL_SHADE_STATE_VERTEXCOLOR:


			break;

			case OGL_SHADE_STATE_COLOR:


			break;
		}

		if(m_pActiveTriGroup->m_pGlobalReflTexLayer){

			if(m_ActiveState & OGL_RENDER_STATE_USE_TEXTURING){
				glActiveTexture(GL_TEXTURE0 + m_l + 1);
			}else{
				glActiveTexture(GL_TEXTURE0);
			}
            glDisable(GL_TEXTURE_2D);
			glDisable(GL_TEXTURE_GEN_S);
			glDisable(GL_TEXTURE_GEN_T);

		}

	}else{

		if(m_ActiveState & OGL_RENDER_STATE_USE_TEXTURING && (m_pGlobalRenderSettings->m_GlobalRenderState & OGL_RENDER_STATE_USE_TEXTURING)){

			for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; m_l++ ){
				glActiveTexture(GL_TEXTURE0 + m_l);
				glDisable(GL_TEXTURE_2D);
			}

			if(m_pActiveTriGroup->m_pGlobalReflTexLayer){
				glActiveTexture(GL_TEXTURE0 + m_l + 1);
				glDisable(GL_TEXTURE_2D);
				glDisable(GL_TEXTURE_GEN_S);
				glDisable(GL_TEXTURE_GEN_T);
			}

		}

		if(m_ActiveState & OGL_RENDER_STATE_USE_LIGHTING && (m_pGlobalRenderSettings->m_GlobalRenderState & OGL_RENDER_STATE_USE_LIGHTING)){

			glDisable(GL_LIGHTING);

		}

	}

}

void COGLRenderer::RenderShader(){
	m_pActiveTriGroup->m_RenderInfo->m_pCGProgram->SetupAndBind();
	m_pActiveTriGroup->m_RenderInfo->m_pCGProgram->Unbind();
}	

void COGLRenderer::RenderMegaSlow(){

	switch(m_ActiveState){

		case OGL_SHADE_STATE_LIGHTING_TEXTURING:

			glBegin(GL_TRIANGLES);
			for(m_j = 0 ; m_j < (int)m_pActiveTriGroup->m_NumIndices ; ++m_j)
			{

				glNormal3fv((float*)(m_pActiveNormalPtr + m_pActiveIndexArray[m_j]));
				for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; ++m_l){
					glMultiTexCoord2fv(GL_TEXTURE0 + m_l,(float*)(m_pActiveTriGroup->m_TextureLayers[m_l]->m_pUvs.m_DataArray + m_pActiveIndexArray[m_j]));
				}				
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[m_j]]);

				glNormal3fv((float*)&m_pActiveNormalPtr[m_pActiveIndexArray[++m_j]]);
				for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; ++m_l){
					glMultiTexCoord2fv(GL_TEXTURE0 + m_l,(float*)(m_pActiveTriGroup->m_TextureLayers[m_l]->m_pUvs.m_DataArray + m_pActiveIndexArray[m_j]));
				}
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[m_j]]);

				glNormal3fv((float*)&m_pActiveNormalPtr[m_pActiveIndexArray[++m_j]]);
				for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; ++m_l){
					glMultiTexCoord2fv(GL_TEXTURE0 + m_l,(float*)(m_pActiveTriGroup->m_TextureLayers[m_l]->m_pUvs.m_DataArray + m_pActiveIndexArray[m_j]));
				}
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[m_j]]);

			}
			glEnd();

		break;

		case OGL_SHADE_STATE_COLOR_TEXTURING:

			glBegin(GL_TRIANGLES);
			for(m_j = 0 ; m_j < (int)m_pActiveTriGroup->m_NumIndices ; ++m_j)
			{
				for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; ++m_l){
					glMultiTexCoord2fv(GL_TEXTURE0 + m_l,(float*)(m_pActiveTriGroup->m_TextureLayers[m_l]->m_pUvs.m_DataArray + m_pActiveIndexArray[m_j]));
				}		
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[m_j]]);

				for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; ++m_l){
					glMultiTexCoord2fv(GL_TEXTURE0 + m_l,(float*)(m_pActiveTriGroup->m_TextureLayers[m_l]->m_pUvs.m_DataArray + m_pActiveIndexArray[++m_j]));
				}		
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[m_j]]);

				for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; ++m_l){
					glMultiTexCoord2fv(GL_TEXTURE0 + m_l,(float*)(m_pActiveTriGroup->m_TextureLayers[m_l]->m_pUvs.m_DataArray + m_pActiveIndexArray[++m_j]));
				}		
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[m_j]]);
			}
			glEnd();

		break;

		case OGL_SHADE_STATE_VERTEXCOLOR_TEXTURING:

			glBegin(GL_TRIANGLES);
			for(m_j = 0 ; m_j < (int)m_pActiveTriGroup->m_NumIndices ; ++m_j)
			{
				glColor4fv((float*)&m_pActiveColorPtr[m_pActiveIndexArray[m_j]]);
				for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; ++m_l){
					glMultiTexCoord2fv(GL_TEXTURE0 + m_l,(float*)(m_pActiveTriGroup->m_TextureLayers[m_l]->m_pUvs.m_DataArray + m_pActiveIndexArray[m_j]));
				}		
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[m_j]]);

				glColor4fv((float*)&m_pActiveColorPtr[m_pActiveIndexArray[++m_j]]);
				for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; ++m_l){
					glMultiTexCoord2fv(GL_TEXTURE0 + m_l,(float*)(m_pActiveTriGroup->m_TextureLayers[m_l]->m_pUvs.m_DataArray + m_pActiveIndexArray[m_j]));
				}		
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[m_j]]);

				glColor4fv((float*)&m_pActiveColorPtr[m_pActiveIndexArray[++m_j]]);
				for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; ++m_l){
					glMultiTexCoord2fv(GL_TEXTURE0 + m_l,(float*)(m_pActiveTriGroup->m_TextureLayers[m_l]->m_pUvs.m_DataArray + m_pActiveIndexArray[m_j]));
				}		
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[m_j]]);
			}
			glEnd();

		break;

		case OGL_SHADE_STATE_LIGHTING:

			glBegin(GL_TRIANGLES);
			for(m_j = 0 ; m_j < (int)m_pActiveTriGroup->m_NumIndices ; ++m_j)
			{
				glNormal3fv((float*)(m_pActiveNormalPtr + m_pActiveIndexArray[m_j]));
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[m_j]]);

				glNormal3fv((float*)&m_pActiveNormalPtr[m_pActiveIndexArray[++m_j]]);
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[m_j]]);

				glNormal3fv((float*)&m_pActiveNormalPtr[m_pActiveIndexArray[++m_j]]);
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[m_j]]);
			}
			glEnd();
		
		break;

		case OGL_SHADE_STATE_VERTEXCOLOR:

			glBegin(GL_TRIANGLES);
			for(m_j = 0 ; m_j < (int)m_pActiveTriGroup->m_NumIndices ; ++m_j)
			{
				glColor4fv((float*)&m_pActiveColorPtr[m_pActiveIndexArray[m_j]]);
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[m_j]]);

				glColor4fv((float*)&m_pActiveColorPtr[m_pActiveIndexArray[++m_j]]);
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[m_j]]);

				glColor4fv((float*)&m_pActiveColorPtr[m_pActiveIndexArray[++m_j]]);
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[m_j]]);
			}
			glEnd();

		break;

		case OGL_SHADE_STATE_COLOR:

			glBegin(GL_TRIANGLES);
			for(m_j = 0 ; m_j < (int)m_pActiveTriGroup->m_NumIndices ; ++m_j)
			{
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[m_j]]);
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[++m_j]]);
				glVertex3fv((float*)&m_pActiveCoordPtr[m_pActiveIndexArray[++m_j]]);
			}
			glEnd();

		break;
	}
}	

void COGLRenderer::RenderVertexArrays(){

    //ToDo: Lock the vertex array! (or lock it even sooner)
	glEnableClientState(GL_VERTEX_ARRAY);
	glVertexPointer(3, GL_FLOAT, 0, m_pActiveCoordPtr);

	switch(m_ActiveState){

		case OGL_SHADE_STATE_LIGHTING_TEXTURING:

			glDisableClientState(GL_COLOR_ARRAY);

			//Lighting
			glEnableClientState(GL_NORMAL_ARRAY);
			glNormalPointer(GL_FLOAT, 0, m_pActiveTriGroup->m_pNormalArray.m_DataArray);
			
			//Texturing
			for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; m_l++ ){
				glClientActiveTexture(GL_TEXTURE0 + m_l);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);
				glTexCoordPointer(2, GL_FLOAT, 0, m_pActiveTriGroup->m_TextureLayers[m_l]->m_pUvs.m_DataArray);
			}

		break;

		case OGL_SHADE_STATE_COLOR_TEXTURING:

			glDisableClientState(GL_COLOR_ARRAY);
			glDisableClientState(GL_NORMAL_ARRAY);

			//Texturing
			for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; m_l++ ){
				glClientActiveTexture(GL_TEXTURE0 + m_l);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);
				glTexCoordPointer(2, GL_FLOAT, 0, m_pActiveTriGroup->m_TextureLayers[m_l]->m_pUvs.m_DataArray);
			}

		break;

		case OGL_SHADE_STATE_VERTEXCOLOR_TEXTURING:
			
			glDisableClientState(GL_NORMAL_ARRAY);
			
			glEnableClientState(GL_COLOR_ARRAY);
			glColorPointer(4, GL_FLOAT, 0, m_pActiveTriGroup->m_pVertexColorArray.m_DataArray);

			//Texturing
			for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; m_l++ ){
				glClientActiveTexture(GL_TEXTURE0 + m_l);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);
				glTexCoordPointer(2, GL_FLOAT, 0, m_pActiveTriGroup->m_TextureLayers[m_l]->m_pUvs.m_DataArray);
			}
		
		break;

		case OGL_SHADE_STATE_LIGHTING:

			glDisableClientState(GL_COLOR_ARRAY);

			//Lighting
			glEnableClientState(GL_NORMAL_ARRAY);
			glNormalPointer(GL_FLOAT, 0, m_pActiveTriGroup->m_pNormalArray.m_DataArray);
			
		break;

		case OGL_SHADE_STATE_VERTEXCOLOR:

			glDisableClientState(GL_NORMAL_ARRAY);

			glEnableClientState(GL_COLOR_ARRAY);
			glColorPointer(4, GL_FLOAT, 0, m_pActiveTriGroup->m_pVertexColorArray.m_DataArray);
			
		break;

		case OGL_SHADE_STATE_COLOR:

			glDisableClientState(GL_NORMAL_ARRAY);
			glDisableClientState(GL_COLOR_ARRAY);

		break;
	}

	glDrawElements(GL_TRIANGLES, m_pActiveTriGroup->m_NumIndices, GL_UNSIGNED_INT, m_pActiveTriGroup->m_pIndexArray.m_DataArray);

	if(m_ActiveState & OGL_RENDER_STATE_USE_TEXTURING){

		for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; m_l++){
			glClientActiveTexture(GL_TEXTURE0 + m_l);
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		}
	}

}

void COGLRenderer::RenderVertexBufferObjects(){

	glEnableClientState(GL_VERTEX_ARRAY);
	glBindBufferARB(GL_ARRAY_BUFFER_ARB,m_pActiveTriGroup->m_pCoordArray.m_ID);
	glVertexPointer(3,GL_FLOAT,0,NULL);

	switch(m_ActiveState){

		case OGL_SHADE_STATE_LIGHTING_TEXTURING:

			glEnableClientState(GL_NORMAL_ARRAY);
			glBindBufferARB(GL_ARRAY_BUFFER_ARB,m_pActiveTriGroup->m_pNormalArray.m_ID);
			glNormalPointer(GL_FLOAT,0,NULL);

			glDisableClientState(GL_COLOR_ARRAY);

			for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; m_l++ ){
				glClientActiveTexture(GL_TEXTURE0 + m_l);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);
				glBindBufferARB(GL_ARRAY_BUFFER_ARB,m_pActiveTriGroup->m_TextureLayers[m_l]->m_pUvs.m_ID);
				glTexCoordPointer(2, GL_FLOAT, 0, NULL);
			}

		break;

		case OGL_SHADE_STATE_COLOR_TEXTURING:

			glDisableClientState(GL_NORMAL_ARRAY);
			glDisableClientState(GL_COLOR_ARRAY);

			for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; m_l++ ){
				glClientActiveTexture(GL_TEXTURE0 + m_l);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);
				glBindBufferARB(GL_ARRAY_BUFFER_ARB,m_pActiveTriGroup->m_TextureLayers[m_l]->m_pUvs.m_ID);
				glTexCoordPointer(2, GL_FLOAT, 0, NULL);
			}

		break;

		case OGL_SHADE_STATE_VERTEXCOLOR_TEXTURING:

			glDisableClientState(GL_NORMAL_ARRAY);

			glEnableClientState(GL_COLOR_ARRAY);
			glBindBufferARB(GL_ARRAY_BUFFER_ARB,m_pActiveTriGroup->m_pVertexColorArray.m_ID);
			glColorPointer(4,GL_FLOAT,0,NULL);

			for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; m_l++ ){
				glClientActiveTexture(GL_TEXTURE0 + m_l);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);
				glBindBufferARB(GL_ARRAY_BUFFER_ARB,m_pActiveTriGroup->m_TextureLayers[m_l]->m_pUvs.m_ID);
				glTexCoordPointer(2, GL_FLOAT, 0, NULL);
			}

		break;

		case OGL_SHADE_STATE_LIGHTING:

			glEnableClientState(GL_NORMAL_ARRAY);
			glBindBufferARB(GL_ARRAY_BUFFER_ARB,m_pActiveTriGroup->m_pNormalArray.m_ID);
			glNormalPointer(GL_FLOAT,0,NULL);

			glDisableClientState(GL_COLOR_ARRAY);

		break;

		case OGL_SHADE_STATE_VERTEXCOLOR:

			glEnableClientState(GL_COLOR_ARRAY);
			glBindBufferARB(GL_ARRAY_BUFFER_ARB,m_pActiveTriGroup->m_pVertexColorArray.m_ID);
			glColorPointer(4,GL_FLOAT,0,NULL);

			glDisableClientState(GL_NORMAL_ARRAY);

		break;

		case OGL_SHADE_STATE_COLOR:

			glDisableClientState(GL_NORMAL_ARRAY);
			glDisableClientState(GL_COLOR_ARRAY);

		break;

	}

	glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB,m_pActiveTriGroup->m_pIndexArray.m_ID);
	glDrawElements(GL_TRIANGLES,m_pActiveTriGroup->m_NumIndices,GL_UNSIGNED_INT,NULL);

	glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB,0);
	glBindBufferARB(GL_ARRAY_BUFFER_ARB,0);

	if(m_ActiveState & OGL_RENDER_STATE_USE_TEXTURING){

		for(m_l = 0 ; m_l < m_ActiveNmbOfTexLayers ; m_l++){
			glClientActiveTexture(GL_TEXTURE0 + m_l);
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		}
	}

}


void COGLRenderer::StartRendering()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);    
}

void COGLRenderer::EndRendering()
{
	//DEBUG
	if(true){
		ReportErrors();
	}
	SwapBuffers(m_hDC);
}

void COGLRenderer::ReportErrors(){
	m_eGlError = glGetError();
	while(m_eGlError != GL_NO_ERROR)
	{
		LOG(TL_ERROR, "OpenGL-Error: \"%s\"", (const char *)gluErrorString(m_eGlError));
		m_eGlError = glGetError();
	}
}

void COGLRenderer::SetBgColor(dword dwColor)
{
	m_pGlobalRenderSettings->m_bgColor = CVector4f(
		((float)((dwColor & 0xFF000000) >> 24)) / 256.0f,
		((float)((dwColor & 0x00FF0000) >> 16)) / 256.0f,
		((float)((dwColor & 0x0000FF00) >>  8)) / 256.0f,
		((float)((dwColor & 0x000000FF)      )) / 256.0f
	);
	
	glClearColor( 
		m_pGlobalRenderSettings->m_bgColor.m_X,
		m_pGlobalRenderSettings->m_bgColor.m_Y,
		m_pGlobalRenderSettings->m_bgColor.m_Z,
		m_pGlobalRenderSettings->m_bgColor.m_W
	);
}

void COGLRenderer::SetMatrixMode(EMatrixMode mode)
{
	if(m_MatrixMode != mode)
	{
		if(mode == EMM_MODELVIEW)
		{
			glMatrixMode(GL_MODELVIEW);
		}
		else
		{
			glMatrixMode(GL_PROJECTION);
		}
		m_MatrixMode = mode;
	}
}

void COGLRenderer::SetMatrix(const CMatrix44 &mat)
{
	glLoadMatrixf(&mat._11);
}

void COGLRenderer::MultMatrix(const CMatrix44 &mat)
{

	glMultMatrixf(&mat._11);
}

void COGLRenderer::SaveMatrix()
{
	glPushMatrix();
}

void COGLRenderer::RestoreMatrix()
{
	glPopMatrix();
}

void COGLRenderer::EnableZTest(float fEnable)
{
	if(fEnable)
	{
		glEnable(GL_DEPTH_TEST);
	}
	else
	{
		glDisable(GL_DEPTH_TEST);
	}
}

void COGLRenderer::EnableZWrite(float fEnable)
{
	if(fEnable)
	{
		glDepthMask(GL_TRUE);
	}
	else
	{
		glDepthMask(GL_FALSE);
	}
}

void COGLRenderer::EnableAlphaTest(float fEnable)
{
	if(fEnable && m_bBlendEnableFlag)
	{
		glEnable(GL_ALPHA_TEST);
	}
	else
	{
		glDisable(GL_ALPHA_TEST);
	}
}

void COGLRenderer::EnableBlending(float fEnable)
{
	if(fEnable && m_bBlendEnableFlag)
	{
		glEnable(GL_BLEND);
	}
	else
	{
		glDisable(GL_BLEND);
	}
}

void COGLRenderer::SetLightDir(const CVector3f &vDir)
{
	m_vLightDir.Set(vDir.m_X, vDir.m_Y, vDir.m_Z, 0.0f);
	glLightfv(GL_LIGHT0,GL_POSITION, &m_vLightDir.m_X);
}

const CVector4f &COGLRenderer::GetLightDir() const
{
	return m_vLightDir;
}

void COGLRenderer::SetLightColor(byte lightNmb,const CVector4f & ambientColor,const CVector4f &diffuseColor){

	assert( (lightNmb <= 8) && "GL_LIGHT8 or higher was called in COGLRenderer::SetLightColor()");

	m_vLightAmbientColor = ambientColor;
	glLightfv(GL_LIGHT0 + lightNmb,GL_AMBIENT,&ambientColor.m_X);
	m_vLightDiffuseColor = diffuseColor;
	glLightfv(GL_LIGHT0 + lightNmb,GL_DIFFUSE,&diffuseColor.m_X);

}

CVector4f const & COGLRenderer::getAmbientLightColor(byte lightNmb){
		return m_vLightAmbientColor;
}

void COGLRenderer::Free(COGLModelGeometry *obj){
	
	for(m_i = 0 ; m_i < (int)obj->m_TriangleGroups.size() ; ++m_i){
		
        if(glIsList(obj->getTriangleGroup(m_i)->m_RenderInfo->m_dispListNmb)){
			glDeleteLists(obj->getTriangleGroup(m_i)->m_RenderInfo->m_dispListNmb,1);
		}
	}
}

void COGLRenderer::Free(COGLStockGeometry *obj){
	if(glIsList(obj->m_dispListNmb)){
		glDeleteLists(obj->m_dispListNmb,1);
	}
}


void COGLRenderer::Destroy()
{
	m_FontObjects.clear();

    if(m_hGLRC)
    {
        ::wglMakeCurrent(NULL, NULL);
        ::wglDeleteContext(m_hGLRC);
    }
    if(m_hDC && (bool) m_pWindow)
    {
        ::ReleaseDC(m_pWindow->GetWindowHandle(), m_hDC);
    }
    if(m_pWindow)
    {
        m_pWindow->UnregisterHandler(this);
    }

	SAFE_DELETE_ARRAY(m_pPattern);

    Clear();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void COGLRenderer::Clear()
{
	m_bVertexBufferSupported   = false;
	m_bTextureCubeMapSupported = false;
	m_eGlError = GL_NO_ERROR;
	m_pWindow = NULL;
    m_hDC = NULL;
    m_hGLRC = NULL;
	m_pPattern = NULL;
	m_pGlobalRenderSettings = NULL;
	
}

void COGLRenderer::OnWndSize(word wWidth, word wHeight)
{
    glViewport(0, 0, wWidth, wHeight);
}

//debug
void toggleLight(){}

