/****************************************************************************\
*                                                                            *
* 12.04.2006                                                                 *
* CGameCtrl - Source File                                                    *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <algorithm>
#include "CGameCtrl.h"
#include <CEngineFactory.h>
#include "States/CMainMenuState.h"
#include "States/CNewLevelState.h"
#include "States/CBuildState.h"
#include "States/CBattleState.h"
#include "States/CPostLevelState.h"
#include "States/CPostGameState.h"
#include <Renderer/COGLGlobalRenderSettings.h>
#include <Renderer/COGLMultiPass.h>
#include "CSettings.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
struct SRemoveDeletedObj
{
	bool operator()(const TSmartPointer<IGameObj> &pObj)
	{
		return pObj->IsDestroyed();
	}
};

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CGameCtrl::Create(const TSmartPointer<CClock> &pClock)
{
    m_pRenderer = CEngineFactory::InstancePtr()->GetRenderer();
	m_pGlobalRenderSettings = m_pRenderer->GetGlobalSettings();
   
	m_pModelFactory = CEngineFactory::InstancePtr()->GetModelFactory();
    m_pInputMgr = CEngineFactory::InstancePtr()->GetInputMgr();

    m_pGuiMgr = CEngineFactory::InstancePtr()->GetGuiMgr();
	m_pGuiMgr->EnableInfoRender(m_bGRenderGuiInfo = true);

    m_pClock = pClock;

    m_pCamera = new CCamera;

    //Debug:
    m_pCamera->SetPerspective(MY_PI/3.0f, 4.0f/3.0f, 0.01f, 5000.0f);
    m_pCamera->SetPos(CVector3f(500.0f, 10.0f, 500.0f));
    m_pCamera->SetRot(CVector3f(-MY_PI/2.0f, 0.0f, 0.0f));
    //End debug

	//Read the player position
	std::string sValue;
	if(!CSettings::InstancePtr()->GetValue("GAME", "StartPos", sValue))
	{
		LOG(TL_ERROR, "Error: Couldn't get \"StartPos\"-Value from \"GAME\" in CGameCtrl::Create");
		return false;
	}

	if(sscanf(sValue.c_str(), "%f;%f;%f", &m_vPlayerStartPos.m_X, &m_vPlayerStartPos.m_Y, &m_vPlayerStartPos.m_Z) != 3)
	{
		LOG(TL_ERROR, "Error: Couldn't parse \"StartPos\"-Value from \"GAME\"in CGameCtrl::Create: %s", sValue.c_str());
		return false;
	}

	//MultiPass
	m_pMultiPass = new COGLMultiPass();
	m_pMultiPass->Create(m_pCamera,5);

	m_wLevel = 0;

    m_pMainMenuState  = new CMainMenuState;
    m_pNewLevelState  = new CNewLevelState;
    m_pBuildState     = new CBuildState;
    m_pBattleState    = new CBattleState;
	m_pPostLevelState = new CPostLevelState;
	m_pPostGameState = new CPostGameState;

    if(!m_pMainMenuState->Create(this))
    {
        LOG(TL_ERROR, "Error: Couldn't create MainMenu-State");
        return false;
    }
    if(!m_pNewLevelState->Create(this))
    {
        LOG(TL_ERROR, "Error: Couldn't create NewLevel-State");
        return false;
    }
    if(!m_pBuildState->Create(this))
    {
        LOG(TL_ERROR, "Error: Couldn't create Build-State");
        return false;
    }
    if(!m_pBattleState->Create(this))
    {
        LOG(TL_ERROR, "Error: Couldn't create Battle-State");
        return false;
    }
	if(!m_pPostLevelState->Create(this))
	{
		LOG(TL_ERROR, "Error: Couldn't create PostLevel-State");
		return false;
	}
	if(!m_pPostGameState->Create(this))
	{
		LOG(TL_ERROR, "Error: Couldn't create PostGame-State");
		return false;
	}
    //ToDo: Create other game states

	//Setting physics collision flags:
	//Player and PlayerProjectiles shouldn't collide
	CEngineFactory::InstancePtr()->GetPhysicsMgr()->SetGroupCollisionTesting(ECG_PLAYER, ECG_PLAYER_PROJECTILE, false);
	//Player and EnemyProjectiles shouldn't collide
	CEngineFactory::InstancePtr()->GetPhysicsMgr()->SetGroupCollisionTesting(ECG_PLAYER, ECG_ENEMY_PROJECTILE, false);
	//Player and Enemies shouldn't collide
	CEngineFactory::InstancePtr()->GetPhysicsMgr()->SetGroupCollisionTesting(ECG_PLAYER, ECG_ENEMY, false);
	//Enemies shouldn't collide with each other
	CEngineFactory::InstancePtr()->GetPhysicsMgr()->SetGroupCollisionTesting(ECG_ENEMY, ECG_ENEMY, false);
	//Enemies and EnemyProjectiles shouldn't collide
	CEngineFactory::InstancePtr()->GetPhysicsMgr()->SetGroupCollisionTesting(ECG_ENEMY, ECG_ENEMY_PROJECTILE, false);
	//Tower and TowerProjectiles shouldn't collide
	CEngineFactory::InstancePtr()->GetPhysicsMgr()->SetGroupCollisionTesting(ECG_TOWER, ECG_TOWER_PROJECTILE, false);

    SetCurrentState(m_pMainMenuState);

    //Debug:
    SetPriority(100);
    m_pInputMgr->RegisterHandler(this);
    //End debug

    return true;
}

bool CGameCtrl::Update()
{
	//Update the input manager
	m_pInputMgr->Update();
	
	//Update the game state
	if(!m_pCurrState || !m_pCurrState->Update())
		return false;

	//Update the gui manager
	m_pGuiMgr->Update();
	
	//Update the cam control
	if(m_pCamCtrl)
		m_pCamCtrl->Update();

	//Begin Rendering
	m_pRenderer->StartRendering();

	//Render the scene graph
	if(m_pSceneGraph)
		m_pSceneGraph->Render(m_bUseFrustumCulling);

	//Render the gui manager
	m_pGuiMgr->Render();

	m_pRenderer->EndRendering();

	return true;
}

void CGameCtrl::UpdateGameObjects(float dt)
{

	GameObjectList::iterator it;
	//Add objects that should be added
	if(m_ObjectsToAdd.size())
	{
		for(it = m_ObjectsToAdd.begin(); it != m_ObjectsToAdd.end(); ++it)
		{
			m_GameObjects.push_back(*it);
		}
		m_ObjectsToAdd.clear();
	}
	for(it = m_GameObjects.begin(); it != m_GameObjects.end(); ++it)
	{
		(*it)->Update(dt);
	}
	//Remove all objects that were destroyed:
	m_GameObjects.erase(std::remove_if(m_GameObjects.begin(), m_GameObjects.end(), SRemoveDeletedObj()), m_GameObjects.end());
}

void CGameCtrl::ResetGame()
{
	m_wLevel = 0;
	m_fScore = 0.0f;
    m_GameObjects.clear();
    m_ObjectsToAdd.clear();
    SetTerrain(NULL);
    SetEnvironment(NULL);
    SetCamCtrl(NULL);
    SetSceneGraph(NULL);
}

void CGameCtrl::LevelDone()
{
	GameObjectList::iterator it;
	for(it = m_GameObjects.begin(); it != m_GameObjects.end(); ++it)
	{
		(*it)->LevelDone();
	}
	//Remove all objects that were destroyed:
	m_GameObjects.erase(std::remove_if(m_GameObjects.begin(), m_GameObjects.end(), SRemoveDeletedObj()), m_GameObjects.end());
	//Noch zu addende Objekte aufrumen (mssen Projektile oder hnliches sein)
	m_ObjectsToAdd.clear();
}

void CGameCtrl::Destroy()
{
	m_pCamCtrl = NULL;

	m_GameObjects.clear();
	m_ObjectsToAdd.clear();

	SAFE_DELETE(m_pMultiPass);

    SAFE_DELETE(m_pBattleState);
    SAFE_DELETE(m_pBuildState);
    SAFE_DELETE(m_pNewLevelState);
    SAFE_DELETE(m_pMainMenuState);
	SAFE_DELETE(m_pPostLevelState);
	SAFE_DELETE(m_pPostGameState);

    Clear();
}

void CGameCtrl::SetCurrentState(IGameState *pState)
{
    if(m_pCurrState)
        m_pCurrState->Leave();
    m_pCurrState = pState;
    if(m_pCurrState)
        m_pCurrState->Enter();
}

IGameState *CGameCtrl::GetBuildState() const
{
	return m_pBuildState;
}

IGameState *CGameCtrl::GetBattleState() const
{
	return m_pBattleState;
}

void CGameCtrl::SetSceneGraph(const TSmartPointer<ISceneGraph> &pSceneGraph)
{
    m_pSceneGraph = pSceneGraph;
	if(m_pSceneGraph)
	{
		m_pSceneGraph->SetCamera(m_pCamera);
	}
}

void CGameCtrl::SetCamCtrl(const TSmartPointer<ICamCtrl> &pCamCtrl)
{
    if(m_pCamCtrl)
    {
        m_pCamCtrl->Disable();
    }
    m_pCamCtrl = pCamCtrl;
    if(m_pCamCtrl)
    {
        m_pCamCtrl->Enable(m_pCamera);
    }
}

const CVector3f CGameCtrl::GetCameraRay() const
{
    float fX, fY;
    CEngineFactory::InstancePtr()->GetInputMgr()->GetNormalizedCursorPos(fX, fY);
    return m_pCamera->GetRay(fX, fY);
}

void CGameCtrl::SetTerrain(const TSmartPointer<CTerrain> &pTerrain)
{
    m_pTerrain = pTerrain;
//    CEngineFactory::InstancePtr()->GetCollisionMgr()->SetTerrain(m_pTerrain);
}

void CGameCtrl::SetEnvironment(const TSmartPointer<CEnvironment> &pEnvironment)
{
	m_pEnvironment = pEnvironment;
    if(m_pSceneGraph)
    {
        if(m_pEnvironment)
        {
    	    m_pSceneGraph->SetEnvironment(m_pEnvironment->getEnvironmentRep());
			m_pSceneGraph->SetSunDir(m_pEnvironment->GetSunDir());
			m_pRenderer->SetLightColor(0,m_pEnvironment->GetSunAmbientColor(),m_pEnvironment->GetSunDiffuseColor());
        }
        else
        {
            m_pSceneGraph->SetEnvironment(NULL);
        }
    }
}

void CGameCtrl::AddObject(const TSmartPointer<IGameObj> &pObj)
{
    m_ObjectsToAdd.push_back(pObj);
}

void CGameCtrl::AddMoney(float fAmount)
{
	AddScore(fAmount);
	m_pBuildState->AddMoney(fAmount);
}

void CGameCtrl::RemoveMoney(float fAmount)
{
	m_pBuildState->RemoveMoney(fAmount);
}

void CGameCtrl::AddScore(float fAmount)
{
	m_fScore += fAmount;
	m_pBattleState->ScoreChanged();
}

/*
void CGameCtrl::RemoveObject(const TSmartPointer<IGameObj> &pObj)
{
    GameObjectList::iterator it(find(m_GameObjects.begin(), m_GameObjects.end(), pObj));
    if(it != m_GameObjects.end())
    {
        m_GameObjects.erase(it);
    }
}
*/

CBuildingObj *CGameCtrl::GetRandomBuilding()
{
	//Ok, lets see how many buldings we have:
	dword dwNumBuildings = 0;
	GameObjectList::iterator it(m_GameObjects.begin());
	while(it != m_GameObjects.end())
	{
		if((*it)->GetType() == EOT_BUILDING)
		{
			++dwNumBuildings;
		}
		++it;
	}
	if(dwNumBuildings == 0)
		return NULL;

	dword dwBuilding = rand() % dwNumBuildings;
	for(it = m_GameObjects.begin(); it != m_GameObjects.end(); ++it)
	{
		if((*it)->GetType() == EOT_BUILDING)
		{
			if(dwBuilding == 0)
			{
				return (CBuildingObj *)((IGameObj *)(*it));
			}
			--dwBuilding;
		}
	}

	LOG(TL_ERROR, "Error: Something went wrong in CGameCtrl::GetRandomBuilding");
	return NULL;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CGameCtrl::Clear()
{
	m_wLevel = 0;
	m_fScore = 0.0f;
	m_bGRenderGuiInfo     = true;
	m_bGRenderWireframe   = false;
	m_bGRenderMipMapping  = true;
	m_eGTexQuality        = OGL_TQ_UNKNOWN;
	m_eGRenderMode        = OGL_RENDER_MODE_UNKNOWN;
	m_eGCacheMode         = OGL_CACHE_MODE_UNKNOWN;
	m_bGUseVFC            = true;
	m_bGRenderTransparent = true;
	m_bRenderMultiPass    = false;
	m_bUseFrustumCulling  = true;
	m_bEnableTransparency = true;

    m_pCamera = NULL;
    m_pTerrain = NULL;
	m_pEnvironment = NULL;

    m_pSceneGraph = NULL;
    m_pCurrState = NULL;

    m_pMainMenuState = NULL;
    m_pNewLevelState = NULL;
    m_pBuildState = NULL;
    m_pBattleState = NULL;
	m_pPostLevelState = NULL;
	m_pPostGameState = NULL;

    m_pClock = NULL;

    m_pGuiMgr = NULL;
    m_pInputMgr = NULL;
    m_pModelFactory = NULL;
    m_pRenderer = NULL;
	m_pGlobalRenderSettings = NULL;

	m_pMultiPass = NULL;
}

bool CGameCtrl::OnKeyDown(dword dwKey)
{
	switch(dwKey){
		case 0x3c://F2
			m_pGuiMgr->EnableInfoRender(m_bGRenderGuiInfo = !m_bGRenderGuiInfo);
			if(m_bGRenderGuiInfo)
			{
				LOG(TL_STDOUT, "OnScreen-Info: Enabled");
			}
			else
			{
				LOG(TL_STDOUT, "OnScreen-Info: Disabled");
			}
			break;
		case 0x3d://F3
			((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetWireFrame(m_bGRenderWireframe = !m_bGRenderWireframe,false);
			if(m_bGRenderWireframe)
			{
				LOG(TL_STDOUT, "WireFrame: Enabled");
			}
			else
			{
				LOG(TL_STDOUT, "WireFrame: Disabled");
			}
			break;
		case 0x3e://F4
			switch(m_eGTexQuality){
				case OGL_TQ_LINEAR:
					((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetTextureQualityOverride(m_eGTexQuality = OGL_TQ_BILINEAR,true);
					LOG(TL_STDOUT, "Texture quality: Bilinear");
					break;
				case OGL_TQ_BILINEAR:
					((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetTextureQualityOverride(m_eGTexQuality = OGL_TQ_TRILINEAR,true);
					LOG(TL_STDOUT, "Texture quality: Triliniear");
					break;
				case OGL_TQ_TRILINEAR:
					((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetTextureQualityOverride(m_eGTexQuality = OGL_TQ_UNKNOWN,false);
					LOG(TL_STDOUT, "Texture quality: Mixed");
					break;
				case OGL_TQ_UNKNOWN:
					((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetTextureQualityOverride(m_eGTexQuality = OGL_TQ_LINEAR,true);
					LOG(TL_STDOUT, "Texture quality: Linear");
					break;
			}
			break;
		case 0x3f://F5
			((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetMipMappingOverride(m_bGRenderMipMapping = !m_bGRenderMipMapping);
			if(m_bGRenderMipMapping)
			{
				LOG(TL_STDOUT, "MipMapping: Enabled");
			}
			else
			{
				LOG(TL_STDOUT, "MipMapping: Disabled");
			}
			break;
		case 0x40://F6
			switch(m_eGRenderMode){
				case OGL_RENDER_MODE_MEGASLOW:
					((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetRenderModeOverride(m_eGRenderMode = OGL_RENDER_MODE_VERTEXARRAY,true);
					LOG(TL_STDOUT, "Render mode: Vertex arrays");
					break;
				case OGL_RENDER_MODE_VERTEXARRAY:
					if( ((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetRenderModeOverride(m_eGRenderMode = OGL_RENDER_MODE_VERTEXBUFFEROBJECT,true) ){
						LOG(TL_STDOUT, "Render mode: Vertex buffer objects");
						break;
					}else{
						LOG(TL_STDOUT, "Render mode: Vertex buffer objects not compatible with current CacheModeSettings(VBO)");
					}
				case OGL_RENDER_MODE_VERTEXBUFFEROBJECT:
					((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetRenderModeOverride(m_eGRenderMode = OGL_RENDER_MODE_UNKNOWN,false);
					LOG(TL_STDOUT, "Render mode: Mixed");
					break;
				case OGL_RENDER_MODE_UNKNOWN:
					((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetRenderModeOverride(m_eGRenderMode = OGL_RENDER_MODE_MEGASLOW,true);
					LOG(TL_STDOUT, "Render mode: Mega slow (immediate mode)");
					break;
			}
			break;
		case 0x41://F7
			switch(m_eGCacheMode){
				case OGL_CACHE_MODE_EXECUTE:
					((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetCacheModeOverride(m_eGCacheMode = OGL_CACHE_MODE_DISABLED,true);
					LOG(TL_STDOUT, "Display lists: Disabled");
					break;
				case OGL_CACHE_MODE_DISABLED:
					if(((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetCacheModeOverride(m_eGCacheMode = OGL_CACHE_MODE_UNKNOWN,false))
					{
						LOG(TL_STDOUT, "Display lists: Mixed");
					}
					else{
						LOG(TL_STDOUT, "Display lists: DisplayLists not compatible with curren RenderModeSettings(VBO)");
						((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetCacheModeOverride(m_eGCacheMode = OGL_CACHE_MODE_DISABLED,true);
						LOG(TL_STDOUT, "Display lists: Disabled");
					}
					break;
				case OGL_CACHE_MODE_UNKNOWN:
					if( ((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetCacheModeOverride(m_eGCacheMode = OGL_CACHE_MODE_EXECUTE,true) ){
						LOG(TL_STDOUT, "Display lists: Enabled");
					}else{
						LOG(TL_STDOUT, "Display lists: DisplayLists not compatible with curren RenderModeSettings(VBO)");
						((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetCacheModeOverride(m_eGCacheMode = OGL_CACHE_MODE_DISABLED,true);
						LOG(TL_STDOUT, "Display lists: Disabled");
					}
					break;
			}
			break;
		case 0x42://F8
			m_bUseFrustumCulling = !m_bUseFrustumCulling;
			if(m_bUseFrustumCulling)
			{
				LOG(TL_STDOUT, "Frustum culling: Enabled");
			}
			else
			{
				LOG(TL_STDOUT, "Frustum culling: Disabled");
			}
			break;
		case 0x43://F9
			((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->EnableBlendingEffects(m_bEnableTransparency = !m_bEnableTransparency);
			if(m_bEnableTransparency)
			{
				LOG(TL_STDOUT, "Transparency: Enabled");
			}
			else
			{
				LOG(TL_STDOUT, "Transparency: Disabled");
			}
			break;
		case 0x44://F10
			//LOG(TL_DEBUG,"%s","F10 - ToggleMultiPass");
			//m_bRenderMultiPass = !m_bRenderMultiPass;
			break;
		//Debug:
		case cg_dwKeyG:
			m_pBattleState->OnBirdDestroyed();
			break;
		//End debug
	}
	return false;
}




