/****************************************************************************\
*                                                                            *
* 07.05.2006                                                                 *
* CBuildingObj - Source File                                                 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CBuildingObj.h"
#include "CBuildingRep.h"
#include <CEngineFactory.h>
#include "Objects/Birds/CBirdObj.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
const float cg_fBuildingMaxHeightDiff = 1.0f;

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CBuildingObj::CBuildingObj(CGameCtrl *pGameCtrl, const std::string &sModel, float fMaxHitpoints, float fMaxValue, ECollisionGroup cg)
	: ICollidableObj(pGameCtrl, EOT_BUILDING), m_fMaxHitpoints(fMaxHitpoints), m_fMaxValue(fMaxValue), m_CollisionGroup(cg), m_State(EBS_INVALIDPOS), m_bReadyForDestroy(false)
{
    CBuildingRep *pRep = new CBuildingRep();
    if(!pRep->Create(pGameCtrl->GetSceneGraph(), sModel, this))
    {
        LOG(TL_ERROR, "Error: pRep->Create failed in CBuildingObj::CBuildingObj");
    }

	SetHitpoints(fMaxHitpoints);

    m_pRepresentation = (IRepresentation *) pRep;
    m_pRepresentation->SetPickable(false);
}

void CBuildingObj::SetHitpoints(float fHitpoints)
{
	m_fHitpoints = fHitpoints;
	if(m_fHitpoints <= 0.0f)
	{
		m_State = EBS_DESTROYED;
		m_bReadyForDestroy = true;
	}
}

void CBuildingObj::CollidedWith(IGameObj *pOther)
{
	switch(pOther->GetType())
	{
		case EOT_BIRDPROJECTILE:
			SetHitpoints(GetHitpoints() - 10.0f);
			break;
		case EOT_BIRD:
			if(((CBirdObj *)pOther)->GetHitpoints() > 0.0f)
			{
				//This was a kamikaze!
				SetHitpoints(GetHitpoints() - 30.0f);
			}
			break;
	}
}

void CBuildingObj::LevelDone()
{
	if(m_State == EBS_VALIDPOS)
	{
		//Add money depending on the condition of the building
		m_pGameCtrl->AddMoney(m_fMaxValue * m_fHitpoints / m_fMaxHitpoints);
	}
	else
	{
		Destroy();
	}
}

bool CBuildingObj::Build()
{
	if(m_State == EBS_VALIDPOS)
	{
		//Clear the forest at this position
		m_pGameCtrl->GetTerrain()->ClearForest(this);
		//Make us pickable
//		m_pRepresentation->SetPickable(true);
		//Create the building actor
		CEngineFactory::InstancePtr()->GetPhysicsMgr()->BeginActorCreation(GetTransformation(), m_CollisionGroup, false);
		CAABox3f aabox(m_pRepresentation->GetOriginalBoundingBox());
		CMatrix44 mat;
		mat.Translation(aabox.Center());
		CEngineFactory::InstancePtr()->GetPhysicsMgr()->AddCube(mat, aabox.Size());
		SetActor(CEngineFactory::InstancePtr()->GetPhysicsMgr()->FinishActorCreation());
		return true;
	}
	return false;
}

void CBuildingObj::UpdateState()
{
	if(CEngineFactory::InstancePtr()->GetPhysicsMgr()->CheckOverlapping(m_pRepresentation->GetOriginalBoundingBox() + GetPos(),
		0xFFFFFFFF & ~((1 << ECG_TERRAIN) | (1 << ECG_VEGETATION))))	//Ignore terrain and trees
	{
		m_State = EBS_INVALIDPOS;
	}
	else
	{
		//Make sure that the terrain is flat enough
		CBox3f box(GetBoundingBox());
		CVector3f vCenter(box.Center());
		//Get the corners and check terrain height there
		float fMinHeight = -1000.0f;
		float fMaxHeight = -1000.0f;
		AdjustHeight(box.m_vPoint, vCenter, fMinHeight, fMaxHeight);
		AdjustHeight(box.m_vPoint + box.m_vA, vCenter, fMinHeight, fMaxHeight);
		AdjustHeight(box.m_vPoint + box.m_vB, vCenter, fMinHeight, fMaxHeight);
		AdjustHeight(box.m_vPoint + box.m_vC, vCenter, fMinHeight, fMaxHeight);
		AdjustHeight(box.m_vPoint + box.m_vA + box.m_vB, vCenter, fMinHeight, fMaxHeight);
		AdjustHeight(box.m_vPoint + box.m_vA + box.m_vC, vCenter, fMinHeight, fMaxHeight);
		AdjustHeight(box.m_vPoint + box.m_vB + box.m_vC, vCenter, fMinHeight, fMaxHeight);
		AdjustHeight(box.m_vPoint + box.m_vA + box.m_vB + box.m_vC, vCenter, fMinHeight, fMaxHeight);
		CVector3f vPos(GetPos());
		if(fMinHeight != -1000.0f && vPos.m_Y > fMinHeight)
		{
			vPos.m_Y = fMinHeight;
			SetPos(vPos);
		}
		if(fMaxHeight - fMinHeight > cg_fBuildingMaxHeightDiff)
		{
			m_State = EBS_INVALIDPOS;
		}
		else
		{
			m_State = EBS_VALIDPOS;
		}
	}
}

CBuildingObj::~CBuildingObj()
{

}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/
void CBuildingObj::SubUpdate(float dt)
{
	if(m_bReadyForDestroy)
	{
		NotifyAllDeleteObservers();
		SetActor(NULL);
	}
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CBuildingObj::AdjustHeight(const CVector3f &vCorner, const CVector3f &vCenter, float &fMinHeight, float &fMaxHeight)
{
	if(vCorner.m_Y < vCenter.m_Y)
	{
		float fHeight = m_pGameCtrl->GetTerrain()->HeightAt(vCorner.m_X, vCorner.m_Z);
		if(fMinHeight == -1000.0f || fHeight < fMinHeight)
			fMinHeight = fHeight;
		if(fMaxHeight == -1000.0f || fHeight > fMaxHeight)
			fMaxHeight = fHeight;
	}
}